@@ -31,6 +31,7 @@ extern WARN_UNUSED bool stress_cpu_x86_has_clflushopt(void);
 extern WARN_UNUSED bool stress_cpu_x86_has_clwb(void);
 extern WARN_UNUSED bool stress_cpu_x86_has_cldemote(void);
 extern WARN_UNUSED bool stress_cpu_x86_has_clfsh(void);
+extern WARN_UNUSED bool stress_cpu_x86_has_lahf_lm(void);
 extern WARN_UNUSED bool stress_cpu_x86_has_mmx(void);
 extern WARN_UNUSED bool stress_cpu_x86_has_msr(void);
 extern WARN_UNUSED bool stress_cpu_x86_has_prefetchwt1(void);
@@ -43,6 +44,7 @@ extern WARN_UNUSED bool stress_cpu_x86_has_sse2(void);
 extern WARN_UNUSED bool stress_cpu_x86_has_syscall(void);
 extern WARN_UNUSED bool stress_cpu_x86_has_tsc(void);
 extern WARN_UNUSED bool stress_cpu_x86_has_waitpkg(void);
+extern WARN_UNUSED bool stress_cpu_x86_has_movdiri(void);
 
 extern void stress_cpu_disable_fp_subnormals(void);
 extern void stress_cpu_enable_fp_subnormals(void);
