@@ -76,9 +76,10 @@ static void stress_klog_kernel_cmdline(void)
 	if (already_dumped)
 		return;
 
-	ret = stress_system_read("/proc/cmdline", buffer, sizeof(buffer));
+	ret = stress_system_read("/proc/cmdline", buffer, sizeof(buffer) - 1);
 	if (ret < 0)
 		return;
+	buffer[ret] = '\0';
 
 	for (ptr = buffer; *ptr && (ptr < (buffer + ret)); ptr++) {
 		if (*ptr == '\n') {
@@ -194,11 +195,6 @@ void stress_klog_start(void)
 				dump_procs = true;
 				goto log_err;
 			}
-			if (strstr(buf, "watchdog") && strstr(buf, "hard LOCKUP")) {
-				msg = "hard lockup";
-				dump_procs = true;
-				goto log_err;
-			}
 			if (strstr(buf, "Out of memory")) {
 				msg = "out of memory";
 				goto log_info;
