@@ -41,7 +41,7 @@ const struct option stress_long_options[] = {
 	{ "aio",		1,	0,	OPT_aio },
 	{ "aio-ops",		1,	0,	OPT_aio_ops },
 	{ "aio-requests",	1,	0,	OPT_aio_requests },
-	{ "aiol",		1,	0,	OPT_aiol},
+	{ "aiol",		1,	0,	OPT_aiol },
 	{ "aiol-ops",		1,	0,	OPT_aiol_ops },
 	{ "aiol-requests",	1,	0,	OPT_aiol_requests },
 	{ "alarm",		1,	0,	OPT_alarm },
@@ -51,6 +51,7 @@ const struct option stress_long_options[] = {
 	{ "apparmor-ops",	1,	0,	OPT_apparmor_ops },
 	{ "atomic",		1,	0,	OPT_atomic },
 	{ "atomic-ops",		1,	0,	OPT_atomic_ops },
+	{ "autogroup",		0,	0,	OPT_autogroup },
 	{ "bad-altstack",	1,	0,	OPT_bad_altstack },
 	{ "bad-altstack-ops",	1,	0,	OPT_bad_altstack_ops },
 	{ "bad-ioctl",		1,	0,	OPT_bad_ioctl },
@@ -93,7 +94,7 @@ const struct option stress_long_options[] = {
 	{ "buildinfo",		0,	0,	OPT_buildinfo },
 	{ "c-states",		0,	0,	OPT_c_states },
 	{ "cache",		1,	0, 	OPT_cache },
-	{ "cache-size",		1,	0, 	OPT_cache_size},
+	{ "cache-size",		1,	0, 	OPT_cache_size },
 	{ "cache-cldemote",	0,	0,	OPT_cache_cldemote },
 	{ "cache-clflushopt",	0,	0,	OPT_cache_clflushopt },
 	{ "cache-clwb",		0,	0,	OPT_cache_clwb },
@@ -127,9 +128,9 @@ const struct option stress_long_options[] = {
 	{ "chdir-ops",		1,	0, 	OPT_chdir_ops },
 	{ "chmod",		1,	0, 	OPT_chmod },
 	{ "chmod-ops",		1,	0,	OPT_chmod_ops },
-	{ "chown",		1,	0, 	OPT_chown},
+	{ "chown",		1,	0, 	OPT_chown },
 	{ "chown-ops",		1,	0,	OPT_chown_ops },
-	{ "chroot",		1,	0, 	OPT_chroot},
+	{ "chroot",		1,	0, 	OPT_chroot },
 	{ "chroot-ops",		1,	0,	OPT_chroot_ops },
 	{ "chyperbolic",	1,	0,	OPT_chyperbolic },
 	{ "chyperbolic-method",	1,	0,	OPT_chyperbolic_method },
@@ -263,7 +264,7 @@ const struct option stress_long_options[] = {
 	{ "far-branch-pages",	1,	0,	OPT_far_branch_pages },
 	{ "fault",		1,	0,	OPT_fault },
 	{ "fault-ops",		1,	0,	OPT_fault_ops },
-	{ "fcntl",		1,	0,	OPT_fcntl},
+	{ "fcntl",		1,	0,	OPT_fcntl },
 	{ "fcntl-ops",		1,	0,	OPT_fcntl_ops },
 	{ "fd-abuse",		1,	0,	OPT_fd_abuse },
 	{ "fd-abuse-ops",	1,	0,	OPT_fd_abuse_ops },
@@ -318,7 +319,7 @@ const struct option stress_long_options[] = {
 	{ "fp",			1,	0,	OPT_fp },
 	{ "fp-method",		1,	0,	OPT_fp_method },
 	{ "fp-ops",		1,	0,	OPT_fp_ops },
-	{ "fp-error",		1,	0,	OPT_fp_error},
+	{ "fp-error",		1,	0,	OPT_fp_error },
 	{ "fp-error-ops",	1,	0,	OPT_fp_error_ops },
 	{ "fpunch",		1,	0,	OPT_fpunch },
 	{ "fpunch-bytes",	1,	0,	OPT_fpunch_bytes },
@@ -392,7 +393,7 @@ const struct option stress_long_options[] = {
 	{ "icache-ops",		1,	0,	OPT_icache_ops },
 	{ "icmp-flood",		1,	0,	OPT_icmp_flood },
 	{ "icmp-flood-ops",	1,	0,	OPT_icmp_flood_ops },
-	{ "icmp-flood-max-size", 0,	0,	OPT_icmp_flood_max_size },
+	{ "icmp-flood-max-size", 1,	0,	OPT_icmp_flood_max_size },
 	{ "idle-page",		1,	0,	OPT_idle_page },
 	{ "idle-page-ops",	1,	0,	OPT_idle_page_ops },
 	{ "ignite-cpu",		0,	0, 	OPT_ignite_cpu },
@@ -464,9 +465,9 @@ const struct option stress_long_options[] = {
 	{ "l1cache-method",	1,	0,	OPT_l1cache_method },
 	{ "l1cache-mlock",	0,	0,	OPT_l1cache_mlock },
 	{ "l1cache-ops",	1,	0,	OPT_l1cache_ops },
-	{ "l1cache-sets",	1,	0,	OPT_l1cache_sets},
+	{ "l1cache-sets",	1,	0,	OPT_l1cache_sets },
 	{ "l1cache-size",	1,	0,	OPT_l1cache_size },
-	{ "l1cache-ways",	1,	0,	OPT_l1cache_ways},
+	{ "l1cache-ways",	1,	0,	OPT_l1cache_ways },
 	{ "landlock",		1,	0,	OPT_landlock },
 	{ "landlock-ops",	1,	0,	OPT_landlock_ops },
 	{ "led",		1,	0,	OPT_led },
@@ -587,7 +588,7 @@ const struct option stress_long_options[] = {
 	{ "metrics-brief",	0,	0,	OPT_metrics_brief },
 	{ "mincore",		1,	0,	OPT_mincore },
 	{ "mincore-ops",	1,	0,	OPT_mincore_ops },
-	{ "mincore-random",	0,	0,	OPT_mincore_rand },
+	{ "mincore-random",	0,	0,	OPT_mincore_random },
 	{ "min-nanosleep",	1,	0,	OPT_min_nanosleep },
 	{ "min-nanosleep-ops",	1,	0,	OPT_min_nanosleep_ops },
 	{ "min-nanosleep-max",	1,	0,	OPT_min_nanosleep_max },
@@ -633,13 +634,13 @@ const struct option stress_long_options[] = {
 	{ "mmapfiles-ops",	1,	0,	OPT_mmapfiles_ops },
 	{ "mmapfiles-populate",	0,	0,	OPT_mmapfiles_populate },
 	{ "mmapfiles-shared",	0,	0,	OPT_mmapfiles_shared },
-	{ "mmapfixed",		1,	0,	OPT_mmapfixed},
+	{ "mmapfixed",		1,	0,	OPT_mmapfixed },
 	{ "mmapfixed-mlock",	0,	0,	OPT_mmapfixed_mlock },
 	{ "mmapfixed-numa",	0,	0,	OPT_mmapfixed_numa },
 	{ "mmapfixed-ops",	1,	0,	OPT_mmapfixed_ops },
 	{ "mmapfork",		1,	0,	OPT_mmapfork },
 	{ "mmapfork-ops",	1,	0,	OPT_mmapfork_ops },
-	{ "mmapfork-bytes",	1,	0,	OPT_mmapfork_bytes},
+	{ "mmapfork-bytes",	1,	0,	OPT_mmapfork_bytes },
 	{ "mmaphuge",		1,	0,	OPT_mmaphuge },
 	{ "mmaphuge-file",	0,	0,	OPT_mmaphuge_file },
 	{ "mmaphuge-mlock",	0,	0,	OPT_mmaphuge_mlock },
@@ -659,12 +660,12 @@ const struct option stress_long_options[] = {
 	{ "mmaptorture-bytes",	1,	0,	OPT_mmaptorture_bytes },
 	{ "mmaptorture-msync",	1,	0,	OPT_mmaptorture_msync },
 	{ "mmaptorture-ops",	1,	0,	OPT_mmaptorture_ops },
-	{ "module",		1,	0,	OPT_module},
+	{ "module",		1,	0,	OPT_module },
 	{ "module-ops",		1,	0,	OPT_module_ops },
-	{ "module-name",	1,	0,	OPT_module_name},
-	{ "module-no-modver",	0,	0,	OPT_module_no_modver},
-	{ "module-no-vermag",	0,	0,	OPT_module_no_vermag,},
-	{ "module-no-unload",	0,	0,	OPT_module_no_unload},
+	{ "module-name",	1,	0,	OPT_module_name },
+	{ "module-no-modver",	0,	0,	OPT_module_no_modver },
+	{ "module-no-vermag",	0,	0,	OPT_module_no_vermag },
+	{ "module-no-unload",	0,	0,	OPT_module_no_unload },
 	{ "monte-carlo",	1,	0,	OPT_monte_carlo },
 	{ "monte-carlo-method",	1,	0,	OPT_monte_carlo_method },
 	{ "monte-carlo-ops",	1,	0,	OPT_monte_carlo_ops },
@@ -684,7 +685,7 @@ const struct option stress_long_options[] = {
 	{ "mremap-numa",	0,	0,	OPT_mremap_numa },
 	{ "mremap-ops",		1,	0,	OPT_mremap_ops },
 	{ "mseal",		1,	0,	OPT_mseal },
-	{ "mseal-ops",		1,	0,	OPT_mseal_ops,},
+	{ "mseal-ops",		1,	0,	OPT_mseal_ops },
 	{ "msg",		1,	0,	OPT_msg },
 	{ "msg-bytes",		1,	0,	OPT_msg_bytes },
 	{ "msg-ops",		1,	0,	OPT_msg_ops },
@@ -730,6 +731,7 @@ const struct option stress_long_options[] = {
 	{ "numa-shuffle-addr",	0,	0,	OPT_numa_shuffle_addr },
 	{ "numa-shuffle-node",	0,	0,	OPT_numa_shuffle_node },
 	{ "oomable",		0,	0,	OPT_oomable },
+	{ "oom-no-child",	0,	0,	OPT_oom_no_child },
 	{ "oom-avoid",		0,	0,	OPT_oom_avoid },
 	{ "oom-avoid-bytes",	1,	0,	OPT_oom_avoid_bytes },
 	{ "oom-pipe",		1,	0,	OPT_oom_pipe },
@@ -752,8 +754,8 @@ const struct option stress_long_options[] = {
 	{ "parallel",		1,	0,	OPT_all },
 	{ "pathological",	0,	0,	OPT_pathological },
 	{ "pause",		1,	0,	OPT_pause },
-	{ "pci",		1,	0,	OPT_pci},
-	{ "pci-dev",		1,	0,	OPT_pci_dev},
+	{ "pci",		1,	0,	OPT_pci },
+	{ "pci-dev",		1,	0,	OPT_pci_dev },
 	{ "pci-ops",		1,	0,	OPT_pci_ops },
 	{ "pci-ops-rate",	1,	0,	OPT_pci_ops_rate },
 #if defined(STRESS_PERF_STATS) && 	\
@@ -856,6 +858,7 @@ const struct option stress_long_options[] = {
 	{ "randlist-items", 	1,	0,	OPT_randlist_items },
 	{ "randlist-ops",	1,	0,	OPT_randlist_ops },
 	{ "randlist-size", 	1,	0,	OPT_randlist_size },
+	{ "randprocname",	0,	0,	OPT_randprocname },
 	{ "random",		1,	0,	OPT_random },
 	{ "rapl",		0,	0,	OPT_rapl },
 	{ "raplstat",		1,	0,	OPT_raplstat },
@@ -893,6 +896,7 @@ const struct option stress_long_options[] = {
 	{ "rename-ops",		1,	0,	OPT_rename_ops },
 	{ "resched",		1,	0,	OPT_resched },
 	{ "resched-ops",	1,	0,	OPT_resched_ops },
+	{ "resctrl",		1,	0,	OPT_resctrl },
 	{ "resources",		1,	0,	OPT_resources },
 	{ "resources-mlock",	0,	0,	OPT_resources_mlock },
 	{ "resources-ops",	1,	0,	OPT_resources_ops },
@@ -1008,7 +1012,7 @@ const struct option stress_long_options[] = {
 	{ "sigsuspend",		1,	0,	OPT_sigsuspend },
 	{ "sigsuspend-ops",	1,	0,	OPT_sigsuspend_ops },
 	{ "sigtrap",		1,	0,	OPT_sigtrap },
-	{ "sigtrap-ops",	1,	0,	OPT_sigtrap_ops},
+	{ "sigtrap-ops",	1,	0,	OPT_sigtrap_ops },
 	{ "sigurg",		1,	0,	OPT_sigurg },
 	{ "sigurg-ops",		1,	0,	OPT_sigurg_ops },
 	{ "sigvtalrm",		1,	0,	OPT_sigvtalrm },
@@ -1056,7 +1060,7 @@ const struct option stress_long_options[] = {
 	{ "sockpair-ops",	1,	0,	OPT_sockpair_ops },
 	{ "softlockup",		1,	0,	OPT_softlockup },
 	{ "softlockup-ops",	1,	0,	OPT_softlockup_ops },
-	{ "sparsematrix",	1,	0,	OPT_sparsematrix},
+	{ "sparsematrix",	1,	0,	OPT_sparsematrix },
 	{ "sparsematrix-items",	1,	0,	OPT_sparsematrix_items },
 	{ "sparsematrix-method",1,	0,	OPT_sparsematrix_method },
 	{ "sparsematrix-ops",	1,	0,	OPT_sparsematrix_ops },
@@ -1072,7 +1076,7 @@ const struct option stress_long_options[] = {
 	{ "splice",		1,	0,	OPT_splice },
 	{ "splice-bytes",	1,	0,	OPT_splice_bytes },
 	{ "splice-ops",		1,	0,	OPT_splice_ops },
-	{ "stack",		1,	0,	OPT_stack},
+	{ "stack",		1,	0,	OPT_stack },
 	{ "stack-fill",		0,	0,	OPT_stack_fill },
 	{ "stack-mlock",	0,	0,	OPT_stack_mlock },
 	{ "stack-ops",		1,	0,	OPT_stack_ops },
@@ -1145,6 +1149,8 @@ const struct option stress_long_options[] = {
 	{ "timerfd-freq",	1,	0,	OPT_timerfd_freq },
 	{ "timerfd-ops",	1,	0,	OPT_timerfd_ops },
 	{ "timerfd-rand",	0,	0,	OPT_timerfd_rand },
+	{ "timermix",		1,	0,	OPT_timermix },
+	{ "timermix-ops",	1,	0,	OPT_timermix_ops },
 	{ "timer-slack"	,	1,	0,	OPT_timer_slack },
 	{ "time-warp",		1,	0,	OPT_time_warp },
 	{ "time-warp-ops",	1,	0,	OPT_time_warp_ops },
@@ -1177,7 +1183,7 @@ const struct option stress_long_options[] = {
 	{ "times",		0,	0,	OPT_times },
 	{ "timestamp",		0,	0,	OPT_timestamp },
 	{ "tz",			0,	0,	OPT_thermal_zones },
-	{ "tun",		1,	0,	OPT_tun},
+	{ "tun",		1,	0,	OPT_tun },
 	{ "tun-tap",		0,	0,	OPT_tun_tap },
 	{ "tun-ops",		1,	0,	OPT_tun_ops },
 	{ "udp",		1,	0,	OPT_udp },
@@ -1214,7 +1220,7 @@ const struct option stress_long_options[] = {
 	{ "vdso",		1,	0,	OPT_vdso },
 	{ "vdso-func",		1,	0,	OPT_vdso_func },
 	{ "vdso-ops",		1,	0,	OPT_vdso_ops },
-	{ "veccmp",		1,	0,	OPT_veccmp},
+	{ "veccmp",		1,	0,	OPT_veccmp },
 	{ "veccmp-ops",		1,	0,	OPT_veccmp_ops },
 	{ "vecfp",		1,	0,	OPT_vecfp },
 	{ "vecfp-method",	1,	0,	OPT_vecfp_method },
@@ -1224,7 +1230,7 @@ const struct option stress_long_options[] = {
 	{ "vecshuf",		1,	0,	OPT_vecshuf },
 	{ "vecshuf-method",	1,	0,	OPT_vecshuf_method },
 	{ "vecshuf-ops",	1,	0,	OPT_vecshuf_ops },
-	{ "vecwide",		1,	0,	OPT_vecwide},
+	{ "vecwide",		1,	0,	OPT_vecwide },
 	{ "vecwide-ops",	1,	0,	OPT_vecwide_ops },
 	{ "verbose",		0,	0,	OPT_verbose },
 	{ "verify",		0,	0,	OPT_verify },
@@ -1282,7 +1288,7 @@ const struct option stress_long_options[] = {
 	{ "watchdog",		1,	0,	OPT_watchdog },
 	{ "watchdog-ops",	1,	0,	OPT_watchdog_ops },
 	{ "with",		1,	0,	OPT_with },
-	{ "wcs",		1,	0,	OPT_wcs},
+	{ "wcs",		1,	0,	OPT_wcs },
 	{ "wcs-method",		1,	0,	OPT_wcs_method },
 	{ "wcs-ops",		1,	0,	OPT_wcs_ops },
 	{ "workload",		1,	0,	OPT_workload },
@@ -1314,8 +1320,8 @@ const struct option stress_long_options[] = {
 	{ "zlib-method",	1,	0,	OPT_zlib_method },
 	{ "zlib-mem-level",	1,	0,	OPT_zlib_mem_level },
 	{ "zlib-ops",		1,	0,	OPT_zlib_ops },
-	{ "zlib-strategy",	1,	0,	OPT_zlib_strategy, },
-	{ "zlib-stream-bytes",	1,	0,	OPT_zlib_stream_bytes, },
+	{ "zlib-strategy",	1,	0,	OPT_zlib_strategy },
+	{ "zlib-stream-bytes",	1,	0,	OPT_zlib_stream_bytes },
 	{ "zlib-window-bits",	1,	0,	OPT_zlib_window_bits },
 	{ "zombie",		1,	0,	OPT_zombie },
 	{ "zombie-max",		1,	0,	OPT_zombie_max },
