@@ -19,6 +19,7 @@
  */
 #include "stress-ng.h"
 #include "core-attribute.h"
+#include "core-mmap.h"
 #include "core-nt-load.h"
 #include "core-out-of-memory.h"
 
@@ -113,7 +114,7 @@ static inline void OPTIMIZE3 stress_brk_page_resident(
 #endif
 }
 
-static inline size_t PURE stress_brk_abs(const uint8_t *ptr1, const uint8_t *ptr2)
+static inline size_t CONST stress_brk_abs(const uint8_t *ptr1, const uint8_t *ptr2)
 {
 	return (size_t)((ptr1 > ptr2) ? ptr1 - ptr2 : ptr2 - ptr1);
 }
@@ -268,7 +269,7 @@ static int OPTIMIZE3 stress_brk_child(stress_args_t *args, void *context)
 			i = 0;
 			/* remove a page from brk region */
 			if (unmap_ptr) {
-				(void)stress_munmap_retry_enomem((void *)unmap_ptr, page_size);
+				(void)stress_munmap_force((void *)unmap_ptr, page_size);
 				unmap_ptr = NULL;
 			}
 			continue;
@@ -304,8 +305,9 @@ static int stress_brk(stress_args_t *args)
 						PROT_READ | PROT_WRITE,
 						MAP_ANONYMOUS | MAP_SHARED, -1, 0);
 	if (brk_context == MAP_FAILED) {
-		pr_inf_skip("%s: cannot mmap brk context region, errno=%d (%s), skipping stressor\n",
-			args->name, errno, strerror(errno));
+		pr_inf_skip("%s: cannot mmap brk context region%s, errno=%d (%s), skipping stressor\n",
+			args->name, stress_get_memfree_str(),
+			errno, strerror(errno));
 		return EXIT_NO_RESOURCE;
 	}
 
@@ -322,7 +324,7 @@ static int stress_brk(stress_args_t *args)
 	(void)stress_get_setting("brk-notouch", &brk_context->brk_notouch);
 
 #if !defined(MCL_FUTURE)
-	if ((args->instance == 0) && brk_context->brk_mlock) {
+	if (stress_instance_zero(args) && brk_context->brk_mlock) {
 		pr_inf("%s: --brk-mlock option was enabled but support for "
 			"mlockall(MCL_FUTURE) is not available\n",
 			args->name);
@@ -338,7 +340,7 @@ static int stress_brk(stress_args_t *args)
 
 	pr_dbg("%s: %" PRIu64 " occurrences of sbrk out of memory\n",
 		args->name, brk_context->out_of_memory);
-	pr_dbg("%s: %" PRIu64 " successful sbrk expands, %" PRIu64 " succussful sbrk shinks\n",
+	pr_dbg("%s: %" PRIu64 " successful sbrk expands, %" PRIu64 " successful sbrk shinks\n",
 		args->name, brk_context->sbrk_expands, brk_context->sbrk_shrinks);
 
 	rate = (brk_context->sbrk_exp_count > 0.0) ? (double)brk_context->sbrk_exp_duration / brk_context->sbrk_exp_count : 0.0;
@@ -348,7 +350,6 @@ static int stress_brk(stress_args_t *args)
 	stress_metrics_set(args, 1, "nanosecs per sbrk page shrink",
 		rate * STRESS_DBL_NANOSECOND, STRESS_METRIC_HARMONIC_MEAN);
 
-
 	(void)munmap((void *)brk_context, sizeof(*brk_context));
 
 	return rc;
@@ -357,7 +358,7 @@ static int stress_brk(stress_args_t *args)
 const stressor_info_t stress_brk_info = {
 	.stressor = stress_brk,
 	.supported = stress_brk_supported,
-	.class = CLASS_OS | CLASS_VM,
+	.classifier = CLASS_OS | CLASS_VM,
 	.opts = opts,
 	.verify = VERIFY_ALWAYS,
 	.help = help
