@@ -68,13 +68,13 @@ static int do_fchown(
 {
 	int tmp, ret;
 
-	if (stress_chown_check(fchown(fd, uid, gid) < 0))
+	if (stress_chown_check(fchown(fd, uid, gid)) < 0)
 		return -errno;
-	if (stress_chown_check(fchown(fd, (uid_t)-1, gid) < 0))
+	if (stress_chown_check(fchown(fd, (uid_t)-1, gid)) < 0)
 		return -errno;
-	if (stress_chown_check(fchown(fd, uid, (gid_t)-1) < 0))
+	if (stress_chown_check(fchown(fd, uid, (gid_t)-1)) < 0)
 		return -errno;
-	if (stress_chown_check(fchown(fd, (uid_t)-1, (gid_t)-1) < 0))
+	if (stress_chown_check(fchown(fd, (uid_t)-1, (gid_t)-1)) < 0)
 		return -errno;
 
 	if (cap_chown)
@@ -171,6 +171,7 @@ static int stress_chown(stress_args_t *args)
 	const uid_t uid = getuid();
 	const gid_t gid = getgid();
 	bool cap_chown = false;
+	int fsync_counter = 0;
 
 	if (geteuid() == 0)
 		cap_chown = true;
@@ -179,7 +180,7 @@ static int stress_chown(stress_args_t *args)
 	 *  Allow for multiple workers to chown the *same* file
 	 */
 	stress_temp_dir(pathname, sizeof(pathname), args->name, ppid, 0);
-	if (mkdir(pathname, S_IRUSR | S_IRWXU) < 0) {
+	if (mkdir(pathname, S_IRWXU) < 0) {
 		if (errno != EEXIST) {
 			rc = stress_exit_status(errno);
 			pr_fail("%s: mkdir %s failed, errno=%d (%s)\n",
@@ -200,7 +201,7 @@ static int stress_chown(stress_args_t *args)
 	} else {
 		/* Other instances must try to open the file */
 		for (;;) {
-			if ((fd = open(filename, O_RDWR)) > - 1)
+			if ((fd = open(filename, O_RDWR)) > -1)
 				break;
 
 #if defined(__NetBSD__)
@@ -265,7 +266,10 @@ static int stress_chown(stress_args_t *args)
 			rc = EXIT_FAILURE;
 			break;
 		}
-		(void)shim_fsync(fd);
+		if (fsync_counter++ >= 128) {
+			fsync_counter = 0;
+			(void)shim_fsync(fd);
+		}
 		stress_bogo_inc(args);
 	} while (stress_continue(args));
 
