@@ -74,9 +74,6 @@ static const stress_clock_info_t clocks[] = {
 #if defined(CLOCK_THREAD_CPUTIME_ID)
 	CLOCK_INFO(CLOCK_THREAD_CPUTIME_ID),
 #endif
-#if defined(CLOCK_MONOTONIC_ACTIVE)
-	CLOCK_INFO(CLOCK_MONOTONIC_ACTIVE),
-#endif
 #if defined(CLOCK_TAI)
 	CLOCK_INFO(CLOCK_TAI)
 #endif
@@ -278,7 +275,7 @@ static int stress_clock(stress_args_t *args)
 				if (UNLIKELY((ret < 0) && (g_opt_flags & OPT_FLAGS_VERIFY) &&
 					     (errno != EINVAL) &&
 					     (errno != ENOSYS))) {
-					pr_fail("%s: clock_getres failed for timer '%s', errno=%d (%s)\n",
+					pr_fail("%s: clock_gettime failed for timer '%s', errno=%d (%s)\n",
 						args->name, clocks[i].name, errno, strerror(errno));
 					rc = EXIT_FAILURE;
 				}
@@ -288,12 +285,14 @@ static int stress_clock(stress_args_t *args)
 				/* Ensuring clock_settime cannot succeed without privilege */
 				if (!is_root) {
 					ret = shim_clock_settime(clocks[i].id, &t1);
-					if (UNLIKELY(ret != -EPERM)) {
-						/* This is an error, report it! */
-						pr_fail("%s: clock_settime failed, did not have privilege to "
-							"set time, expected -EPERM, instead got errno=%d (%s)\n",
-							args->name, errno, strerror(errno));
-						rc = EXIT_FAILURE;
+					if (ret < 0) {
+						if (UNLIKELY(((errno != EPERM) && (errno != EINVAL)))) {
+							/* This is an error, report it! */
+							pr_fail("%s: clock_settime failed, did not have privilege to "
+								"set time, expected EPERM or EINVAL, instead got errno=%d (%s)\n",
+								args->name, errno, strerror(errno));
+							rc = EXIT_FAILURE;
+						}
 					}
 				}
 
