@@ -192,7 +192,8 @@ static void *stress_close_func(void *arg)
 		 */
 		flag = close_range_flags[stress_mwc8modn(SIZEOF_ARRAY(close_range_flags))];
 		ret = shim_close_range(FDS_START, FDS_START + FDS_TO_DUP, flag);
-		if ((ret < 0) || (errno == ENOSYS)) {
+		if (ret < 0) {
+			/* failed for some reason, redo close to ensure all fds are closed */
 			for (i = 0; i < FDS_TO_DUP; i++)
 				(void)close(fds[i]);
 		}
@@ -260,16 +261,19 @@ static int stress_close(stress_args_t *args)
 		}
 	}
 
-#if defined(HAVE_FACCESSAT)
+#if defined(HAVE_FACCESSAT) || 1
 	{
 		ret = stress_temp_dir_mk_args(args);
-		if (ret < 0)
-			return stress_exit_status(-ret);
+		if (ret < 0) {
+			rc = stress_exit_status(-ret);
+			goto tidy;
+		}
 		(void)stress_temp_filename_args(args, filename, sizeof(filename), stress_mwc32());
 		file_fd = open(filename, O_CREAT | O_RDWR, S_IRUSR | S_IWUSR);
 		if (file_fd < 0) {
 			pr_err("%s: cannot create %s\n", args->name, filename);
-			return stress_exit_status(errno);
+			rc = stress_exit_status(errno);
+			goto tidy;
 		}
 		(void)shim_unlink(filename);
 	}
@@ -470,8 +474,8 @@ static int stress_close(stress_args_t *args)
 		t2 = stress_time_now();
 		dt = t2 - t1;
 		if (dt > max_duration) {
-			max_duration = dt ;
-			/* max delay is 75% of the duration in microseconds */
+			max_duration = dt;
+			/* max delay is 75% of the max duration in microseconds */
 			max_delay_us = (uint64_t)(duration * 750000);
 		}
 
@@ -512,14 +516,14 @@ tidy:
 
 const stressor_info_t stress_close_info = {
 	.stressor = stress_close,
-	.class = CLASS_OS,
+	.classifier = CLASS_OS,
 	.verify = VERIFY_ALWAYS,
 	.help = help
 };
 #else
 const stressor_info_t stress_close_info = {
 	.stressor = stress_unimplemented,
-	.class = CLASS_OS,
+	.classifier = CLASS_OS,
 	.verify = VERIFY_ALWAYS,
 	.help = help,
 	.unimplemented_reason = "built without pthread support"
