@@ -82,8 +82,9 @@ static int stress_copy_file_fill(
 
 	while (sz > 0) {
 		ssize_t n;
+		ssize_t wr_size = sz > (ssize_t)sizeof(buf) ? (ssize_t)sizeof(buf) : sz;
 
-		n = write(fd, buf, sizeof(buf));
+		n = write(fd, buf, wr_size);
 		if (n < 0)
 			return -1;
 		sz -= n;
@@ -128,7 +129,7 @@ static int stress_copy_file_range_verify(
 		bytes_out = read(fd_out, buf_out, sz);
 		if (bytes_out == 0)
 			return 0;
-		if (bytes_out <= 0)
+		if (bytes_out < 0)
 			break;
 
 		n = STRESS_MINIMUM(bytes_in, bytes_out);
@@ -152,22 +153,40 @@ static int stress_copy_file(stress_args_t *args)
 	int fd_in, fd_out, rc = EXIT_FAILURE, ret;
 	const int fd_bad = stress_get_bad_fd();
 	char filename[PATH_MAX - 5], tmp[PATH_MAX];
-	uint64_t copy_file_bytes = DEFAULT_COPY_FILE_BYTES;
+	uint64_t copy_file_bytes, copy_file_bytes_total = DEFAULT_COPY_FILE_BYTES;
 	double duration = 0.0, bytes = 0.0, rate;
 	const bool verify = !!(g_opt_flags & OPT_FLAGS_VERIFY);
 
-	if (!stress_get_setting("copy-file-bytes", &copy_file_bytes)) {
+	if (!stress_get_setting("copy-file-bytes", &copy_file_bytes_total)) {
 		if (g_opt_flags & OPT_FLAGS_MAXIMIZE)
-			copy_file_bytes = MAX_COPY_FILE_BYTES;
+			copy_file_bytes_total = MAX_COPY_FILE_BYTES;
 		if (g_opt_flags & OPT_FLAGS_MINIMIZE)
-			copy_file_bytes = MIN_COPY_FILE_BYTES;
+			copy_file_bytes_total = MIN_COPY_FILE_BYTES;
+	}
+	if (copy_file_bytes_total < MIN_COPY_FILE_BYTES) {
+		copy_file_bytes_total = MIN_COPY_FILE_BYTES;
+		if (stress_instance_zero(args))
+			pr_inf("%s: --copy-file-bytes too small, using %" PRIu64 " instead\n",
+				args->name, copy_file_bytes_total);
+	}
+	if (copy_file_bytes_total > MAX_COPY_FILE_BYTES) {
+		copy_file_bytes_total = MAX_COPY_FILE_BYTES;
+		if (stress_instance_zero(args))
+			pr_inf("%s: --copy-file-bytes too large, using %" PRIu64 " instead\n",
+				args->name, copy_file_bytes_total);
 	}
 
-	copy_file_bytes /= args->instances;
-	if (copy_file_bytes < DEFAULT_COPY_FILE_SIZE)
+	copy_file_bytes = copy_file_bytes_total / args->instances;
+	if (copy_file_bytes < DEFAULT_COPY_FILE_SIZE) {
 		copy_file_bytes = DEFAULT_COPY_FILE_SIZE * 2;
-	if (copy_file_bytes < MIN_COPY_FILE_BYTES)
+		copy_file_bytes_total = copy_file_bytes * args->instances;
+	}
+	if (copy_file_bytes < MIN_COPY_FILE_BYTES) {
 		copy_file_bytes = MIN_COPY_FILE_BYTES;
+		copy_file_bytes_total = copy_file_bytes * args->instances;
+	}
+	if (stress_instance_zero(args))
+		stress_fs_usage_bytes(args, copy_file_bytes, copy_file_bytes_total);
 
         ret = stress_temp_dir_mk_args(args);
         if (ret < 0)
@@ -321,7 +340,7 @@ tidy_dir:
 
 const stressor_info_t stress_copy_file_info = {
 	.stressor = stress_copy_file,
-	.class = CLASS_FILESYSTEM | CLASS_OS,
+	.classifier = CLASS_FILESYSTEM | CLASS_OS,
 	.opts = opts,
 	.verify = VERIFY_OPTIONAL,
 	.help = help
@@ -329,10 +348,10 @@ const stressor_info_t stress_copy_file_info = {
 #else
 const stressor_info_t stress_copy_file_info = {
 	.stressor = stress_unimplemented,
-	.class = CLASS_FILESYSTEM | CLASS_OS,
+	.classifier = CLASS_FILESYSTEM | CLASS_OS,
 	.opts = opts,
 	.verify = VERIFY_OPTIONAL,
 	.help = help,
-	.unimplemented_reason = "built without clone() system call"
+	.unimplemented_reason = "built without copy_file_range() system call"
 };
 #endif
