@@ -1,6 +1,6 @@
 /*
  * Copyright (C) 2016-2021 Canonical, Ltd.
- * Copyright (C) 2022-2024 Colin Ian King.
+ * Copyright (C) 2022-2025 Colin Ian King.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -19,6 +19,7 @@
  */
 #include "stress-ng.h"
 #include "core-builtin.h"
+#include "core-killpid.h"
 
 #include <sched.h>
 
@@ -32,20 +33,10 @@ static const stress_help_t help[] = {
 
 #define STRESS_CPU_ONLINE_MAX_CPUS	(65536)
 
-static int stress_set_cpu_online_affinity(const char *opt)
-{
-	return stress_set_setting_true("cpu-online-affinity", opt);
-}
-
-static int stress_set_cpu_online_all(const char *opt)
-{
-	return stress_set_setting_true("cpu-online-all", opt);
-}
-
-static const stress_opt_set_func_t opt_set_funcs[] = {
-	{ OPT_cpu_online_affinity,	stress_set_cpu_online_affinity },
-	{ OPT_cpu_online_all,		stress_set_cpu_online_all },
-	{ 0,				NULL },
+static const stress_opt_t opts[] = {
+	{ OPT_cpu_online_affinity, "cpu-online-affinity", TYPE_ID_BOOL, 0, 1, NULL },
+	{ OPT_cpu_online_all,      "cpu-online-all",      TYPE_ID_BOOL, 0, 1, NULL },
+	END_OPT,
 };
 
 #if defined(__linux__)
@@ -79,7 +70,7 @@ static int stress_cpu_online_set(
 	const int setting)
 {
 	char filename[PATH_MAX];
-	char data[3] = { '0' + (char)setting, '\n', 0 };
+	const char data[3] = { '0' + (char)setting, '\n', 0 };
 	ssize_t ret;
 
 	(void)snprintf(filename, sizeof(filename),
@@ -170,10 +161,14 @@ static int stress_cpu_online(stress_args_t *args)
 {
 	int32_t cpus = stress_get_processors_configured();
 	int32_t i, cpu_online_count = 0;
+	uint32_t cpu, prev_cpu;
 	bool *cpu_online;
 	bool cpu_online_affinity = false;
 	bool cpu_online_all = false;
+	bool child_affinity = true;
 	int rc = EXIT_SUCCESS;
+	int fds[2] = { -1, -1 };
+	pid_t pid;
 	double offline_duration = 0.0, offline_count = 0.0;
 	double online_duration  = 0.0, online_count = 0.0;
 	double rate;
@@ -182,7 +177,7 @@ static int stress_cpu_online(stress_args_t *args)
 	(void)stress_get_setting("cpu-online-all", &cpu_online_all);
 
 	if (geteuid() != 0) {
-		if (args->instance == 0)
+		if (stress_instance_zero(args))
 			pr_inf("%s: need root privilege to run "
 				"this stressor\n", args->name);
 		/* Not strictly a test failure */
@@ -201,10 +196,11 @@ static int stress_cpu_online(stress_args_t *args)
 		cpus = STRESS_CPU_ONLINE_MAX_CPUS;
 	}
 
-	cpu_online = calloc((size_t)cpus, sizeof(*cpu_online));
+	cpu_online = (bool *)calloc((size_t)cpus, sizeof(*cpu_online));
 	if (!cpu_online) {
-		pr_inf_skip("%s: out of memory allocating %" PRId32 " boolean flags, "
-			    "skipping stressor\n", args->name, cpus);
+		pr_inf_skip("%s: out of memory allocating %" PRId32 " boolean flags%s, "
+			    "skipping stressor\n", args->name, cpus,
+			    stress_get_memfree_str());
 		return EXIT_NO_RESOURCE;
 	}
 
@@ -218,7 +214,7 @@ static int stress_cpu_online(stress_args_t *args)
 
 		(void)snprintf(filename, sizeof(filename),
 			"/sys/devices/system/cpu/cpu%" PRId32 "/online", i);
-		ret = access(filename, O_RDWR);
+		ret = access(filename, R_OK | W_OK);
 		if (ret == 0) {
 			cpu_online[i] = true;
 			cpu_online_count++;
@@ -229,8 +225,8 @@ static int stress_cpu_online(stress_args_t *args)
 		free(cpu_online);
 		return EXIT_FAILURE;
 	}
-	if ((args->num_instances > 1) && cpu_online_all) {
-		if (args->instance == 0) {
+	if ((args->instances > 1) && cpu_online_all) {
+		if (stress_instance_zero(args)) {
 			pr_inf("%s: disabling --cpu-online-all option because "
 			       "more than 1 %s stressor is being invoked\n",
 				args->name, args->name);
@@ -238,18 +234,93 @@ static int stress_cpu_online(stress_args_t *args)
 		cpu_online_all = false;
 	}
 
-	if ((args->instance == 0) && cpu_online_all) {
+	if (stress_instance_zero(args) && cpu_online_all) {
 		pr_inf("%s: exercising all %" PRId32 " cpus\n",
 			args->name, cpu_online_count + 1);
 	}
 
+	stress_set_proc_state(args->name, STRESS_STATE_SYNC_WAIT);
+	stress_sync_start_wait(args);
 	stress_set_proc_state(args->name, STRESS_STATE_RUN);
 
+	/* Use a pipe to send offlined CPU number to child */
+	if (pipe(fds) < 0) {
+		pr_dbg("%s: unable to create pipe, errno=%d (%s), ignoring "
+			"exercising child affinity\n",
+			args->name, errno, strerror(errno));
+		child_affinity = false;
+	}
+	if (child_affinity) {
+		/*
+		 *  Try to fork child that tries to pin itself to the
+		 *  CPU that the parent offlines.
+		 */
+		pid = fork();
+		if (pid < 0) {
+			pr_dbg("%s: unable to fork child, errno=%d (%s), ignoring "
+				"exercising child affinity\n",
+				args->name, errno, strerror(errno));
+			child_affinity = false;
+		} else if (pid == 0) {
+			cpu = 0;
+
+			stress_set_proc_state(args->name, STRESS_STATE_RUN);
+			/*
+			 *  Try to pin child process to that of the
+			 *  offline'd CPU. This runs at 20Hz trying
+			 *  to read the next CPU that the parent is
+			 *  offlining and setting affinity to this.
+			 */
+			do {
+				fd_set rfds;
+				struct timeval timeout;
+				int ret;
+
+				FD_ZERO(&rfds);
+				FD_SET(fds[0], &rfds);
+				timeout.tv_sec = 0;
+				timeout.tv_usec = 50000;
+
+				ret = select(fds[0] + 1, &rfds, NULL, NULL, &timeout);
+				if (ret < 0)
+					break;
+				if ((ret == 1) && (FD_ISSET(fds[0], &rfds))) {
+					/* Read fail, bail out, pipe maybe closed */
+					if (read(fds[0], &cpu, sizeof(cpu)) < 0)
+						break;
+				}
+				/* This may fail if the CPU is offlined */
+				stress_cpu_online_set_affinity(cpu);
+			} while (stress_continue(args));
+			(void)close(fds[0]);
+			(void)close(fds[1]);
+			_exit(0);
+		}
+	}
+
 	/*
 	 *  Now randomly offline/online them all
 	 */
+	cpu = 0;
+	prev_cpu = cpus;
 	do {
-		const uint32_t cpu = stress_mwc32modn((uint32_t)cpus);
+		switch (args->instance) {
+		case 1:
+			cpu++;
+			if (cpu >= (uint32_t)cpus)
+				cpu = 0;
+			break;
+		case 2:
+			if (cpu == 0)
+				cpu = cpus;
+			cpu--;
+			break;
+		default:
+			cpu = stress_mwc32modn((uint32_t)cpus);
+			if (cpu == prev_cpu)
+				continue;
+			break;
+		}
 
 		/*
 		 *  Only allow CPU 0 to be offlined if --cpu-online-all has been enabled
@@ -260,6 +331,22 @@ static int stress_cpu_online(stress_args_t *args)
 			double t;
 			int setting;
 
+			/* Don't try if already offline */
+			stress_cpu_online_get(cpu, &setting);
+			if (setting == 0) {
+				(void)shim_sched_yield();
+				continue;
+			}
+
+			if (child_affinity && (fds[1] != -1)) {
+				if (write(fds[1], &cpu, sizeof(cpu)) < 0) {
+					(void)close(fds[0]);
+					(void)close(fds[1]);
+					fds[0] = -1;
+					fds[1] = -1;
+				}
+			}
+
 			if (cpu_online_affinity)
 				stress_cpu_online_set_affinity(cpu);
 
@@ -269,7 +356,7 @@ static int stress_cpu_online(stress_args_t *args)
 				break;
 			if (rc == EXIT_SUCCESS) {
 				rc = stress_cpu_online_get(cpu, &setting);
-				if ((rc == EXIT_SUCCESS) && (setting != 0)) {
+				if ((rc == EXIT_SUCCESS) && (args->instances == 0) && (setting != 0)) {
 					pr_inf("%s: set cpu %" PRIu32 " offline, expecting setting to be 0, got %d instead\n",
 						args->name, cpu, setting);
 				} else {
@@ -284,20 +371,34 @@ static int stress_cpu_online(stress_args_t *args)
 				break;
 			if (rc == EXIT_SUCCESS) {
 				rc = stress_cpu_online_get(cpu, &setting);
-				if ((rc == EXIT_SUCCESS) && (setting != 1)) {
-					pr_inf("%s: set cpu %" PRIu32 " offline, expecting setting to be 1, got %d instead\n",
+				if ((rc == EXIT_SUCCESS) && (args->instances == 0) && (setting != 1)) {
+					pr_inf("%s: set cpu %" PRIu32 " online, expecting setting to be 1, got %d instead\n",
 						args->name, cpu, setting);
 				} else {
 					online_duration += stress_time_now() - t;
 					online_count += 1.0;
+					if (cpu_online_affinity)
+						stress_cpu_online_set_affinity(cpu);
 				}
 			}
 			stress_bogo_inc(args);
+			(void)shim_sched_yield();
 		}
 	} while (stress_continue(args));
 
 	stress_set_proc_state(args->name, STRESS_STATE_DEINIT);
 
+	/*
+	 *  Child running? close pipe and kill it
+	 */
+	if (child_affinity) {
+		if (fds[0] != -1)
+			(void)close(fds[0]);
+		if (fds[1] != -1)
+			(void)close(fds[1]);
+		(void)stress_kill_and_wait(args, pid, SIGKILL, false);
+	}
+
 	/*
 	 *  Force CPUs all back online
 	 */
@@ -309,27 +410,27 @@ static int stress_cpu_online(stress_args_t *args)
 
 	rate = (offline_count > 0.0) ? (double)offline_duration / offline_count : 0.0;
 	stress_metrics_set(args, 0, "millisecs per offline action",
-		rate * STRESS_DBL_MILLISECOND, STRESS_HARMONIC_MEAN);
+		rate * STRESS_DBL_MILLISECOND, STRESS_METRIC_HARMONIC_MEAN);
 	rate = (online_count > 0.0) ? (double)online_duration / online_count : 0.0;
 	stress_metrics_set(args, 1, "millisecs per online action",
-		rate * STRESS_DBL_MILLISECOND, STRESS_HARMONIC_MEAN);
+		rate * STRESS_DBL_MILLISECOND, STRESS_METRIC_HARMONIC_MEAN);
 
 	return rc;
 }
 
-stressor_info_t stress_cpu_online_info = {
+const stressor_info_t stress_cpu_online_info = {
 	.stressor = stress_cpu_online,
 	.supported = stress_cpu_online_supported,
-	.class = CLASS_CPU | CLASS_OS | CLASS_PATHOLOGICAL,
-	.opt_set_funcs = opt_set_funcs,
+	.classifier = CLASS_CPU | CLASS_OS | CLASS_PATHOLOGICAL,
+	.opts = opts,
 	.verify = VERIFY_ALWAYS,
 	.help = help
 };
 #else
-stressor_info_t stress_cpu_online_info = {
+const stressor_info_t stress_cpu_online_info = {
 	.stressor = stress_unimplemented,
-	.class = CLASS_CPU | CLASS_OS | CLASS_PATHOLOGICAL,
-	.opt_set_funcs = opt_set_funcs,
+	.classifier = CLASS_CPU | CLASS_OS | CLASS_PATHOLOGICAL,
+	.opts = opts,
 	.verify = VERIFY_ALWAYS,
 	.help = help,
 	.unimplemented_reason = "only supported on Linux"
