@@ -18,6 +18,7 @@
  *
  */
 #include "stress-ng.h"
+#include "core-capabilities.h"
 
 #if defined(NSIG)
 #define MAX_SIGNUM      NSIG
@@ -128,6 +129,7 @@ static int stress_make_daemon(
 				goto err_close_fds2;
 			(void)umask(0);
 			VOID_RET(int, stress_drop_capabilities(args->name));
+			stress_set_proc_state(args->name, STRESS_STATE_RUN);
 
 			sz = write(fd, &rc, sizeof(rc));
 			if (sz != sizeof(rc))
@@ -234,7 +236,7 @@ static const stress_opt_t opts[] = {
 
 const stressor_info_t stress_daemon_info = {
 	.stressor = stress_daemon,
-	.class = CLASS_SCHEDULER | CLASS_OS,
+	.classifier = CLASS_SCHEDULER | CLASS_OS,
 	.opts = opts,
 	.verify = VERIFY_ALWAYS,
 	.help = help
