@@ -249,21 +249,21 @@ static double TARGET_CLONES OPTIMIZE3 name(				\
 
 #if defined(HAVE_Decimal32)
 STRESS_DFP_ADD(d32, stress_dfp_d32_add, true)
-STRESS_DFP_ADD(d32, stress_dfp_d32_sub, true)
+STRESS_DFP_SUB(d32, stress_dfp_d32_sub, true)
 STRESS_DFP_MUL(d32, stress_dfp_d32_mul, true)
 STRESS_DFP_DIV(d32, stress_dfp_d32_div, true)
 #endif
 
 #if defined(HAVE_Decimal64)
 STRESS_DFP_ADD(d64, stress_dfp_d64_add, true)
-STRESS_DFP_ADD(d64, stress_dfp_d64_sub, true)
+STRESS_DFP_SUB(d64, stress_dfp_d64_sub, true)
 STRESS_DFP_MUL(d64, stress_dfp_d64_mul, true)
 STRESS_DFP_DIV(d64, stress_dfp_d64_div, true)
 #endif
 
 #if defined(HAVE_Decimal128)
 STRESS_DFP_ADD(d128, stress_dfp_d128_add, true)
-STRESS_DFP_ADD(d128, stress_dfp_d128_sub, true)
+STRESS_DFP_SUB(d128, stress_dfp_d128_sub, true)
 STRESS_DFP_MUL(d128, stress_dfp_d128_mul, true)
 STRESS_DFP_DIV(d128, stress_dfp_d128_div, true)
 #endif
@@ -358,7 +358,7 @@ static int stress_dfp_call_method(
 	metrics->duration += dt;
 	metrics->count += (DFP_ELEMENTS * LOOPS_PER_CALL);
 
-	if ((method > 0) && (method < STRESS_NUM_DFP_FUNCS && verify)) {
+	if ((method > 0) && (method < STRESS_NUM_DFP_FUNCS) && verify) {
 		register size_t i;
 		const int dfp_type = stress_dfp_funcs[method].dfp_type;
 		const char *method_name = stress_dfp_funcs[method].name;
@@ -389,6 +389,20 @@ static int stress_dfp_call_method(
 				r0 = (long double)dfp_data[i].d32.r[0];
 				r1 = (long double)dfp_data[i].d32.r[1];
 				break;
+#endif
+#if defined(HAVE_Decimal64)
+			case STRESS_DFP_TYPE_DECIMAL64:
+				ret = shim_memcmp(&dfp_data[i].d64.r[0], &dfp_data[i].d64.r[1], sizeof(dfp_data[i].d64.r[0]));
+				r0 = (long double)dfp_data[i].d64.r[0];
+				r1 = (long double)dfp_data[i].d64.r[1];
+				break;
+#endif
+#if defined(HAVE_Decimal128)
+			case STRESS_DFP_TYPE_DECIMAL128:
+				ret = shim_memcmp(&dfp_data[i].d128.r[0], &dfp_data[i].d128.r[1], sizeof(dfp_data[i].d128.r[0]));
+				r0 = (long double)dfp_data[i].d128.r[0];
+				r1 = (long double)dfp_data[i].d128.r[1];
+				break;
 #endif
 			default:
 				/* Should never happen! */
@@ -424,7 +438,7 @@ static int stress_dfp(stress_args_t *args)
 {
 	size_t i, mmap_size;
 	dfp_data_t *dfp_data;
-	size_t fp_method = 0;	/* "all" */
+	size_t dfp_method = 0;	/* "all" */
 	const bool verify = !!(g_opt_flags & OPT_FLAGS_VERIFY);
 	int rc = EXIT_SUCCESS;
 
@@ -442,7 +456,7 @@ static int stress_dfp(stress_args_t *args)
 	stress_set_vma_anon_name(dfp_data, mmap_size, "dfp-data");
 	(void)stress_madvise_mergeable(dfp_data, mmap_size);
 
-	(void)stress_get_setting("fp-method", &fp_method);
+	(void)stress_get_setting("dfp-method", &dfp_method);
 
 	stress_set_proc_state(args->name, STRESS_STATE_SYNC_WAIT);
 	stress_sync_start_wait(args);
@@ -541,7 +555,7 @@ static int stress_dfp(stress_args_t *args)
 	}
 
 	do {
-		if (UNLIKELY(stress_dfp_call_method(args, dfp_data, fp_method, verify) == EXIT_FAILURE)) {
+		if (UNLIKELY(stress_dfp_call_method(args, dfp_data, dfp_method, verify) == EXIT_FAILURE)) {
 			rc = EXIT_FAILURE;
 			break;
 		}
