@@ -1,5 +1,6 @@
 /*
  * Copyright (C) 2013-2021 Canonical, Ltd.
+ * Copyright (C) 2021-2025 Colin Ian King.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -15,33 +16,26 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * This code is a complete clean re-write of the stress tool by
- * Colin Ian King <colin.king@canonical.com> and attempts to be
- * backwardly compatible with the stress tool by Amos Waterland
- * <apw@rossby.metr.ou.edu> but has more stress tests and more
- * functionality.
- *
  */
 #include "stress-ng.h"
+#include "core-affinity.h"
+#include "core-builtin.h"
+#include "core-killpid.h"
+
+#if defined(HAVE_SYS_EVENTFD_H)
+#include <sys/eventfd.h>
+#endif
 
 static const stress_help_t help[] = {
-	{ NULL,	"eventfd N",	 "start N workers stressing eventfd read/writes" },
-	{ NULL,	"eventfd-ops N", "stop eventfd workers after N bogo operations" },
-	{ NULL, "eventfs-nonblock", "poll with non-blocking I/O on eventfd fd" },
-	{ NULL,	NULL,		 NULL }
+	{ NULL,	"eventfd N",		"start N workers stressing eventfd read/writes" },
+	{ NULL, "eventfd-nonblock",	"poll with non-blocking I/O on eventfd fd" },
+	{ NULL,	"eventfd-ops N",	"stop eventfd workers after N bogo operations" },
+	{ NULL,	NULL,			NULL }
 };
 
-static int stress_set_eventfd_nonblock(const char *opt)
-{
-	bool eventfd_nonblock = true;
-
-	(void)opt;
-	return stress_set_setting("eventfd-nonblock", TYPE_ID_BOOL, &eventfd_nonblock);
-}
-
-static const stress_opt_set_func_t opt_set_funcs[] = {
-	{ OPT_eventfd_nonblock,	stress_set_eventfd_nonblock },
-	{ 0,			NULL }
+static const stress_opt_t opts[] = {
+	{ OPT_eventfd_nonblock, "eventfd-nonblock", TYPE_ID_BOOL, 0, 1, NULL },
+	END_OPT,
 };
 
 #if defined(HAVE_SYS_EVENTFD_H) && \
@@ -52,11 +46,11 @@ static const stress_opt_set_func_t opt_set_funcs[] = {
  *  stress_eventfd
  *	stress eventfd read/writes
  */
-static int stress_eventfd(const stress_args_t *args)
+static int stress_eventfd(stress_args_t *args)
 {
 	pid_t pid;
 	int fd1, fd2, test_fd, rc;
-	int flags = 0;
+	int flags = 0, parent_cpu;
 	bool eventfd_nonblock = false;
 
 	(void)stress_get_setting("eventfd-nonblock", &eventfd_nonblock);
@@ -74,14 +68,14 @@ static int stress_eventfd(const stress_args_t *args)
 
 	fd1 = eventfd(0, flags);
 	if (fd1 < 0) {
-		rc = exit_status(errno);
+		rc = stress_exit_status(errno);
 		pr_fail("%s: eventfd failed, errno=%d (%s)\n",
 			args->name, errno, strerror(errno));
 		return rc;
 	}
 	fd2 = eventfd(0, flags);
 	if (fd2 < 0) {
-		rc = exit_status(errno);
+		rc = stress_exit_status(errno);
 		pr_fail("%s: eventfd failed, errno=%d (%s)\n",
 			args->name, errno, strerror(errno));
 		(void)close(fd1);
@@ -93,17 +87,20 @@ static int stress_eventfd(const stress_args_t *args)
 	if (test_fd >= 0)
 		(void)close(test_fd);
 
+	stress_set_proc_state(args->name, STRESS_STATE_SYNC_WAIT);
+	stress_sync_start_wait(args);
 	stress_set_proc_state(args->name, STRESS_STATE_RUN);
 again:
+	parent_cpu = stress_get_cpu();
 	pid = fork();
 	if (pid < 0) {
-		if (stress_redo_fork(errno))
+		if (stress_redo_fork(args, errno))
 			goto again;
 
 		(void)close(fd1);
 		(void)close(fd2);
 
-		if (!keep_stressing(args))
+		if (UNLIKELY(!stress_continue(args)))
 			return EXIT_SUCCESS;
 		pr_fail("%s: fork failed, errno=%d (%s)\n",
 			args->name, errno, strerror(errno));
@@ -111,24 +108,24 @@ again:
 	} else if (pid == 0) {
 		int n = 0;
 
-		(void)setpgid(0, g_pgrp);
+		stress_set_proc_state(args->name, STRESS_STATE_RUN);
+		(void)stress_change_cpu(args, parent_cpu);
 		stress_parent_died_alarm();
 		(void)sched_settings_apply(true);
 
-		while (keep_stressing_flag()) {
+		while (stress_continue_flag()) {
 			uint64_t val = 0;
 			ssize_t ret;
 			char re[7];
 
 			/* Exercise read on small buffer */
-			ret = read(fd1, re, sizeof(re));
-			(void)ret;
+			VOID_RET(ssize_t, read(fd1, re, sizeof(re)));
 
 			for (;;) {
-				if (!keep_stressing_flag())
+				if (UNLIKELY(!stress_continue_flag()))
 					goto exit_child;
 				ret = read(fd1, &val, sizeof(val));
-				if (ret < 0) {
+				if (UNLIKELY(ret < 0)) {
 					if ((errno == EAGAIN) ||
 					    (errno == EINTR))
 						continue;
@@ -136,7 +133,7 @@ again:
 						args->name, errno, strerror(errno));
 					goto exit_child;
 				}
-				if (ret < (ssize_t)sizeof(val)) {
+				if (UNLIKELY(ret < (ssize_t)sizeof(val))) {
 					pr_fail("%s child short read, got %zd, expecting %zd bytes\n",
 						args->name, ret, (ssize_t)sizeof(val));
 					goto exit_child;
@@ -147,27 +144,24 @@ again:
 			/*
 			 *  Periodically exercise with invalid writes
 			 */
-			if (n++ >= 64) {
+			if (UNLIKELY(n++ >= 64)) {
 				n = 0;
 
-
 				/* Exercise write using small buffer */
-				(void)memset(re, 0, sizeof(re));
-				ret = write(fd1, re, sizeof(re));
-				(void)ret;
+				(void)shim_memset(re, 0, sizeof(re));
+				VOID_RET(ssize_t, write(fd1, re, sizeof(re)));
 
 				/* Exercise write on buffer out of range */
 				val = ~0UL;
-				ret = write(fd1, &val, sizeof(val));
-				(void)ret;
+				VOID_RET(ssize_t, write(fd1, &val, sizeof(val)));
 			}
 
 			val = 1;
 			for (;;) {
-				if (!keep_stressing_flag())
+				if (UNLIKELY(!stress_continue_flag()))
 					goto exit_child;
 				ret = write(fd2, &val, sizeof(val));
-				if (ret < 0) {
+				if (UNLIKELY(ret < 0)) {
 					if ((errno == EAGAIN) ||
 					    (errno == EINTR))
 						continue;
@@ -175,7 +169,7 @@ again:
 						args->name, errno, strerror(errno));
 					goto exit_child;
 				}
-				if (ret < (ssize_t)sizeof(val)) {
+				if (UNLIKELY(ret < (ssize_t)sizeof(val))) {
 					pr_fail("%s child short write, got %zd, expecting %zd bytes\n",
 						args->name, ret, (ssize_t)sizeof(val));
 					goto exit_child;
@@ -188,7 +182,6 @@ exit_child:
 		(void)close(fd2);
 		_exit(EXIT_SUCCESS);
 	} else {
-		int status;
 		const pid_t self = getpid();
 
 		do {
@@ -202,11 +195,11 @@ exit_child:
 			(void)stress_read_fdinfo(self, stress_mwc1() ? fd1 : fd2);
 
 			for (;;) {
-				if (!keep_stressing_flag())
+				if (UNLIKELY(!stress_continue_flag()))
 					goto exit_parent;
 
 				ret = write(fd1, &val, sizeof(val));
-				if (ret < 0) {
+				if (UNLIKELY(ret < 0)) {
 					if ((errno == EAGAIN) ||
 					    (errno == EINTR))
 						continue;
@@ -214,7 +207,7 @@ exit_child:
 						args->name, errno, strerror(errno));
 					goto exit_parent;
 				}
-				if (ret < (ssize_t)sizeof(val)) {
+				if (UNLIKELY(ret < (ssize_t)sizeof(val))) {
 					pr_fail("%s parent short write, got %zd, expecting %zd bytes\n",
 						args->name, ret, (ssize_t)sizeof(val));
 					goto exit_parent;
@@ -223,11 +216,11 @@ exit_child:
 			}
 
 			for (;;) {
-				if (!keep_stressing_flag())
+				if (UNLIKELY(!stress_continue_flag()))
 					goto exit_parent;
 
 				ret = read(fd2, &val, sizeof(val));
-				if (ret < 0) {
+				if (UNLIKELY(ret < 0)) {
 					if ((errno == EAGAIN) ||
 					    (errno == EINTR))
 						continue;
@@ -235,37 +228,39 @@ exit_child:
 						args->name, errno, strerror(errno));
 					goto exit_parent;
 				}
-				if (ret < (ssize_t)sizeof(val)) {
+				if (UNLIKELY(ret < (ssize_t)sizeof(val))) {
 					pr_fail("%s parent short read, got %zd, expecting %zd bytes\n",
 						args->name, ret, (ssize_t)sizeof(val));
 					goto exit_parent;
 				}
 				break;
 			}
-			inc_counter(args);
-		} while (keep_stressing(args));
+			stress_bogo_inc(args);
+		} while (stress_continue(args));
 exit_parent:
 		stress_set_proc_state(args->name, STRESS_STATE_DEINIT);
 
-		(void)kill(pid, SIGKILL);
-		(void)shim_waitpid(pid, &status, 0);
+		(void)stress_kill_pid_wait(pid, NULL);
 		(void)close(fd1);
 		(void)close(fd2);
 	}
 	return EXIT_SUCCESS;
 }
 
-stressor_info_t stress_eventfd_info = {
+const stressor_info_t stress_eventfd_info = {
 	.stressor = stress_eventfd,
-	.class = CLASS_FILESYSTEM | CLASS_SCHEDULER | CLASS_OS,
-	.opt_set_funcs = opt_set_funcs,
+	.classifier = CLASS_FILESYSTEM | CLASS_SCHEDULER | CLASS_OS,
+	.opts = opts,
+	.verify = VERIFY_ALWAYS,
 	.help = help
 };
 #else
-stressor_info_t stress_eventfd_info = {
-	.stressor = stress_not_implemented,
-	.class = CLASS_FILESYSTEM | CLASS_SCHEDULER | CLASS_OS,
-	.opt_set_funcs = opt_set_funcs,
-	.help = help
+const stressor_info_t stress_eventfd_info = {
+	.stressor = stress_unimplemented,
+	.classifier = CLASS_FILESYSTEM | CLASS_SCHEDULER | CLASS_OS,
+	.opts = opts,
+	.verify = VERIFY_ALWAYS,
+	.help = help,
+	.unimplemented_reason = "built without sys/eventfd.h or eventfd() support"
 };
 #endif
