@@ -20,6 +20,7 @@
 #include "stress-ng.h"
 #include "core-builtin.h"
 #include "core-capabilities.h"
+#include "core-mmap.h"
 #include "core-pthread.h"
 
 #if defined(HAVE_SPAWN_H)
@@ -36,6 +37,7 @@ UNEXPECTED
 #define EXEC_METHOD_ALL		(0x00)
 #define EXEC_METHOD_EXECVE	(0x01)
 #define EXEC_METHOD_EXECVEAT	(0x02)
+#define EXEC_METHOD_FEXECVE	(0x03)
 
 #if defined(HAVE_CLONE)
 #define EXEC_FORK_METHOD_CLONE	(0x10)
@@ -68,7 +70,8 @@ typedef struct {
 	char *str;			/* huge argv and env string */
 	char *argv[4];			/* executable argv[] */
 	char *env[2];			/* executable env[] */
-#if defined(HAVE_EXECVEAT) &&	\
+#if (defined(HAVE_EXECVEAT) ||	\
+     defined(HAVE_FEXECVE)) &&	\
     defined(O_PATH)
 	int fdexec;			/* fd of program to exec */
 #endif
@@ -89,7 +92,7 @@ typedef struct stress_pid_hash {
 static size_t stress_pid_cache_index = 0;
 static size_t stress_pid_cache_items = 0;
 static stress_pid_hash_t *stress_pid_cache;
-static stress_pid_hash_t *stress_pid_hash_table[HASH_EXECS];
+static stress_pid_hash_t **stress_pid_hash_table;
 static stress_pid_hash_t *free_list;
 
 typedef struct {
@@ -101,6 +104,7 @@ static const stress_exec_method_t stress_exec_methods[] = {
 	{ "all",	EXEC_METHOD_ALL },
 	{ "execve",	EXEC_METHOD_EXECVE },
 	{ "execveat",	EXEC_METHOD_EXECVEAT },
+	{ "fexecve",	EXEC_METHOD_FEXECVE },
 };
 
 static const stress_exec_method_t stress_exec_fork_methods[] = {
@@ -345,6 +349,12 @@ static int stress_call_exec_method(const stress_exec_context_t *context)
 		else
 			ret = shim_execveat(context->fdexec, "", context->argv, context->env, AT_EMPTY_PATH);
 		break;
+#endif
+#if defined(HAVE_FEXECVE) &&	\
+    defined(O_PATH)
+	case EXEC_METHOD_FEXECVE:
+		ret = fexecve(context->fdexec, context->argv, context->env);
+		break;
 #endif
 	default:
 		ret = execve(context->exec_prog, context->argv, context->env);
@@ -418,7 +428,7 @@ static inline int stress_do_exec(stress_exec_context_t *context)
 
 			ret = pthread_join(pthread_exec, (void *)&exec_ret);
 			if (ret == 0) {
-				if (ret_dummy)
+				if (ret_dummy == 0)
 					(void)pthread_kill(pthread_dummy, SIGKILL);
 				return *exec_ret;
 			}
@@ -453,14 +463,27 @@ static int stress_exec_child(void *arg)
 	int method = argp->exec_method;
 	const bool big_env = ((argp->rnd8 >= 128 + 64) && (argp->rnd8 < 128 + 80));
 	const bool big_arg = ((argp->rnd8 >= 128 + 80) && (argp->rnd8 < 128 + 96));
-#if defined(HAVE_EXECVEAT) &&	\
+#if (defined(HAVE_EXECVEAT) ||	\
+     defined(HAVE_FEXECVE)) &&	\
     defined(O_PATH)
 	bool exec_garbage = ((argp->rnd8 >= 128) && (argp->rnd8 < 128 + 64));
 #else
 	bool exec_garbage = false;
 #endif
-	if (method == EXEC_METHOD_ALL)
-		method = stress_mwc1() ? EXEC_METHOD_EXECVE : EXEC_METHOD_EXECVEAT;
+	if (method == EXEC_METHOD_ALL) {
+		switch (stress_mwc8modn(3)) {
+		default:
+		case 0:
+			method = EXEC_METHOD_EXECVE;
+			break;
+		case 1:
+			method = EXEC_METHOD_EXECVEAT;
+			break;
+		case 2:
+			method = EXEC_METHOD_FEXECVE;
+			break;
+		}
+	}
 
 	stress_parent_died_alarm();
 	(void)sched_settings_apply(true);
@@ -486,7 +509,8 @@ static int stress_exec_child(void *arg)
 	/*
 	 *  Create a garbage executable
 	 */
-#if defined(HAVE_EXECVEAT) &&	\
+#if (defined(HAVE_EXECVEAT) ||	\
+     defined(HAVE_FEXECVE)) &&	\
     defined(O_PATH)
 	if (exec_garbage) {
 		char buffer[1024];
@@ -532,7 +556,8 @@ do_exec:
 		argp->env[0] = argp->str;
 	if (big_arg)
 		argp->argv[2] = argp->str;
-#if defined(HAVE_EXECVEAT) &&	\
+#if (defined(HAVE_EXECVEAT) ||	\
+     defined(HAVE_FEXECVE)) &&	\
     defined(O_PATH)
 	context.fdexec = exec_garbage ? fd : argp->fdexec;
 #endif
@@ -648,7 +673,8 @@ static int stress_exec(stress_args_t *args)
 	char exec_path[PATH_MAX];
 	char garbage_prog[PATH_MAX];
 	int ret, rc = EXIT_FAILURE;
-#if defined(HAVE_EXECVEAT) &&	\
+#if (defined(HAVE_EXECVEAT) ||	\
+     defined(HAVE_FEXECVE)) &&	\
     defined(O_PATH)
 	int fdexec;
 #endif
@@ -659,7 +685,7 @@ static int stress_exec(stress_args_t *args)
 	int exec_method = EXEC_METHOD_ALL;
 	int exec_fork_method = EXEC_FORK_METHOD_FORK;
 	bool exec_no_pthread = false;
-	size_t arg_max, cache_max;
+	size_t arg_max, cache_max, stress_pid_hash_table_size;
 	char *str;
 
 	if (!stress_get_setting("exec-max", &exec_max)) {
@@ -691,18 +717,31 @@ static int stress_exec(stress_args_t *args)
 	/* Remind folk that vfork can only do execve in this stressor */
 	if ((exec_fork_method == EXEC_FORK_METHOD_VFORK) &&
 	    (exec_method != EXEC_METHOD_EXECVE) &&
+	    (exec_method != EXEC_METHOD_FEXECVE) &&
 	    (stress_instance_zero(args))) {
-		pr_inf("%s: limiting vfork to only use execve()\n", args->name);
+		pr_inf("%s: limiting vfork to only use execve() or fexecve()\n", args->name);
 	}
 #endif
 
+	stress_pid_hash_table_size = sizeof(*stress_pid_hash_table) * HASH_EXECS;
+	stress_pid_hash_table = (stress_pid_hash_t **)
+		stress_mmap_populate(NULL, stress_pid_hash_table_size, PROT_READ | PROT_WRITE,
+			MAP_ANONYMOUS | MAP_PRIVATE, -1, 0);
+	if (stress_pid_hash_table == MAP_FAILED) {
+		pr_inf_skip("%s: failed to allocate %zu byte PID hash table%s, skipping stressor\n",
+			args->name, stress_pid_hash_table_size, stress_get_memfree_str());
+		return EXIT_NO_RESOURCE;
+	}
+	stress_set_vma_anon_name(stress_pid_hash_table, stress_pid_hash_table_size, "pid-hash");
+
 	cache_max = sizeof(*stress_pid_cache) * exec_max;
 	stress_pid_cache = (stress_pid_hash_t*)
-		mmap(NULL, cache_max, PROT_READ | PROT_WRITE,
+		stress_mmap_populate(NULL, cache_max, PROT_READ | PROT_WRITE,
 			MAP_ANONYMOUS | MAP_PRIVATE, -1, 0);
 	if (stress_pid_cache == MAP_FAILED) {
 		pr_inf_skip("%s: failed to allocate %zu byte PID hash cache%s, skipping stressor\n",
 			args->name, cache_max, stress_get_memfree_str());
+		(void)munmap((void *)stress_pid_hash_table, stress_pid_hash_table_size);
 		return EXIT_NO_RESOURCE;
 	}
 	stress_set_vma_anon_name(stress_pid_cache, cache_max, "pid-cache");
@@ -723,8 +762,16 @@ static int stress_exec(stress_args_t *args)
 	if (stress_instance_zero(args) &&
 	    ((exec_method == EXEC_METHOD_ALL) ||
 	     (exec_method == EXEC_METHOD_EXECVEAT))) {
-		pr_inf("%s: execveat not available, just using execve\n", args->name);
-		exec_method = EXEC_METHOD_EXECVE;
+		pr_inf("%s: execveat not available, defaulting to all\n", args->name);
+		exec_method = EXEC_METHOD_ALL;
+	}
+#endif
+#if !defined(HAVE_FEXECVE)
+	if (stress_instance_zero(args) &&
+	    ((exec_method == EXEC_METHOD_ALL) ||
+	     (exec_method == EXEC_METHOD_FEXECVE))) {
+		pr_inf("%s: fexecve not available, defauting to all\n", args->name);
+		exec_method = EXEC_METHOD_ALL;
 	}
 #endif
 	ret = stress_temp_dir_mk_args(args);
@@ -733,7 +780,8 @@ static int stress_exec(stress_args_t *args)
 	(void)stress_temp_filename_args(args,
 		garbage_prog, sizeof(garbage_prog), stress_mwc32());
 
-#if defined(HAVE_EXECVEAT) &&	\
+#if (defined(HAVE_EXECVEAT) ||	\
+     defined(HAVE_FEXECVE)) &&	\
     defined(O_PATH)
 	fdexec = open(exec_prog, O_PATH);
 	if (fdexec < 0) {
@@ -774,7 +822,8 @@ static int stress_exec(stress_args_t *args)
 			sph->arg.no_pthread = exec_no_pthread;
 			sph->arg.str = str;
 			sph->arg.args = args;
-#if defined(HAVE_EXECVEAT) &&	\
+#if (defined(HAVE_EXECVEAT) ||	\
+     defined(HAVE_FEXECVE)) &&	\
     defined(O_PATH)
 			sph->arg.fdexec = fdexec;
 #endif
@@ -790,6 +839,7 @@ static int stress_exec(stress_args_t *args)
 			case EXEC_FORK_METHOD_FORK:
 				pid = fork();
 				if (pid == 0) {
+					stress_set_proc_state(args->name, STRESS_STATE_RUN);
 					_exit(stress_exec_child(&sph->arg));
 				}
 				break;
@@ -799,6 +849,7 @@ static int stress_exec(stress_args_t *args)
 			case EXEC_FORK_METHOD_RFORK:
 				pid = rfork(RFPROC | RFFDG);
 				if (pid == 0) {
+					stress_set_proc_state(args->name, STRESS_STATE_RUN);
 					_exit(stress_exec_child(&sph->arg));
 				}
 				break;
@@ -819,12 +870,14 @@ static int stress_exec(stress_args_t *args)
 			case EXEC_FORK_METHOD_CLONE:
 				stack_top = (char *)stress_get_stack_top(sph->stack, CLONE_STACK_SIZE);
 				stack_top = stress_align_stack(stack_top);
+				stress_set_proc_state(args->name, STRESS_STATE_RUN);
 				pid = clone(stress_exec_child, stack_top, CLONE_VM | SIGCHLD, &sph->arg);
 				break;
 #endif
 #if defined(HAVE_SPAWN_H) &&	\
     defined(HAVE_POSIX_SPAWN)
 			case EXEC_FORK_METHOD_SPAWN:
+				stress_set_proc_state(args->name, STRESS_STATE_RUN);
 				if (posix_spawn(&pid, exec_prog, NULL, NULL, sph->arg.argv, sph->arg.env) != 0)
 					pid = -1;
 				break;
@@ -888,7 +941,8 @@ static int stress_exec(stress_args_t *args)
 	stress_set_proc_state(args->name, STRESS_STATE_DEINIT);
 
 
-#if defined(HAVE_EXECVEAT) &&	\
+#if (defined(HAVE_EXECVEAT) ||	\
+     defined(HAVE_FEXECVE)) &&	\
     defined(O_PATH)
 	(void)close(fdexec);
 #endif
@@ -899,14 +953,20 @@ static int stress_exec(stress_args_t *args)
 	}
 
 	rc = EXIT_SUCCESS;
-#if defined(HAVE_EXECVEAT) &&	\
+#if (defined(HAVE_EXECVEAT) ||	\
+     defined(HAVE_FEXECVE)) &&	\
     defined(O_PATH)
 err:
 #endif
 	stress_exec_free_pid();
 
+
 	if (str)
 		(void)munmap((void *)str, arg_max);
+
+	(void)munmap((void *)stress_pid_hash_table, stress_pid_hash_table_size);
+	(void)munmap((void *)stress_pid_cache, cache_max);
+
 	(void)shim_unlink(garbage_prog);
 	(void)stress_temp_dir_rm_args(args);
 
