@@ -376,7 +376,7 @@ static int OPTIMIZE3 stress_far_branch(stress_args_t *args)
 	NOCLOBBER stress_ret_func_t *funcs = NULL;
 	NOCLOBBER void **pages = NULL;
 	NOCLOBBER size_t total_funcs = 0, total_file_mapped_funcs = 0;
-	NOCLOBBER size_t n_pages_failed, n_pages_mapped = 0;
+	NOCLOBBER size_t n_pages_failed = 0, n_pages_mapped = 0;
 	NOCLOBBER double calls = 0.0;
 	NOCLOBBER bool far_branch_flush = false;
 	NOCLOBBER bool far_branch_pageout = false;
@@ -484,11 +484,12 @@ static int OPTIMIZE3 stress_far_branch(stress_args_t *args)
 	 */
 	for (k = 0, i = 0; i < PAGE_MULTIPLES; i++) {
 		for (j = 0; k < n_pages; j++, k++) {
-			const size_t shift = (16 + j) & 0xf;
+			const size_t shift = 16 + (j & 0xf);
 			size_t offset = ((uintptr_t)1 << shift) + (4 * page_size * (i + 4));
 
 			pages[k] = stress_far_mmap(fd, page_size, base, offset,
 						funcs, &total_funcs, &total_file_mapped_funcs);
+
 			if (pages[k] != MAP_FAILED) {
 				stress_set_vma_anon_name(pages[k], page_size, "functions-page");
 				n_pages_mapped++;
@@ -510,7 +511,7 @@ static int OPTIMIZE3 stress_far_branch(stress_args_t *args)
 			k = stress_mwc32modn((uint32_t)n_pages);
 			for (j = 0; j < n_pages; j++)  {
 				if (pages[k] != MAP_FAILED) {
-					pages[i] = pages[j];
+					pages[i] = pages[k];
 					break;
 				}
 				k++;
@@ -635,7 +636,7 @@ l2:
 cleanup:
 	if (pages) {
 		for (i = 0; i < n_pages; i++) {
-			if (pages[i])
+			if (pages[i] != MAP_FAILED)
 				(void)munmap((void *)pages[i], page_size);
 		}
 		free(pages);
