@@ -26,9 +26,14 @@
 #endif
 
 #include <sys/ioctl.h>
-#include <sys/file.h>
 #include <sched.h>
 
+#if defined(HAVE_SYS_FILE_H)
+#include <sys/file.h>
+#endif
+#if defined(HAVE_SYS_SELECT_H)
+#include <sys/select.h>
+#endif
 #include <sys/socket.h>
 #if defined(HAVE_SYS_UN_H)
 #include <sys/un.h>
@@ -96,9 +101,9 @@ static char stress_fd_filename[PATH_MAX];
 static double t_now;
 
 static const stress_help_t help[] = {
-	{ NULL,	"fd-abuse N",	"start N workers abusing file descriptors" },
-	{ NULL,	"fd-abuse N",	"stop fd-abuse after bogo operations" },
-	{ NULL,	NULL,		NULL }
+	{ NULL,	"fd-abuse N",		"start N workers abusing file descriptors" },
+	{ NULL,	"fd-abuse-ops N",	"stop fd-abuse after bogo operations" },
+	{ NULL,	NULL,			NULL }
 };
 
 static bool stress_fd_now(double *t, const double next)
@@ -183,7 +188,8 @@ static void stress_fd_open_file_rw_direct(stress_fd_t *fd)
 #endif
 
 #if defined(O_NONBLOCK) &&	\
-    defined(O_DIRECTORY)
+    defined(O_DIRECTORY) &&	\
+    defined(HAVE_OPENAT)
 static void stress_fd_open_temp_path(stress_fd_t *fd)
 {
 	const char *tmp = stress_get_temp_path();
@@ -328,6 +334,7 @@ static void stress_fd_open_pipe2_wr_end(stress_fd_t *fd)
 	fd->flags = 0;
 	if (pipe2(fds, flag) < 0) {
 		fd->fd = -1;
+		return;
 	}
 	(void)close(fds[0]);
 	fd->fd = fds[1];
@@ -454,6 +461,33 @@ static void stress_fd_open_sock_af_x25(stress_fd_t *fd)
 }
 #endif
 
+#if defined(AF_ROSE) &&		\
+    defined(SOCK_SEQPACKET)
+static void stress_fd_open_sock_af_rose(stress_fd_t *fd)
+{
+	fd->fd = socket(AF_ROSE, SOCK_SEQPACKET, 0);
+	fd->flags = FD_FLAG_RECV | FD_FLAG_SEND;
+}
+#endif
+
+#if defined(AF_IRDA) &&	\
+    defined(SOCK_SEQPACKET)
+static void stress_fd_open_sock_af_irda(stress_fd_t *fd)
+{
+	fd->fd = socket(AF_IRDA, SOCK_SEQPACKET, 0);
+	fd->flags = FD_FLAG_RECV | FD_FLAG_SEND;
+}
+#endif
+
+#if defined(AF_WANPIPE) &&	\
+    defined(SOCK_DGRAM)
+static void stress_fd_open_sock_af_wanpipe(stress_fd_t *fd)
+{
+	fd->fd = socket(AF_WANPIPE, SOCK_RAW, 0);
+	fd->flags = FD_FLAG_RECV | FD_FLAG_SEND;
+}
+#endif
+
 #if defined(AF_IPX) &&		\
     defined(SOCK_SEQPACKET)
 static void stress_fd_open_sock_af_ipx(stress_fd_t *fd)
@@ -500,6 +534,24 @@ static void stress_fd_open_sock_af_netlink(stress_fd_t *fd)
 }
 #endif
 
+#if defined(AF_NETROM) &&	\
+    defined(SOCK_SEQPACKET)
+static void stress_fd_open_sock_af_netrom(stress_fd_t *fd)
+{
+	fd->fd = socket(AF_NETROM, SOCK_SEQPACKET, 0);
+	fd->flags = FD_FLAG_RECV | FD_FLAG_SEND;
+}
+#endif
+
+#if defined(AF_ATMPVC) &&	\
+    defined(SOCK_SEQPACKET)
+static void stress_fd_open_sock_af_atmpvc(stress_fd_t *fd)
+{
+	fd->fd = socket(AF_ATMPVC, SOCK_SEQPACKET, 0);
+	fd->flags = FD_FLAG_RECV | FD_FLAG_SEND;
+}
+#endif
+
 #if defined(AF_RDS) &&	\
     defined(SOCK_SEQPACKET)
 static void stress_fd_open_sock_af_rds(stress_fd_t *fd)
@@ -509,6 +561,51 @@ static void stress_fd_open_sock_af_rds(stress_fd_t *fd)
 }
 #endif
 
+#if defined(AF_IUCV) &&	\
+    defined(SOCK_SEQPACKET)
+static void stress_fd_open_sock_af_iucv(stress_fd_t *fd)
+{
+	fd->fd = socket(AF_IUCV, SOCK_SEQPACKET, 0);
+	fd->flags = FD_FLAG_RECV | FD_FLAG_SEND;
+}
+#endif
+
+#if defined(AF_PHONET) &&	\
+    defined(SOCK_DGRAM)
+static void stress_fd_open_sock_af_phonet(stress_fd_t *fd)
+{
+	fd->fd = socket(AF_PHONET, SOCK_DGRAM, 0);
+	fd->flags = FD_FLAG_RECV | FD_FLAG_SEND;
+}
+#endif
+
+#if defined(AF_IEEE802154) &&	\
+    defined(SOCK_DGRAM)
+static void stress_fd_open_sock_af_ieee802154(stress_fd_t *fd)
+{
+	fd->fd = socket(AF_IEEE802154, SOCK_DGRAM, 0);
+	fd->flags = FD_FLAG_RECV | FD_FLAG_SEND;
+}
+#endif
+
+#if defined(AF_NFC) &&		\
+    defined(SOCK_STREAM)
+static void stress_fd_open_sock_af_nfc(stress_fd_t *fd)
+{
+	fd->fd = socket(AF_NFC, SOCK_STREAM, 1);
+	fd->flags = FD_FLAG_RECV | FD_FLAG_SEND;
+}
+#endif
+
+#if defined(AF_SMC) &&		\
+    defined(SOCK_STREAM)
+static void stress_fd_open_sock_af_smc(stress_fd_t *fd)
+{
+	fd->fd = socket(AF_SMC, SOCK_STREAM, 0);
+	fd->flags = FD_FLAG_RECV | FD_FLAG_SEND;
+}
+#endif
+
 #if defined(AF_PPPOX) &&	\
     defined(SOCK_DGRAM)
 static void stress_fd_open_sock_af_ppox(stress_fd_t *fd)
@@ -683,7 +780,8 @@ static open_func_t open_funcs[] = {
 	stress_fd_open_file_rw_sync,
 #endif
 #if defined(O_NONBLOCK) &&	\
-    defined(O_DIRECTORY)
+    defined(O_DIRECTORY) &&	\
+    defined(HAVE_OPENAT)
 	stress_fd_open_temp_path,
 #endif
 	stress_fd_open_pipe_rd_end,
@@ -745,6 +843,18 @@ static open_func_t open_funcs[] = {
     defined(SOCK_SEQPACKET)
 	stress_fd_open_sock_af_x25,
 #endif
+#if defined(AF_ROSE) &&		\
+    defined(SOCK_DGRAM)
+	stress_fd_open_sock_af_rose,
+#endif
+#if defined(AF_IRDA) &&	\
+    defined(SOCK_DGRAM)
+	stress_fd_open_sock_af_irda,
+#endif
+#if defined(AF_WANPIPE) &&	\
+    defined(SOCK_DGRAM)
+	stress_fd_open_sock_af_wanpipe,
+#endif
 #if defined(AF_IPX) &&		\
     defined(SOCK_SEQPACKET)
 	stress_fd_open_sock_af_ipx,
@@ -767,10 +877,38 @@ static open_func_t open_funcs[] = {
     defined(NETLINK_CONNECTOR)
 	stress_fd_open_sock_af_netlink,
 #endif
+#if defined(AF_NETROM) &&	\
+    defined(SOCK_SEQPACKET)
+	stress_fd_open_sock_af_netrom,
+#endif
+#if defined(AF_ATMPVC) &&	\
+    defined(SOCK_SEQPACKET)
+	stress_fd_open_sock_af_atmpvc,
+#endif
 #if defined(AF_RDS) &&	\
     defined(SOCK_SEQPACKET)
 	stress_fd_open_sock_af_rds,
 #endif
+#if defined(AF_IUCV) &&	\
+    defined(SOCK_SEQPACKET)
+	stress_fd_open_sock_af_iucv,
+#endif
+#if defined(AF_PHONET) &&	\
+    defined(SOCK_DGRAM)
+	stress_fd_open_sock_af_phonet,
+#endif
+#if defined(AF_IEEE802154) &&	\
+    defined(SOCK_DGRAM)
+	stress_fd_open_sock_af_ieee802154,
+#endif
+#if defined(AF_NFC) &&		\
+    defined(SOCK_STREAM)
+	stress_fd_open_sock_af_nfc,
+#endif
+#if defined(AF_SMC) &&		\
+    defined(SOCK_STREAM)
+	stress_fd_open_sock_af_smc,
+#endif
 #if defined(AF_PPPOX) &&	\
     defined(SOCK_DGRAM)
 	stress_fd_open_sock_af_ppox,
@@ -784,7 +922,7 @@ static open_func_t open_funcs[] = {
 	stress_fd_open_sock_af_can,
 #endif
 #if defined(AF_TIPC) &&	\
-    defined(SOCK_DGRAM)
+    defined(SOCK_SEQPACKET)
 	stress_fd_open_sock_af_tipc,
 #endif
 #if defined(AF_BLUETOOTH) &&	\
@@ -796,7 +934,7 @@ static open_func_t open_funcs[] = {
 	stress_fd_open_sock_af_kcm,
 #endif
 #if defined(AF_XDP) &&	\
-    defined(SOCK_SEQPACKET)
+    defined(SOCK_RAW)
 	stress_fd_open_sock_af_xdp,
 #endif
 	stress_fd_open_socketpair,
@@ -828,6 +966,7 @@ static open_func_t open_funcs[] = {
 #endif
 };
 
+#if defined(SOL_SOCKET)
 static void stress_fd_sockopt_reuseaddr(stress_fd_t *fd)
 {
 	int so_reuseaddr = 1;
@@ -835,6 +974,7 @@ static void stress_fd_sockopt_reuseaddr(stress_fd_t *fd)
 	(void)setsockopt(fd->fd, SOL_SOCKET, SO_REUSEADDR,
                 &so_reuseaddr, sizeof(so_reuseaddr));
 }
+#endif
 
 static void stress_fd_lseek(stress_fd_t *fd)
 {
@@ -900,11 +1040,11 @@ static void stress_fd_bind_af_inet(stress_fd_t *fd)
 		(void)shutdown(fd->fd, SHUT_RDWR);
 }
 
+#if defined(AF_INET6)
 static void stress_fd_bind_af_inet6(stress_fd_t *fd)
 {
 	struct sockaddr_in6 addr;
 #if defined(__minix__)
-	struct in6_addr in6addr_any = IN6ADDR_ANY_INIT;
 	struct in6_addr in6addr_loopback = IN6ADDR_LOOPBACK_INIT;
 #endif
 
@@ -915,6 +1055,7 @@ static void stress_fd_bind_af_inet6(stress_fd_t *fd)
 	if (bind(fd->fd, (const struct sockaddr *)&addr, (socklen_t)sizeof(addr)) == 0)
 		(void)shutdown(fd->fd, SHUT_RDWR);
 }
+#endif
 
 static void stress_fd_select_rd(stress_fd_t *fd)
 {
@@ -965,6 +1106,8 @@ static void stress_fd_pselect_rdwr(stress_fd_t *fd)
 }
 #endif
 
+#if defined(POLLIN) &&	\
+    defined(POLLOUT)
 static void stress_fd_poll_rdwr(stress_fd_t *fd)
 {
 	struct pollfd fds[1];
@@ -975,8 +1118,11 @@ static void stress_fd_poll_rdwr(stress_fd_t *fd)
 
 	(void)poll(fds, 1, 0);
 }
+#endif
 
-#if defined(HAVE_PPOLL)
+#if defined(HAVE_PPOLL) &&	\
+    defined(POLLIN) &&		\
+    defined(POLLOUT)
 static void stress_fd_ppoll_rdwr(stress_fd_t *fd)
 {
 	struct timespec tv;
@@ -1081,7 +1227,7 @@ static void stress_fd_ftruncate(stress_fd_t *fd)
 		VOID_RET(int, ftruncate(fd->fd, 0));
 }
 
-#if defined(POSIX_FADV_RANDOM) ||	\
+#if defined(POSIX_FADV_RANDOM) &&	\
     defined(HAVE_POSIX_FADVISE)
 static void stress_fd_posix_fadvise(stress_fd_t *fd)
 {
@@ -1372,7 +1518,7 @@ static void stress_fd_lockf(stress_fd_t *fd)
 	if (stress_fd_now(&t, 13.0)) {
 		if (lockf(fd->fd, F_TLOCK, 0) < 0)
 			return;
-		(void)lockf(fd->fd, F_UNLOCK);
+		(void)lockf(fd->fd, F_UNLOCK, 0);
 	}
 }
 #endif
@@ -1634,7 +1780,8 @@ static void stress_fd_sendto(stress_fd_t *fd)
 }
 #endif
 
-#if defined(MSG_DONTWAIT)
+#if defined(MSG_DONTWAIT) &&	\
+    defined(HAVE_RECVMSG)
 static void stress_fd_recvmsg(stress_fd_t *fd)
 {
 	if (fd->flags & (FD_FLAG_RECV | FD_FLAG_READ)) {
@@ -1654,7 +1801,8 @@ static void stress_fd_recvmsg(stress_fd_t *fd)
 }
 #endif
 
-#if defined(MSG_DONTWAIT)
+#if defined(MSG_DONTWAIT) &&	\
+    defined(HAVE_SENDMSG)
 static void stress_fd_sendmsg(stress_fd_t *fd)
 {
 	if (fd->flags & (FD_FLAG_SEND | FD_FLAG_WRITE)) {
@@ -1734,7 +1882,9 @@ static void stress_fd_splice(stress_fd_t *fd)
 #endif
 
 static const fd_func_t fd_funcs[] = {
+#if defined(SOL_SOCKET)
 	stress_fd_sockopt_reuseaddr,
+#endif
 	stress_fd_lseek,
 	stress_fd_dup,
 	stress_fd_dup2,
@@ -1742,14 +1892,21 @@ static const fd_func_t fd_funcs[] = {
 	stress_fd_dup3,
 #endif
 	stress_fd_bind_af_inet,
+#if defined(AF_INET6)
 	stress_fd_bind_af_inet6,
+#endif
 	stress_fd_select_rd,
 	stress_fd_select_wr,
 #if defined(HAVE_PSELECT)
 	stress_fd_pselect_rdwr,
 #endif
+#if defined(POLLIN) &&	\
+    defined(POLLOUT)
 	stress_fd_poll_rdwr,
-#if defined(HAVE_PPOLL)
+#endif
+#if defined(HAVE_PPOLL) &&	\
+    defined(POLLIN) &&		\
+    defined(POLLOUT)
 	stress_fd_ppoll_rdwr,
 #endif
 	stress_fd_mmap_rd,
@@ -1777,7 +1934,7 @@ static const fd_func_t fd_funcs[] = {
 	stress_fd_fcntl_f_getfl,
 #endif
 	stress_fd_ftruncate,
-#if defined(POSIX_FADV_RANDOM) ||	\
+#if defined(POSIX_FADV_RANDOM) &&	\
     defined(HAVE_POSIX_FADVISE)
 	stress_fd_posix_fadvise,
 #endif
@@ -1889,10 +2046,12 @@ static const fd_func_t fd_funcs[] = {
 #if defined(MSG_DONTWAIT)
 	stress_fd_sendto,
 #endif
-#if defined(MSG_DONTWAIT)
+#if defined(MSG_DONTWAIT) &&	\
+    defined(HAVE_RECVMSG)
 	stress_fd_recvmsg,
 #endif
-#if defined(MSG_DONTWAIT)
+#if defined(MSG_DONTWAIT) &&	\
+    defined(HAVE_SENDMSG)
 	stress_fd_sendmsg,
 #endif
 #if defined(HAVE_SYS_SENDFILE_H) &&	\
@@ -1936,6 +2095,7 @@ static int stress_fd_abuse_process(stress_args_t *args, void *context)
 	 *  fds for more of a stress mix.
 	 */
 	pid = fork();
+	stress_set_proc_state(args->name, STRESS_STATE_RUN);
 	do {
 		size_t j;
 
