@@ -151,8 +151,8 @@ static void stress_fd_race_close_fds(
 			j = (size_t)stress_mwc32modn(n);
 
 			tmp = fds[i];
-			fds[j] = fds[i];
-			fds[i] = tmp;
+			fds[i] = fds[j];
+			fds[j] = tmp;
 		}
 		for (i = 0; i < n; i++)
 			(void)close(fds[i]);
@@ -362,7 +362,8 @@ retry:
 		(void)close(fd);
 	} while (stress_continue(args));
 
-#if defined(HAVE_SOCKADDR_UN)
+#if defined(HAVE_SYS_UN_H) &&	\
+    defined(HAVE_SOCKADDR_UN)
 	if (addr) {
 		const struct sockaddr_un *addr_un = (struct sockaddr_un *)addr;
 
@@ -387,14 +388,16 @@ static void *stress_fd_race_current(void *ptr)
 
 	do {
 		int current_fd = context->current_fd;
-		int fd = current_fd + stress_mwc1();
-		const int fd_end = fd + 10;
+		int fd, fd_end;
 
 		if (context->current_fd == -1) {
 			(void)shim_usleep(200000);
 			continue;
 		}
 
+		fd = current_fd + stress_mwc1();
+		fd_end = fd + 10;
+
 		while (fd < fd_end) {
 			struct stat statbuf;
 			uint8_t rnd;
@@ -623,7 +626,8 @@ retry:
 die_close:
 	(void)close(fd);
 die:
-#if defined(HAVE_SOCKADDR_UN)
+#if defined(HAVE_SYS_UN_H) &&	\
+    defined(HAVE_SOCKADDR_UN)
 	if (addr) {
 		const struct sockaddr_un *addr_un = (struct sockaddr_un *)addr;
 
@@ -893,6 +897,7 @@ again:
 			args->name, errno, strerror(errno));
 		goto tidy_barrier;
 	} else if (pid == 0) {
+		stress_set_proc_state(args->name, STRESS_STATE_RUN);
 		stress_set_oom_adjustment(args, false);
 		rc = stress_race_fd_client(&context);
 		_exit(rc);
