@@ -22,11 +22,13 @@
 #include "core-out-of-memory.h"
 
 #include <sys/types.h>
-#include <sys/file.h>
 #include <sys/ioctl.h>
 #include <dirent.h>
 #include <time.h>
 
+#if defined(HAVE_SYS_FILE_H)
+#include <sys/file.h>
+#endif
 #if defined(HAVE_LINUX_FS_H)
 #include <linux/fs.h>
 #endif
@@ -179,6 +181,22 @@ static void stress_filerace_lseek_end(const int fd, const char *filename)
 	VOID_RET(off_t, lseek(fd, 0, SEEK_END));
 }
 
+#if defined(SEEK_DATA)
+static void stress_filerace_lseek_data(const int fd, const char *filename)
+{
+	(void)filename;
+	VOID_RET(off_t, lseek(fd, (off_t)stress_mwc32(), SEEK_DATA));
+}
+#endif
+
+#if defined(SEEK_HOLE)
+static void stress_filerace_lseek_hole(const int fd, const char *filename)
+{
+	(void)filename;
+	VOID_RET(off_t, lseek(fd, (off_t)stress_mwc32(), SEEK_HOLE));
+}
+#endif
+
 static void stress_filerace_fchmod(const int fd, const char *filename)
 {
 	(void)filename;
@@ -347,11 +365,12 @@ static void stress_filerace_ftruncate_filename(const int fd, const char *filenam
 	if (tmp_fd != -1) {
 		const off_t offset = (off_t)stress_mwc16();
 
-		VOID_RET(int, ftruncate(fd, offset));
+		VOID_RET(int, ftruncate(tmp_fd, offset));
 		(void)close(tmp_fd);
 	}
 }
 
+#if defined(HAVE_UTIMES)
 static void stress_filerace_utimes(const int fd, const char *filename)
 {
 	struct timeval tv[2];
@@ -359,12 +378,13 @@ static void stress_filerace_utimes(const int fd, const char *filename)
 	(void)fd;
 	tv[0].tv_sec = (time_t)stress_mwc64() & 0x3ffffffffULL;
 	tv[0].tv_usec = stress_mwc32modn(1000000);
-	tv[0].tv_sec = (time_t)stress_mwc64() & 0x3ffffffffULL;
+	tv[1].tv_sec = (time_t)stress_mwc64() & 0x3ffffffffULL;
 	tv[1].tv_usec = stress_mwc32modn(1000000);
 
 	errno = 0;
 	VOID_RET(int, utimes(filename, tv));
 }
+#endif
 
 #if defined(HAVE_FUTIMES)
 static void stress_filerace_futimes(const int fd, const char *filename)
@@ -374,10 +394,10 @@ static void stress_filerace_futimes(const int fd, const char *filename)
 	(void)filename;
 
 	tv[0].tv_sec = (time_t)stress_mwc64() & 0x3ffffffffULL;
-	tv[0].tv_usec = (time_t)stress_mwc32modn(1000000);
+	tv[0].tv_usec = (suseconds_t)stress_mwc32modn(1000000);
 
 	tv[1].tv_sec = (time_t)stress_mwc64() & 0x3ffffffffULL;
-	tv[1].tv_usec = (time_t)stress_mwc32modn(1000000);
+	tv[1].tv_usec = (suseconds_t)stress_mwc32modn(1000000);
 
 	VOID_RET(int, futimes(fd, tv));
 }
@@ -532,7 +552,7 @@ static void stress_filerace_open_rd(const int fd, const char *filename)
 
 	(void)fd;
 	new_fd = open(filename, O_RDONLY | flag);
-	if (new_fd > -1)
+	if (new_fd != -1)
 		(void)close(new_fd);
 }
 
@@ -543,7 +563,7 @@ static void stress_filerace_open_wr(const int fd, const char *filename)
 
 	(void)fd;
 	new_fd = open(filename, O_APPEND | O_WRONLY | flag);
-	if (new_fd > -1)
+	if (new_fd != -1)
 		(void)close(new_fd);
 }
 
@@ -613,13 +633,14 @@ static void stress_filerace_openmany(const int fd, const char *filename)
 
 	int fds[MAX_FDS];
 
+	(void)fd;
 	for (i = 0; i < SIZEOF_ARRAY(fds); i++) {
 		fds[i] = open(filename, O_CREAT | O_RDWR | O_APPEND, S_IRUSR | S_IWUSR);
-		if (fds[i] > -1)
-			stress_filerace_write_random_uint32(fd);
+		if (fds[i] != -1)
+			stress_filerace_write_random_uint32(fds[i]);
 	}
 	for (i = 0; i < SIZEOF_ARRAY(fds); i++) {
-		if (fds[i] > -1)
+		if (fds[i] != -1)
 			(void)close(fds[i]);
 	}
 }
@@ -827,6 +848,7 @@ static void NORETURN MLOCKED_TEXT stress_filerace_mmap_sigbus_handler(int sig)
 	(void)sig;
 
 	siglongjmp(mmap_jmpbuf, 1);
+	stress_no_return();
 }
 
 static void stress_filerace_mmap(const int fd, const char *filename)
@@ -1070,7 +1092,7 @@ static void stress_filerace_sendfile_fd(const int fd, const char *filename)
 
 	(void)filename;
 	fd_zero = open("/dev/urandom", O_RDONLY);
-	if (fd_zero < 0)
+	if (fd_zero == -1)
 		return;
 
 	offset = ((off_t)stress_mwc32()) & OFFSET_MASK;
@@ -1117,10 +1139,16 @@ static void stress_filerace_name_to_handle_at(const int fd, const char *filename
 }
 #endif
 
-static stress_filerace_fops_t stress_filerace_fops[] = {
+static const stress_filerace_fops_t stress_filerace_fops[] = {
 	stress_filerace_fstat,
 	stress_filerace_lseek_set,
 	stress_filerace_lseek_end,
+#if defined(SEEK_DATA)
+	stress_filerace_lseek_data,
+#endif
+#if defined(SEEK_HOLE)
+	stress_filerace_lseek_hole,
+#endif
 	stress_filerace_fchmod,
 	stress_filerace_fchown,
 #if defined(F_GETFL)
@@ -1159,7 +1187,9 @@ static stress_filerace_fops_t stress_filerace_fops[] = {
 #endif
 	stress_filerace_ftruncate,
 	stress_filerace_ftruncate_filename,
+#if defined(HAVE_UTIMES)
 	stress_filerace_utimes,
+#endif
 #if defined(HAVE_FUTIMES)
 	stress_filerace_futimes,
 #endif
@@ -1321,7 +1351,7 @@ static void stress_filerace_filename(const char *pathname, char *filename, const
 
 	/* wrap or -ve time? */
 	if (t < 0) {
-		t_start = t;
+		t_start = time(NULL);
 		t = 0;
 	}
 	t &= 0x3f;
@@ -1358,7 +1388,7 @@ static void stress_filerace_child(stress_args_t *args, const char *pathname, con
 			(void)shim_unlink(filename);
 			(void)shim_rmdir(filename);
 			fds[fd_idx] = creat(filename, S_IRUSR | S_IWUSR);
-			if (fds[fd_idx] > 0) {
+			if (fds[fd_idx] != -1) {
 				if (stress_continue(args))
 					stress_filerace_file(fds[fd_idx], filename);
 				fd_idx++;
@@ -1373,7 +1403,7 @@ static void stress_filerace_child(stress_args_t *args, const char *pathname, con
 			stress_filerace_filename(pathname, filename, sizeof(filename));
 			flag = open_wr_flags[stress_mwc8modn((uint8_t)SIZEOF_ARRAY(open_wr_flags))];
 			fds[fd_idx] = open(filename, O_CREAT | O_RDWR | O_APPEND | flag, S_IRUSR | S_IWUSR);
-			if (fds[fd_idx] > 0) {
+			if (fds[fd_idx] != -1) {
 				if (stress_continue(args))
 					stress_filerace_file(fds[fd_idx], filename);
 				fd_idx++;
@@ -1383,7 +1413,7 @@ static void stress_filerace_child(stress_args_t *args, const char *pathname, con
 			stress_filerace_filename(pathname, filename, sizeof(filename));
 			flag = open_wr_flags[stress_mwc8modn((uint8_t)SIZEOF_ARRAY(open_wr_flags))];
 			fds[fd_idx] = open(filename, O_CREAT | O_RDWR | flag, S_IRUSR | S_IWUSR);
-			if (fds[fd_idx] > 0) {
+			if (fds[fd_idx] != -1) {
 				if (stress_continue(args))
 					stress_filerace_file(fds[fd_idx], filename);
 				fd_idx++;
@@ -1412,6 +1442,7 @@ static void stress_filerace_child(stress_args_t *args, const char *pathname, con
 			} else {
 #if defined(O_DIRECTORY)
 				fd = open(pathname, O_DIRECTORY);
+				if (fd != -1)
 					(void)close(fd);
 #endif
 				VOID_RET(int, stat(pathname, &buf));
@@ -1444,7 +1475,7 @@ static void stress_filerace_child(stress_args_t *args, const char *pathname, con
 			VOID_RET(int, mkdir(filename, S_IRUSR | S_IWUSR | S_IXUSR));
 #if defined(O_DIRECTORY)
 			fd = open(filename, O_DIRECTORY);
-			if (fd >= 0) {
+			if (fd != -1) {
 				if (stress_continue(args))
 					stress_filerace_file(fd, filename);
 				(void)close(fd);
@@ -1457,7 +1488,7 @@ static void stress_filerace_child(stress_args_t *args, const char *pathname, con
 
 				(void)snprintf(filename, sizeof(filename), "%s/%2.2" PRIx8, pathname, n);
 				tmp_fd = open(filename, O_CREAT | O_RDWR | O_APPEND, S_IRUSR | S_IWUSR);
-				if (tmp_fd > -1)
+				if (tmp_fd != -1)
 					(void)close(tmp_fd);
 			}
 			break;
@@ -1469,7 +1500,7 @@ static void stress_filerace_child(stress_args_t *args, const char *pathname, con
 
 				flag = open_wr_flags[stress_mwc8modn((uint8_t)SIZEOF_ARRAY(open_wr_flags))];
 				tmp_fd = open(filename, O_CREAT | O_RDWR | flag, S_IRUSR | S_IWUSR);
-				if (tmp_fd > -1)
+				if (tmp_fd != 1)
 					(void)close(tmp_fd);
 				(void)unlink(filename);
 				(void)rmdir(filename);
@@ -1510,6 +1541,8 @@ static void stress_filerace_child(stress_args_t *args, const char *pathname, con
 			if (stress_continue(args))
 				pid = fork();
 
+			stress_set_proc_state(args->name, STRESS_STATE_RUN);
+
 			for (i = 0; i < SIZEOF_ARRAY(fds); i++) {
 				if (fds[i] != -1) {
 					if (stress_continue(args))
@@ -1608,7 +1641,7 @@ tidy_dir:
 
 const stressor_info_t stress_filerace_info = {
 	.stressor = stress_filerace,
-	.class = CLASS_FILESYSTEM | CLASS_OS,
+	.classifier = CLASS_FILESYSTEM | CLASS_OS,
 	.verify = VERIFY_NONE,
 	.help = help
 };
