@@ -23,6 +23,7 @@
 #include "core-asm-ret.h"
 #include "core-builtin.h"
 #include "core-cpu-cache.h"
+#include "core-mmap.h"
 #include "core-numa.h"
 #include "core-out-of-memory.h"
 
@@ -278,8 +279,8 @@ static int stress_flushcache_child(stress_args_t *args, void *ctxt)
 			PROT_READ | PROT_WRITE,
 			MAP_ANONYMOUS | MAP_SHARED, -1, 0);
 	if (context->d_addr == MAP_FAILED) {
-		pr_inf_skip("%s: failed to mmap %zd bytes, skipping stressor\n",
-			args->name, context->d_size);
+		pr_inf_skip("%s: failed to mmap %zu bytes%s, skipping stressor\n",
+			args->name, context->d_size, stress_get_memfree_str());
 		return EXIT_NO_RESOURCE;
 	}
 	stress_set_vma_anon_name(context->d_addr, context->d_size, "d-cache");
@@ -306,7 +307,7 @@ static int stress_flushcache_child(stress_args_t *args, void *ctxt)
 
 	stress_set_proc_state(args->name, STRESS_STATE_DEINIT);
 
-	(void)munmap(context->d_addr, args->page_size);
+	(void)munmap(context->d_addr, context->d_size);
 
 	return EXIT_SUCCESS;
 }
@@ -319,18 +320,21 @@ static int stress_flushcache_child(stress_args_t *args, void *ctxt)
 static int stress_flushcache(stress_args_t *args)
 {
 	const size_t page_size = args->page_size;
-	const int numa_nodes = stress_numa_nodes();
+	long int numa_nodes = stress_numa_nodes();
 	stress_flushcache_context_t context;
 	int ret;
 
+	if (UNLIKELY(numa_nodes < 1))
+		numa_nodes = 1;
+
 	context.x86_clfsh = stress_cpu_x86_has_clfsh();
 	context.x86_demote = stress_cpu_x86_has_cldemote();
 	context.i_addr = stress_mmap_populate(NULL, page_size,
 				PROT_READ | PROT_WRITE | PROT_EXEC,
 				MAP_ANONYMOUS | MAP_SHARED, -1, 0);
 	if (context.i_addr == MAP_FAILED) {
-		pr_inf_skip("%s: could not mmap %zd sized page, skipping stressor\n",
-			args->name, page_size);
+		pr_inf_skip("%s: could not mmap %zu sized page%s, skipping stressor\n",
+			args->name, page_size, stress_get_memfree_str());
 		return EXIT_NO_RESOURCE;
 	}
 	stress_set_vma_anon_name(context.i_addr, page_size, "i-cache");
@@ -346,8 +350,8 @@ static int stress_flushcache(stress_args_t *args)
 		context.cl_size = 64;
 
 	context.d_size *= numa_nodes;
-	if ((args->instance == 0) && (numa_nodes > 1))
-		pr_inf("%s: scaling cache size by number of numa nodes %d to %zdK\n",
+	if (stress_instance_zero(args) && (numa_nodes > 1))
+		pr_inf("%s: scaling cache size by number of numa nodes %ld to %zuK\n",
 			args->name, numa_nodes, context.d_size / 1024);
 	ret = stress_oomable_child(args, (void *)&context, stress_flushcache_child, STRESS_OOMABLE_NORMAL);
 
@@ -357,14 +361,14 @@ static int stress_flushcache(stress_args_t *args)
 
 const stressor_info_t stress_flushcache_info = {
 	.stressor = stress_flushcache,
-	.class = CLASS_CPU_CACHE,
+	.classifier = CLASS_CPU_CACHE,
 	.supported = stress_asm_ret_supported,
 	.help = help
 };
 #else
 const stressor_info_t stress_flushcache_info = {
 	.stressor = stress_unimplemented,
-	.class = CLASS_CPU_CACHE,
+	.classifier = CLASS_CPU_CACHE,
 	.supported = stress_asm_ret_supported,
 	.help = help,
 	.unimplemented_reason = "built without cache flush support"
