@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2022-2024 Colin Ian King.
+ * Copyright (C) 2022-2025 Colin Ian King.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -38,6 +38,7 @@ static void stress_fsize_handler(int signum)
 {
 	if (signum == SIGXFSZ) {
 		sigxfsz = true;
+		/* intentionally racy */
 		sigxfsz_count++;
 	}
 }
@@ -100,7 +101,7 @@ static int stress_fsize_boundary(
 	new_rlim.rlim_cur = offset;
 
 	if (setrlimit(RLIMIT_FSIZE, &new_rlim) < 0) {
-		pr_fail("%s: failed to set RLIMIT_FSIZE to %jd (0x%jx), errno=%d (%s)\n",
+		pr_fail("%s: failed to set RLIMIT_FSIZE to %" PRIdMAX " (0x%" PRIxMAX "), errno=%d (%s)\n",
 			args->name, (intmax_t)new_rlim.rlim_cur, (intmax_t)new_rlim.rlim_cur,
 			errno, strerror(errno));
 	}
@@ -110,7 +111,7 @@ static int stress_fsize_boundary(
 	ret = shim_fallocate(fd, 0, off, size);
 	if (ret < 0) {
 		if ((errno != EFBIG) && (errno != ENOSPC) && (errno != EINTR)) {
-			pr_fail("%s: fallocate failed at offset %jd (0x%jx) with unexpected error: %d (%s)\n",
+			pr_fail("%s: fallocate failed at offset %" PRIdMAX" (0x%" PRIxMAX ") with unexpected error, errno=%d (%s)\n",
 				args->name, (intmax_t)off, (intmax_t)off,
 				errno, strerror(errno));
 			rc = EXIT_FAILURE;
@@ -118,9 +119,15 @@ static int stress_fsize_boundary(
 		return rc;
 	}
 	if (sigxfsz) {
-		pr_fail("%s: got an unexpected SIGXFSZ signal at offset %jd (0x%jx)\n",
+		pr_fail("%s: got an unexpected SIGXFSZ signal at offset %" PRIdMAX " (0x%" PRIxMAX ")\n",
 			args->name, (intmax_t)off, (intmax_t)off);
-		rc = EXIT_FAILURE;
+		return EXIT_FAILURE;
+	}
+	/* We should be able to ftruncate a file to zero bytes */
+	if (ftruncate(fd, 0) < 0) {
+		pr_inf("%s: truncating file to zero bytes failed, errno=%d (%s)\n",
+			args->name, errno, strerror(errno));
+		return EXIT_FAILURE;
 	}
 
 	sigxfsz = false;
@@ -128,18 +135,19 @@ static int stress_fsize_boundary(
 	ret = shim_fallocate(fd, 0, off, size);
 	if (ret == 0) {
 		if (!stress_fsize_reported(off, FSIZE_TYPE_FALLOC)) {
-			pr_inf("%s: fallocate unexpectedly succeeded at offset %jd (0x%jx), expecting EFBIG error\n",
+			pr_inf("%s: fallocate unexpectedly succeeded at offset %" PRIdMAX " (0x%" PRIxMAX "), expecting EFBIG error\n",
 				args->name, (intmax_t)off, (intmax_t)off);
+			return EXIT_FAILURE;
 		}
 		return rc;
 	} else if ((errno != EFBIG) && (errno != ENOSPC) && (errno != EINTR)) {
-		pr_fail("%s: fallocate failed at offset %jd (0x%jx) with unexpected error: %d (%s)\n",
+		pr_fail("%s: fallocate failed at offset %" PRIdMAX" (0x%" PRIxMAX ") with unexpected error, errno=%d (%s)\n",
 			args->name, (intmax_t)off, (intmax_t)off,
 			errno, strerror(errno));
 		return EXIT_FAILURE;
 	}
 	if (!sigxfsz && !stress_fsize_reported(off, FSIZE_TYPE_SIGXFSZ)) {
-		pr_inf("%s: did not get expected SIGXFSZ signal at offset %jd (0x%jx)\n",
+		pr_inf("%s: did not get expected SIGXFSZ signal at offset %" PRIdMAX " (0x%" PRIxMAX ")\n",
 			args->name, (intmax_t)off, (intmax_t)off);
 		return EXIT_FAILURE;
 	}
@@ -171,7 +179,6 @@ static int stress_fsize(stress_args_t *args)
 	char filename[PATH_MAX];
 	int fd, ret, rc = EXIT_SUCCESS;
 	struct rlimit old_rlim;
-	off_t offset;
 	rlim_t max;
 	const off_t max_offset = stress_fsize_max_off_t();
 	double t, duration, rate;
@@ -202,18 +209,21 @@ static int stress_fsize(stress_args_t *args)
 
 	stress_file_rw_hint_short(fd);
 
+	stress_set_proc_state(args->name, STRESS_STATE_SYNC_WAIT);
+	stress_sync_start_wait(args);
 	stress_set_proc_state(args->name, STRESS_STATE_RUN);
 
 	t = stress_time_now();
 	max = STRESS_MINIMUM(old_rlim.rlim_max, 1024 * 256);
 	do {
 		struct rlimit new_rlim;
+		off_t offset;
 
 		new_rlim.rlim_cur = max;
 		new_rlim.rlim_max = old_rlim.rlim_max;
 
 		if (setrlimit(RLIMIT_FSIZE, &new_rlim) < 0) {
-			pr_fail("%s: failed to set RLIMIT_FSIZE to %jd (0x%jx), errno=%d (%s)\n",
+			pr_fail("%s: failed to set RLIMIT_FSIZE to %" PRIdMAX " (0x%" PRIxMAX "), errno=%d (%s)\n",
 				args->name, (intmax_t)max, (intmax_t)max, errno, strerror(errno));
 		}
 
@@ -231,13 +241,13 @@ static int stress_fsize(stress_args_t *args)
 		if (shim_fallocate(fd, 0, 0, (off_t)max) < 0) {
 			if ((errno == ENOSPC) || (errno == EINTR)) {
 				/* No resource */
-				pr_inf_skip("%s: allocating file to %jd (0x%jx) bytes failed, errno=%d (%s), "
+				pr_inf_skip("%s: allocating file to %" PRIdMAX " (0x%" PRIxMAX ") bytes failed, errno=%d (%s), "
 					"skipping stressor\n", args->name, (intmax_t)max, (intmax_t)max,
 					errno, strerror(errno));
 				rc = EXIT_NO_RESOURCE;
 			} else {
 				/* A real issue, report it */
-				pr_inf("%s: allocating file to %jd bytes (0x%jx) failed, errno=%d (%s), "
+				pr_inf("%s: allocating file to %" PRIdMAX " bytes (0x%" PRIxMAX ") failed, errno=%d (%s), "
 					"terminating stressor\n", args->name, (intmax_t)max, (intmax_t)max,
 					errno, strerror(errno));
 				rc = EXIT_FAILURE;
@@ -246,16 +256,16 @@ static int stress_fsize(stress_args_t *args)
 		}
 		sigxfsz = false;
 		if (shim_fallocate(fd, 0, (off_t)max, 4096) == 0) {
-			pr_fail("%s: fallocate unexpectedly succeeded at offset %jd (0x%jx), expecting EFBIG error\n",
+			pr_fail("%s: fallocate unexpectedly succeeded at offset %" PRIdMAX " (0x%" PRIxMAX "), expecting EFBIG error\n",
 				args->name, (intmax_t)max, (intmax_t)max);
 			rc = EXIT_FAILURE;
 		} else if ((errno != EFBIG) && (errno != ENOSPC) && (errno != EINTR)) {
-			pr_fail("%s: failed at offset %jd (0x%jx) with unexpected error: %d (%s)\n",
+			pr_fail("%s: failed at offset %" PRIdMAX " (0x%" PRIxMAX ") with unexpected error, errno=%d (%s)\n",
 				args->name, (intmax_t)max, (intmax_t)max, errno, strerror(errno) );
 			rc = EXIT_FAILURE;
 		}
 		if (!sigxfsz) {
-			pr_fail("%s: expected a SIGXFSZ signal at offset %jd (0x%jx), nothing happened\n",
+			pr_fail("%s: expected a SIGXFSZ signal at offset %" PRIdMAX " (0x%" PRIxMAX "), nothing happened\n",
 				args->name, (intmax_t)max, (intmax_t)max);
 			rc = EXIT_FAILURE;
 		}
@@ -272,13 +282,15 @@ static int stress_fsize(stress_args_t *args)
 			rc = EXIT_FAILURE;
 			break;
 		}
-		if (stress_fsize_boundary(args, fd, &old_rlim, offset, max - offset) == EXIT_FAILURE)
+		if (stress_fsize_boundary(args, fd, &old_rlim, offset, max - offset) == EXIT_FAILURE) {
 			rc = EXIT_FAILURE;
+			break;
+		}
 
 		/* Should be able to set back to original size */
 		new_rlim = old_rlim;
 		if (setrlimit(RLIMIT_FSIZE, &new_rlim) < 0) {
-			pr_fail("%s: failed to set RLIMIT_FSIZE to %jd (0x%jx), errno=%d (%s)\n",
+			pr_fail("%s: failed to set RLIMIT_FSIZE to %" PRIdMAX " (0x%" PRIxMAX "), errno=%d (%s)\n",
 				args->name, (intmax_t)new_rlim.rlim_cur, (intmax_t)new_rlim.rlim_cur,
 				errno, strerror(errno));
 			rc = EXIT_FAILURE;
@@ -287,23 +299,26 @@ static int stress_fsize(stress_args_t *args)
 		/*
 		 *  Test #3, work through all off_t sizes in powers of 2 - 1
 		 */
-		if (ftruncate(fd, 0) < 0) {
-			pr_inf("%s: truncating file to zero bytes failed, errno=%d (%s)\n",
-				args->name, errno, strerror(errno));
-			rc = EXIT_FAILURE;
-			break;
-		}
 		for (offset = 1; offset < max_offset; offset = (offset << 1 | 1)) {
-			if (stress_fsize_boundary(args, fd, &old_rlim, offset, 1) == EXIT_FAILURE)
+			if (ftruncate(fd, 0) < 0) {
+				pr_inf("%s: truncating file to zero bytes failed, errno=%d (%s)\n",
+					args->name, errno, strerror(errno));
 				rc = EXIT_FAILURE;
+				goto err;
+			}
+			if (stress_fsize_boundary(args, fd, &old_rlim, offset, 1) == EXIT_FAILURE) {
+				rc = EXIT_FAILURE;
+				goto err;
+			}
 		}
 		stress_bogo_inc(args);
 	} while (stress_continue(args));
 
+err:
 	duration = stress_time_now() - t;
 	rate = (duration > 0.0) ? (double)sigxfsz_count / duration : 0.0;
 	stress_metrics_set(args, 0, "SIGXFSZ signals per sec",
-		rate, STRESS_HARMONIC_MEAN);
+		rate, STRESS_METRIC_HARMONIC_MEAN);
 
 	stress_set_proc_state(args->name, STRESS_STATE_DEINIT);
 	if (fd != -1)
@@ -313,16 +328,16 @@ static int stress_fsize(stress_args_t *args)
 	return rc;
 }
 
-stressor_info_t stress_fsize_info = {
+const stressor_info_t stress_fsize_info = {
 	.stressor = stress_fsize,
-	.class = CLASS_FILESYSTEM | CLASS_OS,
+	.classifier = CLASS_FILESYSTEM | CLASS_OS,
 	.verify = VERIFY_ALWAYS,
 	.help = help
 };
 #else
-stressor_info_t stress_fsize_info = {
+const stressor_info_t stress_fsize_info = {
 	.stressor = stress_unimplemented,
-	.class = CLASS_FILESYSTEM | CLASS_OS,
+	.classifier = CLASS_FILESYSTEM | CLASS_OS,
 	.verify = VERIFY_ALWAYS,
 	.help = help,
 	.unimplemented_reason = "built without fallocate(), RLIMIT_FSIZE or SIGXFSZ"
