@@ -207,7 +207,10 @@ static ssize_t stress_hdd_write(
 #endif
 	errno = 0;
 	if (hdd_flags & HDD_OPT_IOVEC) {
-#if defined(HAVE_SYS_UIO_H)
+#if defined(HAVE_SYS_UIO_H) &&	\
+    (defined(HAVE_PWRITEV2) ||	\
+     defined(HAVE_PWRITEV) ||	\
+     defined(HAVE_WRITEV))
 		struct iovec iov[HDD_IO_VEC_MAX];
 		size_t i;
 		uint8_t *data = buf;
@@ -335,7 +338,10 @@ static ssize_t stress_hdd_read(
 
 	errno = 0;
 	if (hdd_flags & HDD_OPT_IOVEC) {
-#if defined(HAVE_SYS_UIO_H)
+#if defined(HAVE_SYS_UIO_H) &&	\
+    (defined(HAVE_PREADV2) ||	\
+     defined(HAVE_PREADV) ||	\
+     defined(HAVE_READV))
 		struct iovec iov[HDD_IO_VEC_MAX];
 		size_t i;
 		uint8_t *data = buf;
@@ -412,14 +418,20 @@ static ssize_t stress_hdd_read(
  */
 static void stress_hdd_invalid_read(const int fd, uint8_t *buf)
 {
-#if defined(HAVE_SYS_UIO_H)
+#if defined(HAVE_SYS_UIO_H) &&	\
+    (defined(HAVE_PREADV2) ||	\
+     defined(HAVE_PREADV) ||	\
+     defined(HAVE_READV))
 	struct iovec iov[HDD_IO_VEC_MAX];
 	size_t i;
 	uint8_t *data = buf;
 #endif
 	(void)fd;
 
-#if defined(HAVE_SYS_UIO_H)
+#if defined(HAVE_SYS_UIO_H) &&	\
+    (defined(HAVE_PREADV2) ||	\
+     defined(HAVE_PREADV) ||	\
+     defined(HAVE_READV))
 	for (i = 0; i < HDD_IO_VEC_MAX; i++) {
 		iov[i].iov_base = (void *)data;
 		iov[i].iov_len = (size_t)1;
@@ -468,14 +480,20 @@ static void stress_hdd_invalid_read(const int fd, uint8_t *buf)
  */
 static void stress_hdd_invalid_write(const int fd, uint8_t *buf)
 {
-#if defined(HAVE_SYS_UIO_H)
+#if defined(HAVE_SYS_UIO_H) &&	\
+    (defined(HAVE_PWRITEV2) ||	\
+     defined(HAVE_PWRITEV) ||	\
+     defined(HAVE_WRITEV))
 	struct iovec iov[HDD_IO_VEC_MAX];
 	size_t i;
 	uint8_t *data = buf;
 #endif
 	(void)fd;
 
-#if defined(HAVE_SYS_UIO_H)
+#if defined(HAVE_SYS_UIO_H) &&	\
+    (defined(HAVE_PWRITEV2) ||	\
+     defined(HAVE_PWRITEV) ||	\
+     defined(HAVE_WRITEV))
 	for (i = 0; i < HDD_IO_VEC_MAX; i++) {
 		iov[i].iov_base = (void *)data;
 		iov[i].iov_len = (size_t)1;
@@ -553,9 +571,9 @@ static void stress_hdd_opts(const char *opt_name, const char *opt_arg, stress_ty
 				const int exclude = hdd_flags & hdd_opts[i].exclude;
 
 				if (exclude) {
-					int j;
+					size_t j;
 
-					for (j = 0; hdd_opts[j].opt; j++) {
+					for (j = 0; j < SIZEOF_ARRAY(hdd_opts); j++) {
 						if ((exclude & hdd_opts[j].flag) == exclude) {
 							(void)fprintf(stderr,
 								"%s option '%s' is not "
@@ -629,7 +647,7 @@ static int stress_hdd_advise(stress_args_t *args, const int fd, const int flags)
  *  data_value()
  *	generate 8 bit data value for offsets and instance # into a test file
  */
-static inline ALWAYS_INLINE uint8_t PURE OPTIMIZE3 data_value(
+static inline ALWAYS_INLINE uint8_t CONST OPTIMIZE3 data_value(
 	const uint64_t i,
 	uint64_t j,
 	const uint32_t instance)
@@ -1023,7 +1041,7 @@ seq_rd_retry:
 			uint64_t baddata = 0;
 
 			for (i = 0; i < hdd_bytes_max; i += hdd_write_size) {
-				const size_t offset = (hdd_bytes > hdd_write_size) ?
+				const uint64_t offset = (hdd_bytes > hdd_write_size) ?
 					stress_mwc64modn(hdd_bytes - hdd_write_size) & ~511UL : 0;
 
 				if (lseek(fd, (off_t)offset, SEEK_SET) < 0) {
