@@ -22,6 +22,7 @@
 #include "core-asm-ret.h"
 #include "core-builtin.h"
 #include "core-cpu-cache.h"
+#include "core-mmap.h"
 
 static const stress_help_t help[] = {
 	{ NULL,	"icache N",	"start N CPU instruction cache thrashing workers" },
@@ -112,6 +113,7 @@ static int stress_icache_func(stress_args_t *args, void *page, const size_t page
 			 */
 			val = *vaddr;
 			*vaddr ^= ~0;
+
 			/*
 			 * ARM CPUs need us to clear the I$ between
 			 * each modification of the object code.
@@ -158,8 +160,9 @@ static int stress_icache(stress_args_t *args)
 			PROT_READ | PROT_WRITE | PROT_EXEC,
 			MAP_ANONYMOUS | MAP_SHARED, -1, 0);
 	if (page == MAP_FAILED) {
-		pr_inf_skip("%s: could not mmap %zd sized page, skipping stressor\n",
-			args->name, page_size);
+		pr_inf_skip("%s: could not mmap %zu sized page%s, errno=%d (%s), skipping stressor\n",
+			args->name, page_size, stress_get_memfree_str(),
+			errno, strerror(errno));
 		return EXIT_NO_RESOURCE;
 	}
 	stress_set_vma_anon_name(page, page_size, "opcodes");
@@ -177,14 +180,14 @@ static int stress_icache(stress_args_t *args)
 
 const stressor_info_t stress_icache_info = {
 	.stressor = stress_icache,
-	.class = CLASS_CPU_CACHE,
+	.classifier = CLASS_CPU_CACHE,
 	.supported = stress_asm_ret_supported,
 	.help = help
 };
 #else
 const stressor_info_t stress_icache_info = {
 	.stressor = stress_unimplemented,
-	.class = CLASS_CPU_CACHE,
+	.classifier = CLASS_CPU_CACHE,
 	.supported = stress_asm_ret_supported,
 	.help = help,
 	.unimplemented_reason = "built without mprotect() or userspace instruction cache flushing support"
