@@ -315,7 +315,7 @@ redo:
 		}
 
 		/* Scan through inotify events */
-		while ((i >= 0) && (i <= len - (ssize_t)sizeof(struct inotify_event))) {
+		while (i <= len - (ssize_t)sizeof(struct inotify_event)) {
 			struct inotify_event *event =
 				(struct inotify_event *)&buffer[i];
 			uint32_t f = event->mask & (IN_DELETE_SELF |
@@ -1027,14 +1027,14 @@ static int stress_inotify(stress_args_t *args)
 
 const stressor_info_t stress_inotify_info = {
 	.stressor = stress_inotify,
-	.class = CLASS_FILESYSTEM | CLASS_OS,
+	.classifier = CLASS_FILESYSTEM | CLASS_OS,
 	.verify = VERIFY_OPTIONAL,
 	.help = help
 };
 #else
 const stressor_info_t stress_inotify_info = {
 	.stressor = stress_unimplemented,
-	.class = CLASS_FILESYSTEM | CLASS_OS,
+	.classifier = CLASS_FILESYSTEM | CLASS_OS,
 	.verify = VERIFY_OPTIONAL,
 	.help = help,
 	.unimplemented_reason = "built without sys/epoll.h, sys/inotify.h, inotify(), inotify1() or select() support"
