@@ -41,8 +41,7 @@ static const stress_opt_t opts[] = {
 #if defined(HAVE_GETITIMER) &&	\
     defined(HAVE_SETITIMER)
 
-static volatile uint64_t itimer_counter = 0;
-static uint64_t max_ops;
+static stress_args_t *s_args;
 static double rate_us;
 static double time_end;
 
@@ -87,43 +86,29 @@ static void stress_itimer_set(struct itimerval *timer)
 	timer->it_interval.tv_usec = timer->it_value.tv_usec;
 }
 
-/*
- *  stress_itimer_stress_continue(args)
- *      returns true if we can keep on running a stressor
- */
-static bool OPTIMIZE3 stress_itimer_stress_continue(void)
-{
-	return (LIKELY(stress_continue_flag()) &&
-		LIKELY(!max_ops || (itimer_counter < max_ops)));
-}
-
 /*
  *  stress_itimer_handler()
  *	catch itimer signal and cancel if no more runs flagged
  */
-static void stress_itimer_handler(int sig)
+static void OPTIMIZE3 stress_itimer_handler(int sig)
 {
 	struct itimerval timer;
 	sigset_t mask;
 
 	(void)sig;
 
-	if (!stress_itimer_stress_continue())
-		goto cancel;
-	itimer_counter++;
-
 	if (sigpending(&mask) == 0)
 		if (sigismember(&mask, SIGINT))
 			goto cancel;
+
+	if (LIKELY(!stress_continue(s_args)))
+		goto cancel;
+	stress_bogo_inc(s_args);
 	/* High freq timer, check periodically for timeout */
-	if ((itimer_counter & 65535) == 0)
+	if ((stress_bogo_get(s_args) & 65535) == 0)
 		if (stress_time_now() > time_end)
 			goto cancel;
-	if (LIKELY(stress_continue_flag())) {
-		stress_itimer_set(&timer);
-		return;
-	}
-
+	return;
 cancel:
 	stress_continue_set_flag(false);
 	/* Cancel timer if we detect no more runs */
@@ -141,14 +126,13 @@ static int stress_itimer(stress_args_t *args)
 	sigset_t mask;
 	uint64_t itimer_freq = DEFAULT_ITIMER_FREQ;
 
+	s_args = args;
 	time_end = args->time_end;
 
 	(void)sigemptyset(&mask);
 	(void)sigaddset(&mask, SIGINT);
 	(void)sigprocmask(SIG_SETMASK, &mask, NULL);
 
-	max_ops = args->bogo.max_ops;
-
 	if (!stress_get_setting("itimer-freq", &itimer_freq)) {
 		if (g_opt_flags & OPT_FLAGS_MAXIMIZE)
 			itimer_freq = MAX_ITIMER_FREQ;
@@ -167,7 +151,7 @@ static int stress_itimer(stress_args_t *args)
 	stress_itimer_set(&timer);
 	if (setitimer(ITIMER_PROF, &timer, NULL) < 0) {
 		if (errno == EINVAL) {
-			if (args->instance == 0)
+			if (stress_instance_zero(args))
 				pr_inf_skip("%s: skipping stressor, setitimer with "
 					"ITIMER_PROF is not implemented\n",
 					args->name);
@@ -187,11 +171,9 @@ static int stress_itimer(stress_args_t *args)
 		for (i = 0; i < SIZEOF_ARRAY(stress_itimers); i++) {
 			(void)getitimer(stress_itimers[i], &t);
 		}
-
-		stress_bogo_set(args, itimer_counter);
 	} while (stress_continue(args));
 
-	if (itimer_counter == 0) {
+	if (stress_bogo_get(args) == 0) {
 		pr_fail("%s: did not handle any itimer SIGPROF signals\n",
 			args->name);
 	}
@@ -203,10 +185,9 @@ static int stress_itimer(stress_args_t *args)
 	return EXIT_SUCCESS;
 }
 
-
 const stressor_info_t stress_itimer_info = {
 	.stressor = stress_itimer,
-	.class = CLASS_INTERRUPT | CLASS_OS,
+	.classifier = CLASS_INTERRUPT | CLASS_OS,
 	.opts = opts,
 	.verify = VERIFY_ALWAYS,
 	.help = help
@@ -216,7 +197,7 @@ const stressor_info_t stress_itimer_info = {
 
 const stressor_info_t stress_itimer_info = {
 	.stressor = stress_unimplemented,
-	.class = CLASS_SIGNAL | CLASS_INTERRUPT | CLASS_OS,
+	.classifier = CLASS_INTERRUPT | CLASS_OS,
 	.opts = opts,
 	.verify = VERIFY_ALWAYS,
 	.help = help,
