@@ -1,6 +1,6 @@
 /*
  * Copyright (C) 2013-2021 Canonical, Ltd.
- * Copyright (C) 2022-2023 Colin Ian King.
+ * Copyright (C) 2022-2025 Colin Ian King.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -39,23 +39,9 @@ static const stress_help_t help[] = {
 	{ NULL,	NULL,		NULL }
 };
 
-/*
- *  stress_set_judy_size()
- *      set judy size from given option string
- */
-static int stress_set_judy_size(const char *opt)
-{
-	uint64_t judy_size;
-
-	judy_size = stress_get_uint64(opt);
-	stress_check_range("judy-size", judy_size,
-		MIN_JUDY_SIZE, MAX_JUDY_SIZE);
-	return stress_set_setting("judy-size", TYPE_ID_UINT64, &judy_size);
-}
-
-static const stress_opt_set_func_t opt_set_funcs[] = {
-	{ OPT_judy_size,	stress_set_judy_size },
-	{ 0,			NULL }
+static const stress_opt_t opts[] = {
+	{ OPT_judy_size, "judy-size", TYPE_ID_UINT64, MIN_JUDY_SIZE, MAX_JUDY_SIZE, NULL },
+	END_OPT,
 };
 
 #if defined(HAVE_JUDY_H) && \
@@ -64,7 +50,7 @@ static const stress_opt_set_func_t opt_set_funcs[] = {
  *  generate a unique large index position into a Judy array
  *  from a known small index
  */
-static inline Word_t gen_index(const Word_t idx)
+static inline OPTIMIZE3 Word_t gen_index(const Word_t idx)
 {
 	return ((~idx & 0xff) << 24) | (idx & 0x00ffffff);
 }
@@ -73,13 +59,15 @@ static inline Word_t gen_index(const Word_t idx)
  *  stress_judy()
  *	stress a judy array, exercises cache/memory
  */
-static int stress_judy(const stress_args_t *args)
+static int OPTIMIZE3 stress_judy(stress_args_t *args)
 {
 	uint64_t judy_size = DEFAULT_JUDY_SIZE;
 	size_t n;
-	Word_t i, j;
+	register Word_t i, j;
 	double duration[JUDY_OP_MAX], count[JUDY_OP_MAX];
 	size_t k;
+	const bool verify = !!(g_opt_flags & OPT_FLAGS_VERIFY);
+	int rc = EXIT_FAILURE;
 
 	static const char * const judy_ops[] = {
 		"insert",
@@ -100,12 +88,14 @@ static int stress_judy(const stress_args_t *args)
 		count[k] = 0.0;
 	}
 
+	stress_set_proc_state(args->name, STRESS_STATE_SYNC_WAIT);
+	stress_sync_start_wait(args);
 	stress_set_proc_state(args->name, STRESS_STATE_RUN);
 
 	do {
 		Pvoid_t PJLArray = (Pvoid_t)NULL;
 		Word_t *pvalue;
-		int rc;
+		int judyrc;
 		double t;
 
 		/* Step #1, populate Judy array in sparse index order */
@@ -114,11 +104,13 @@ static int stress_judy(const stress_args_t *args)
 			Word_t idx = gen_index(i);
 
 			JLI(pvalue, PJLArray, idx);
-			if ((pvalue == NULL) || (pvalue == PJERR)) {
+			if (UNLIKELY((pvalue == NULL) || (pvalue == PJERR))) {
 				pr_err("%s: cannot allocate new "
-					"judy node\n", args->name);
+					"judy node%s\n", args->name,
+					stress_get_memfree_str());
 				for (j = 0; j < n; j++) {
-					JLD(rc, PJLArray, idx);
+					idx = gen_index(j);
+					JLD(judyrc, PJLArray, idx);
 				}
 				goto abort;
 			}
@@ -129,22 +121,25 @@ static int stress_judy(const stress_args_t *args)
 
 		/* Step #2, find */
 		t = stress_time_now();
-		for (i = 0; keep_stressing_flag() && i < n; i++) {
-			Word_t idx = gen_index(i);
+		for (i = 0; stress_continue_flag() && (i < n); i++) {
+			const Word_t idx = gen_index(i);
 
 			JLG(pvalue, PJLArray, idx);
-			if (g_opt_flags & OPT_FLAGS_VERIFY) {
-				if (!pvalue) {
+			if (UNLIKELY(verify)) {
+				if (UNLIKELY(!pvalue)) {
 					pr_fail("%s: element %" PRIu32
 						"could not be found\n",
 						args->name, (uint32_t)idx);
+					goto abort;
 				} else {
-					if ((uint32_t)*pvalue != i)
+					if (UNLIKELY((uint32_t)*pvalue != i)) {
 						pr_fail("%s: element "
 							"%" PRIu32 " found %" PRIu32
 							", expecting %" PRIu32 "\n",
 							args->name, (uint32_t)idx,
 							(uint32_t)*pvalue, (uint32_t)i);
+						goto abort;
+					}
 				}
 			}
 		}
@@ -154,44 +149,48 @@ static int stress_judy(const stress_args_t *args)
 		/* Step #3, delete, reverse index order */
 		t = stress_time_now();
 		for (j = n -1, i = 0; i < n; i++, j--) {
-			Word_t idx = gen_index(j);
+			const Word_t idx = gen_index(j);
 
-			JLD(rc, PJLArray, idx);
-			if ((g_opt_flags & OPT_FLAGS_VERIFY) && (rc != 1))
+			JLD(judyrc, PJLArray, idx);
+			if (UNLIKELY(verify && (judyrc != 1))) {
 				pr_fail("%s: element %" PRIu32 " could not "
 					"be found\n", args->name, (uint32_t)idx);
+				goto abort;
+			}
 		}
 		duration[JUDY_OP_DELETE] += stress_time_now() - t;
 		count[JUDY_OP_DELETE] += n;
 
-		inc_counter(args);
-	} while (keep_stressing(args));
+		stress_bogo_inc(args);
+	} while (stress_continue(args));
 
+	rc = EXIT_SUCCESS;
 abort:
 	for (k = 0; k < JUDY_OP_MAX; k++) {
 		char msg[64];
 		const double rate = (duration[k] > 0.0) ? count[k] / duration[k] : 0.0;
 
 		(void)snprintf(msg, sizeof(msg), "Judy %s operations per sec", judy_ops[k]);
-		stress_metrics_set(args, k, msg, rate);
+		stress_metrics_set(args, k, msg, rate, STRESS_METRIC_HARMONIC_MEAN);
 	}
 	stress_set_proc_state(args->name, STRESS_STATE_DEINIT);
 
-	return EXIT_SUCCESS;
+	return rc;
 }
 
-stressor_info_t stress_judy_info = {
+const stressor_info_t stress_judy_info = {
 	.stressor = stress_judy,
-	.class = CLASS_CPU_CACHE | CLASS_CPU | CLASS_MEMORY,
-	.opt_set_funcs = opt_set_funcs,
+	.classifier = CLASS_CPU_CACHE | CLASS_CPU | CLASS_MEMORY,
+	.opts = opts,
 	.verify = VERIFY_OPTIONAL,
 	.help = help
 };
 #else
-stressor_info_t stress_judy_info = {
+const stressor_info_t stress_judy_info = {
 	.stressor = stress_unimplemented,
-	.class = CLASS_CPU_CACHE | CLASS_CPU | CLASS_MEMORY,
-	.opt_set_funcs = opt_set_funcs,
+	.classifier = CLASS_CPU_CACHE | CLASS_CPU | CLASS_MEMORY,
+	.opts = opts,
+	.verify = VERIFY_OPTIONAL,
 	.help = help,
 	.unimplemented_reason = "built without Judy.h or Judy library support"
 };
