@@ -186,6 +186,7 @@ static int stress_memfd_child(stress_args_t *args, void *context)
 	char filename_rndstr[64], filename_unusual[64], filename_pid[64];
 #if defined(HAVE_LINUX_MEMPOLICY_H)
 	stress_numa_mask_t *numa_mask = NULL;
+	stress_numa_mask_t *numa_nodes = NULL;
 #endif
 
 	stress_catch_sigill();
@@ -199,7 +200,7 @@ static int stress_memfd_child(stress_args_t *args, void *context)
 
 #if !defined(HAVE_MADVISE)
 	if (memfd_madvise) {
-		if (args->instance == 0) {
+		if (stress_instance_zero(args)) {
 			pr_inf("%s: disabling --memfd-madvise, madvise() "
 				"not supported\n", args->name);
 		}
@@ -210,7 +211,7 @@ static int stress_memfd_child(stress_args_t *args, void *context)
 #if !defined(HAVE_MADVISE) ||	\
     !defined(MADV_PAGEOUT)
 	if (memfd_zap_pte) {
-		if (args->instance == 0) {
+		if (stress_instance_zero(args)) {
 			pr_inf("%s: disabling --memfd-zap-pte, madvise() "
 				"with MADV_PAGEOUT not supported\n", args->name);
 		}
@@ -250,22 +251,9 @@ static int stress_memfd_child(stress_args_t *args, void *context)
 
 	if (memfd_numa) {
 #if defined(HAVE_LINUX_MEMPOLICY_H)
-		if (stress_numa_nodes() > 1) {
-			numa_mask = stress_numa_mask_alloc();
-			if (!numa_mask) {
-				pr_inf("%s: cannot allocate NUMA mask, disabling --memfd-numa\n",
-					args->name);
-				memfd_numa = false;
-			}
-		} else {
-			if (args->instance == 0) {
-				pr_inf("%s: only 1 NUMA node available, disabling --memfd-numa\n",
-					args->name);
-				memfd_numa = false;
-			}
-		}
+		stress_numa_mask_and_node_alloc(args, &numa_nodes, &numa_mask, "--memfd-numa", &memfd_numa);
 #else
-		if (args->instance == 0)
+		if (stress_instance_zero(args))
 			pr_inf("%s: --memfd-numa selected but not supported by this system, disabling option\n",
 				args->name);
 		memfd_numa = false;
@@ -370,12 +358,12 @@ static int stress_memfd_child(stress_args_t *args, void *context)
 			if (memfd_mlock)
 				(void)shim_mlock(maps[i], size);
 #if defined(HAVE_LINUX_MEMPOLICY_H)
-			if (memfd_numa && numa_mask)
-				stress_numa_randomize_pages(numa_mask, maps[i], size, page_size);
+			if (memfd_numa && numa_mask && numa_nodes)
+				stress_numa_randomize_pages(args, numa_nodes, numa_mask, maps[i], size, page_size);
 #endif
 			stress_memfd_fill_pages_generic(stress_mwc64(), maps[i], size);
 			if (memfd_madvise) {
-				(void)stress_madvise_random(maps[i], size);
+				(void)stress_madvise_randomize(maps[i], size);
 				(void)stress_madvise_mergeable(maps[i], size);
 			}
 
@@ -393,7 +381,7 @@ static int stress_memfd_child(stress_args_t *args, void *context)
 			 *  ..and allocate space, this should fill file with zeros
 			 *  and kernel compaction should kick in.
 			 */
-			VOID_RET(ssize_t, shim_fallocate(fds[i], 0, (off_t)size, 0));
+			VOID_RET(ssize_t, shim_fallocate(fds[i], 0, 0, (off_t)size));
 
 			if (UNLIKELY(!stress_continue_flag()))
 				goto memfd_unmap;
@@ -534,11 +522,6 @@ buf_unmap:
 		if (fd >= 0)
 			(void)close(fd);
 
-		/* Exercise illegal path / in name */
-		fd = shim_memfd_create("/path/in/name", ~0U);
-		if (fd >= 0)
-			(void)close(fd);
-
 		/* Exercise illegal zero length name */
 		fd = shim_memfd_create("", ~0U);
 		if (fd >= 0)
@@ -567,6 +550,8 @@ buf_unmap:
 #if defined(HAVE_LINUX_MEMPOLICY_H)
 	if (numa_mask)
 		stress_numa_mask_free(numa_mask);
+	if (numa_nodes)
+		stress_numa_mask_free(numa_nodes);
 #endif
 	free(maps);
 	free(fds);
@@ -585,7 +570,7 @@ static int stress_memfd(stress_args_t *args)
 
 const stressor_info_t stress_memfd_info = {
 	.stressor = stress_memfd,
-	.class = CLASS_OS | CLASS_MEMORY,
+	.classifier = CLASS_OS | CLASS_MEMORY,
 	.opts = opts,
 	.verify = VERIFY_ALWAYS,
 	.help = help
@@ -593,7 +578,7 @@ const stressor_info_t stress_memfd_info = {
 #else
 const stressor_info_t stress_memfd_info = {
 	.stressor = stress_unimplemented,
-	.class = CLASS_OS | CLASS_MEMORY,
+	.classifier = CLASS_OS | CLASS_MEMORY,
 	.opts = opts,
 	.verify = VERIFY_ALWAYS,
 	.help = help,
