@@ -19,6 +19,7 @@
  */
 #include "stress-ng.h"
 #include "core-builtin.h"
+#include "core-capabilities.h"
 #include "core-killpid.h"
 #include "core-madvise.h"
 #include "core-mincore.h"
@@ -123,7 +124,8 @@ static int stress_mlockmany_child(stress_args_t *args, void *context)
 
 	s_pids = stress_sync_s_pids_mmap(mlockmany_procs);
 	if (s_pids == MAP_FAILED) {
-		pr_inf_skip("%s: failed to mmap %zu PIDs, skipping stressor\n", args->name, mlockmany_procs);
+		pr_inf_skip("%s: failed to mmap %zu PIDs%s, skipping stressor\n",
+			args->name, mlockmany_procs, stress_get_memfree_str());
 		return EXIT_NO_RESOURCE;
 	}
 
@@ -175,6 +177,8 @@ static int stress_mlockmany_child(stress_args_t *args, void *context)
 				void *ptr = MAP_FAILED;
 				size_t mmap_size = mlock_size;
 
+				stress_set_proc_state(args->name, STRESS_STATE_RUN);
+
 				/* In case we've missed SIGALRM */
 				if (UNLIKELY(stress_time_now() > args->time_end))
 					_exit(0);
@@ -183,7 +187,12 @@ static int stress_mlockmany_child(stress_args_t *args, void *context)
 				stress_set_oom_adjustment(args, true);
 				(void)sched_settings_apply(true);
 
+				/* exercise invalid mlockall call */
 				shim_mlockall(0);
+
+				/* unlock all mlocked memory */
+				shim_munlockall();
+
 				stress_get_memlimits(&shmall, &freemem, &totalmem, &freeswap, &totalswap);
 				/* We detected swap being used, bail out */
 				if (last_freeswap > freeswap)
@@ -252,7 +261,8 @@ unmap:
 			if (UNLIKELY(!stress_continue(args)))
 				break;
 		}
-		stress_kill_and_wait_many(args, s_pids, n, SIGALRM, false);
+		/* and kill! */
+		stress_kill_and_wait_many(args, s_pids, n, SIGKILL, false);
 	} while (stress_continue(args));
 
 	stress_set_proc_state(args->name, STRESS_STATE_DEINIT);
@@ -273,7 +283,7 @@ static int stress_mlockmany(stress_args_t *args)
 
 const stressor_info_t stress_mlockmany_info = {
 	.stressor = stress_mlockmany,
-	.class = CLASS_VM | CLASS_OS | CLASS_PATHOLOGICAL,
+	.classifier = CLASS_VM | CLASS_OS | CLASS_PATHOLOGICAL,
 	.opts = opts,
 	.help = help
 };
@@ -282,7 +292,7 @@ const stressor_info_t stress_mlockmany_info = {
 
 const stressor_info_t stress_mlockmany_info = {
 	.stressor = stress_unimplemented,
-	.class = CLASS_VM | CLASS_OS | CLASS_PATHOLOGICAL,
+	.classifier = CLASS_VM | CLASS_OS | CLASS_PATHOLOGICAL,
 	.opts = opts,
 	.help = help,
 	.unimplemented_reason = "built without mlock() support"
