@@ -1,6 +1,6 @@
 /*
  * Copyright (C) 2017-2021 Canonical, Ltd.
- * Copyright (C)      2022 Colin Ian King.
+ * Copyright (C) 2022-2025 Colin Ian King.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -18,7 +18,9 @@
  *
  */
 #include "stress-ng.h"
+#include "core-builtin.h"
 #include "core-capabilities.h"
+#include <sys/socket.h>
 
 #if defined(HAVE_LINUX_CN_PROC_H)
 #include <linux/cn_proc.h>
@@ -74,7 +76,7 @@ static int stress_netlink_proc_supported(const char *name)
  *   monitor()
  *	monitor system activity
  */
-static int monitor(const stress_args_t *args, const int sock)
+static int monitor(stress_args_t *args, const int sock)
 {
 	struct nlmsghdr *nlmsghdr;
 	ssize_t len;
@@ -94,25 +96,31 @@ static int monitor(const stress_args_t *args, const int sock)
 	}
 
 	for (nlmsghdr = (struct nlmsghdr *)buf;
-		NLMSG_OK(nlmsghdr, (unsigned int)len);
+		(len > 0) && NLMSG_OK(nlmsghdr, (unsigned int)len);
 		nlmsghdr = NLMSG_NEXT(nlmsghdr, len)) {
 
-		struct cn_msg *cn_msg;
-		struct proc_event *proc_ev;
+		const struct cn_msg *cn_msg;
+		const struct proc_event *proc_ev;
 
-		if (!keep_stressing_flag())
+		if (UNLIKELY(!stress_continue_flag()))
 			return 0;
 
 		if ((nlmsghdr->nlmsg_type == NLMSG_ERROR) ||
 		    (nlmsghdr->nlmsg_type == NLMSG_NOOP))
 			continue;
 
+		if (nlmsghdr->nlmsg_len < NLMSG_LENGTH(sizeof(struct cn_msg)))
+			continue;
+
 		cn_msg = NLMSG_DATA(nlmsghdr);
 		if ((cn_msg->id.idx != CN_IDX_PROC) ||
 		    (cn_msg->id.val != CN_VAL_PROC))
 			continue;
 
-		proc_ev = (struct proc_event *)cn_msg->data;
+		if (cn_msg->len < sizeof(struct proc_event))
+			continue;
+
+		proc_ev = (const struct proc_event *)cn_msg->data;
 
 		switch (proc_ev->what) {
 #if LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,14)
@@ -136,7 +144,7 @@ static int monitor(const stress_args_t *args, const int sock)
 #if LINUX_VERSION_CODE >= KERNEL_VERSION(3,1,0)
 		case PROC_EVENT_PTRACE:
 #endif
-			inc_counter(args);
+			stress_bogo_inc(args);
 			break;
 		default:
 			break;
@@ -158,14 +166,14 @@ static void spawn_several(const char *name, const int n, const int max)
 	if (pid == 0) {
 		char newname[128];
 
-		(void)snprintf(newname, sizeof(newname), "stress-ng-%d", n);
+		(void)snprintf(newname, sizeof(newname), "%d", n);
 		stress_set_proc_name(newname);
 
 		if (n >= max) {
-			stress_set_proc_name("stress-ng-dead");
+			stress_set_proc_name("dead");
 			_exit(0);
 		} else {
-			stress_set_proc_name("stress-ng-spawn");
+			stress_set_proc_name("spawn");
 			spawn_several(name, n + 1, max);
 		}
 	} else if (pid < 0) {
@@ -174,7 +182,7 @@ static void spawn_several(const char *name, const int n, const int max)
 		int status;
 
 		if (n != 0)
-			stress_set_proc_name("stress-ng-wait");
+			stress_set_proc_name("wait");
 		(void)shim_waitpid(pid, &status, 0);
 		if (n != 0)
 			_exit(0);
@@ -185,7 +193,7 @@ static void spawn_several(const char *name, const int n, const int max)
  *  stress_netlink_proc()
  *	stress netlink proc events
  */
-static int stress_netlink_proc(const stress_args_t *args)
+static int stress_netlink_proc(stress_args_t *args)
 {
 	int sock = -1;
 	struct sockaddr_nl addr;
@@ -200,31 +208,37 @@ static int stress_netlink_proc(const stress_args_t *args)
 				args->name, errno, strerror(errno));
 			return EXIT_NO_RESOURCE;
 		}
-		pr_fail("%s: socket failed: errno=%d (%s)\n",
+		pr_fail("%s: socket failed, errno=%d (%s)\n",
 			args->name, errno, strerror(errno));
 		return EXIT_FAILURE;
 	}
 
-	(void)memset(&addr, 0, sizeof(addr));
+	(void)shim_memset(&addr, 0, sizeof(addr));
 	addr.nl_pid = (uint32_t)getpid();
 	addr.nl_family = AF_NETLINK;
 	addr.nl_groups = CN_IDX_PROC;
 
 	if (bind(sock, (struct sockaddr *)&addr, sizeof(addr)) < 0) {
-		pr_err("%s: bind failed: errno=%d (%s)\n",
+		if (errno == EPERM) {
+			pr_inf_skip("%s: bind failed, no permission, "
+				"skipping stressor\n", args->name);
+			(void)close(sock);
+			return EXIT_NO_RESOURCE;
+		}
+		pr_err("%s: bind failed, errno=%d (%s)\n",
 			args->name, errno, strerror(errno));
 		(void)close(sock);
 		return EXIT_FAILURE;
 	}
 
-	(void)memset(&nlmsghdr, 0, sizeof(nlmsghdr));
+	(void)shim_memset(&nlmsghdr, 0, sizeof(nlmsghdr));
 	nlmsghdr.nlmsg_len = NLMSG_LENGTH(sizeof(cn_msg) + sizeof(op));
 	nlmsghdr.nlmsg_pid = (uint32_t)getpid();
 	nlmsghdr.nlmsg_type = NLMSG_DONE;
 	iov[0].iov_base = &nlmsghdr;
 	iov[0].iov_len = sizeof(nlmsghdr);
 
-	(void)memset(&cn_msg, 0, sizeof(cn_msg));
+	(void)shim_memset(&cn_msg, 0, sizeof(cn_msg));
 	cn_msg.id.idx = CN_IDX_PROC;
 	cn_msg.id.val = CN_VAL_PROC;
 	cn_msg.len = sizeof(enum proc_cn_mcast_op);
@@ -237,24 +251,26 @@ static int stress_netlink_proc(const stress_args_t *args)
 
 	if (writev(sock, iov, 3) < 0) {
 		if (errno == ECONNREFUSED) {
-			pr_inf_skip("%s: net link write failed, errno=%d (%s), skipping stress test\n",
+			pr_inf_skip("%s: net link write failed, errno=%d (%s), skipping stressor\n",
 				args->name, errno, strerror(errno));
 			(void)close(sock);
 			return EXIT_NO_RESOURCE;
 		}
-		pr_fail("%s: writev failed: errno=%d (%s)\n",
+		pr_fail("%s: writev failed, errno=%d (%s)\n",
 			args->name, errno, strerror(errno));
 		(void)close(sock);
 		return EXIT_FAILURE;
 	}
 
+	stress_set_proc_state(args->name, STRESS_STATE_SYNC_WAIT);
+	stress_sync_start_wait(args);
 	stress_set_proc_state(args->name, STRESS_STATE_RUN);
 
 	do {
 		spawn_several(args->name, 0, 5);
 		if (monitor(args, sock) < 0)
 			break;
-	} while (keep_stressing(args));
+	} while (stress_continue(args));
 
 	stress_set_proc_state(args->name, STRESS_STATE_DEINIT);
 
@@ -263,16 +279,17 @@ static int stress_netlink_proc(const stress_args_t *args)
 	return EXIT_SUCCESS;
 }
 
-stressor_info_t stress_netlink_proc_info = {
+const stressor_info_t stress_netlink_proc_info = {
 	.stressor = stress_netlink_proc,
 	.supported = stress_netlink_proc_supported,
-	.class = CLASS_SCHEDULER | CLASS_OS,
+	.classifier = CLASS_SCHEDULER | CLASS_OS,
 	.help = help
 };
 #else
-stressor_info_t stress_netlink_proc_info = {
-	.stressor = stress_not_implemented,
-	.class = CLASS_SCHEDULER | CLASS_OS,
-	.help = help
+const stressor_info_t stress_netlink_proc_info = {
+	.stressor = stress_unimplemented,
+	.classifier = CLASS_SCHEDULER | CLASS_OS,
+	.help = help,
+	.unimplemented_reason = "built without linux/connector.h, linux/netlink.h or linux/cn_proc.h support"
 };
 #endif
