@@ -21,7 +21,9 @@
 #include "core-arch.h"
 #include "core-builtin.h"
 #include "core-bitops.h"
+#include "core-capabilities.h"
 #include "core-killpid.h"
+#include "core-mmap.h"
 #include "core-pragma.h"
 
 #include <math.h>
@@ -188,8 +190,10 @@ static void MLOCKED_TEXT stress_opcode_child_sighandler(int signum)
 	 *  try to continue for less significant signals
 	 *  such as SIGFPE and SIGTRAP
 	 */
-	if (jmp_env_set)
+	if (jmp_env_set) {
 		siglongjmp(jmp_env, 1);
+		stress_no_return();
+	}
 #else
 	_exit(1);
 #endif
@@ -252,7 +256,7 @@ PRAGMA_UNROLL_N(8)
 }
 
 #if !defined(STRESS_OPCODE_SIZE)
-/* Not defimed? Default to 64 bit */
+/* Not defined? Default to 64 bit */
 #define STRESS_OPCODE_SIZE	(64)
 #define STRESS_OPCODE_MASK	(0xffffffffffffffffULL)
 #endif
@@ -287,9 +291,9 @@ static void OPTIMIZE3 stress_opcode_inc(
 		break;
 	default:
 	case 32: {
-			register const uint32_t tmp32 = *op & 0xffffffffL;
+			register const uint32_t tmp32 = *op & 0xffffffffU;
 			register uint32_t *ops = (uint32_t *)ops_begin;
-			register size_t i = (ssize_t)(page_size >> 2);
+			register ssize_t i = (ssize_t)(page_size >> 2);
 
 			while (i--) {
 				*(ops++) = tmp32;
@@ -300,7 +304,7 @@ static void OPTIMIZE3 stress_opcode_inc(
 		{
 			register const uint64_t tmp64 = *op;
 			register uint8_t *ops = (uint8_t *)ops_begin;
-			register size_t i = (ssize_t)(page_size / 6);
+			register ssize_t i = (ssize_t)(page_size / 6);
 
 			while (i--) {
 				*(ops++) = (tmp64 >> 0);
@@ -318,7 +322,7 @@ static void OPTIMIZE3 stress_opcode_inc(
 	case 64: {
 			register const uint64_t tmp64 = *op;
 			register uint64_t *ops = (uint64_t *)ops_begin;
-			register size_t i = (ssize_t)(page_size >> 3);
+			register ssize_t i = (ssize_t)(page_size >> 3);
 
 			while (i--)
 				*(ops++) = tmp64;
@@ -431,9 +435,10 @@ static int stress_opcode(stress_args_t *args)
 			PROT_READ | PROT_WRITE,
 			MAP_ANONYMOUS | MAP_SHARED, -1, 0);
 	if (state == MAP_FAILED) {
-		pr_inf_skip("%s: mmap of %zu bytes failed, errno=%d (%s) "
+		pr_inf_skip("%s: mmap of %zu bytes failed%s, errno=%d (%s) "
 			"skipping stressor\n",
-			args->name, args->page_size, errno, strerror(errno));
+			args->name, args->page_size, stress_get_memfree_str(),
+			errno, strerror(errno));
 		return EXIT_NO_RESOURCE;
 	}
 	stress_set_vma_anon_name(state, sizeof(*state), "state");
@@ -443,8 +448,9 @@ static int stress_opcode(stress_args_t *args)
 				PROT_READ | PROT_WRITE,
 				MAP_PRIVATE | MAP_ANONYMOUS, -1, 0);
 	if (opcodes == MAP_FAILED) {
-		pr_fail("%s: mmap failed, errno=%d (%s)\n",
-			args->name, errno, strerror(errno));
+		pr_fail("%s: mmap of %zu bytes failed%s, errno=%d (%s)\n",
+			args->name, page_size * (PAGES + 2),
+			stress_get_memfree_str(), errno, strerror(errno));
 		(void)munmap((void *)state, sizeof(*state));
 		return EXIT_NO_RESOURCE;
 	}
@@ -498,6 +504,7 @@ again:
 			void *ops_end = (void *)((uint8_t *)ops_begin + ops_size);
 			NOCLOBBER void *ops_ptr;
 
+			stress_set_proc_state(args->name, STRESS_STATE_RUN);
 			(void)sched_settings_apply(true);
 
 			for (i = 0; i < SIZEOF_ARRAY(sigs); i++) {
@@ -620,8 +627,8 @@ finish:
 	rc = EXIT_SUCCESS;
 
 	if ((duration > 0.0) &&
-	    (vstate->ops_attempted > 0.0) &&
-	    (args->instance == 0) &&
+	    (vstate->ops_attempted > 0) &&
+	    stress_instance_zero(args) &&
 	    (args->instances > 0)) {
 		const double secs_in_tropical_year = 365.2422 * 24.0 * 60.0 * 60.0;
 		double estimated_duration = (duration * num_opcodes / vstate->ops_attempted) / args->instances;
@@ -642,22 +649,22 @@ finish:
 	rate = (duration > 0.0) ? (double)vstate->ops_attempted / duration : 0.0;
 	stress_metrics_set(args, 0, "opcodes exercised per sec",
 		rate, STRESS_METRIC_HARMONIC_MEAN);
-	percent = (vstate->ops_attempted > 0.0) ? 100.0 * (double)vstate->ops_ok / (double)vstate->ops_attempted : 0.0;
+	percent = (vstate->ops_attempted > 0) ? 100.0 * (double)vstate->ops_ok / (double)vstate->ops_attempted : 0.0;
 	stress_metrics_set(args, 1, "% opcodes successfully executed",
 		percent, STRESS_METRIC_GEOMETRIC_MEAN);
-	percent = (vstate->ops_attempted > 0.0) ? 100.0 * (double)vstate->sig_count[SIGILL] / (double)vstate->ops_attempted : 0.0;
+	percent = (vstate->ops_attempted > 0) ? 100.0 * (double)vstate->sig_count[SIGILL] / (double)vstate->ops_attempted : 0.0;
 	stress_metrics_set(args, 2, "% illegal opcodes executed",
 		percent, STRESS_METRIC_GEOMETRIC_MEAN);
-	percent = (vstate->ops_attempted > 0.0) ? 100.0 * (double)vstate->sig_count[SIGBUS] / (double)vstate->ops_attempted : 0.0;
+	percent = (vstate->ops_attempted > 0) ? 100.0 * (double)vstate->sig_count[SIGBUS] / (double)vstate->ops_attempted : 0.0;
 	stress_metrics_set(args, 3, "% opcodes generated SIGBUS",
 		percent, STRESS_METRIC_GEOMETRIC_MEAN);
-	percent = (vstate->ops_attempted > 0.0) ? 100.0 * (double)vstate->sig_count[SIGSEGV] / (double)vstate->ops_attempted : 0.0;
+	percent = (vstate->ops_attempted > 0) ? 100.0 * (double)vstate->sig_count[SIGSEGV] / (double)vstate->ops_attempted : 0.0;
 	stress_metrics_set(args, 4, "% opcodes generated SIGSEGV",
 		percent, STRESS_METRIC_GEOMETRIC_MEAN);
-	percent = (vstate->ops_attempted > 0.0) ? 100.0 * (double)vstate->sig_count[SIGFPE] / (double)vstate->ops_attempted : 0.0;
+	percent = (vstate->ops_attempted > 0) ? 100.0 * (double)vstate->sig_count[SIGFPE] / (double)vstate->ops_attempted : 0.0;
 	stress_metrics_set(args, 5, "% opcodes generated SIGFPE",
 		percent, STRESS_METRIC_GEOMETRIC_MEAN);
-	percent = (vstate->ops_attempted > 0.0) ? 100.0 * (double)vstate->sig_count[SIGTRAP] / (double)vstate->ops_attempted : 0.0;
+	percent = (vstate->ops_attempted > 0) ? 100.0 * (double)vstate->sig_count[SIGTRAP] / (double)vstate->ops_attempted : 0.0;
 	stress_metrics_set(args, 6, "% opcodes generated SIGTRAP",
 		percent, STRESS_METRIC_GEOMETRIC_MEAN);
 	rate = (duration > 0.0) ? (double)forks / duration : 0.0;
@@ -683,7 +690,7 @@ static const stress_opt_t opts[] = {
 
 const stressor_info_t stress_opcode_info = {
 	.stressor = stress_opcode,
-	.class = CLASS_CPU | CLASS_OS,
+	.classifier = CLASS_CPU | CLASS_OS,
 	.opts = opts,
 	.help = help
 };
@@ -696,7 +703,7 @@ static const stress_opt_t opts[] = {
 
 const stressor_info_t stress_opcode_info = {
 	.stressor = stress_unimplemented,
-	.class = CLASS_CPU | CLASS_OS,
+	.classifier = CLASS_CPU | CLASS_OS,
 	.opts = opts,
 	.help = help,
 	.unimplemented_reason = "built without linux/seccomp.h, linux/audit.h, linux/filter.h, sys/prctl.h or mprotect()"
