@@ -23,6 +23,7 @@
 #include "core-builtin.h"
 #include "core-capabilities.h"
 #include "core-hash.h"
+#include "core-mmap.h"
 #include "core-pthread.h"
 #include "core-put.h"
 
@@ -470,7 +471,9 @@ redo:
 		 *  Multiple randomly sized reads
 		 */
 		if (procfs_flag & PROCFS_FLAG_READ) {
-			for (i = 0; i < 4096 * PROC_BUF_SZ; i++) {
+			off_t dec;
+
+			for (i = 0; i < 4096 * PROC_BUF_SZ; ) {
 				const ssize_t sz = 1 + stress_mwc32modn((uint32_t)sizeof(buffer));
 
 				if (UNLIKELY(!stress_continue_flag()))
@@ -546,7 +549,6 @@ redo:
 			 *  exercise 13 x 5 byte reads backwards through procfs file to
 			 *  ensure we perform some weird misaligned non-word sized reads
 			 */
-			off_t dec;
 
 			pos = lseek(fd, 0, SEEK_END);
 			if (pos < 0)
@@ -573,12 +575,8 @@ mmap_test:
 			/*
 			 *  mmap it
 			 */
-			ptr = (uint8_t *)mmap(NULL, page_size, PROT_READ,
-#if defined(MAP_POPULATE)
-				MAP_POPULATE |
-#endif
-				MAP_SHARED | MAP_ANONYMOUS,
-				fd, 0);
+			ptr = (uint8_t *)stress_mmap_populate(NULL, page_size, PROT_READ,
+				MAP_SHARED | MAP_ANONYMOUS, fd, 0);
 			if (ptr != MAP_FAILED) {
 				stress_uint8_put(*ptr);
 				(void)munmap((void *)ptr, page_size);
@@ -651,7 +649,7 @@ mmap_test:
 				goto err;
 			pos = lseek(fd, 1, SEEK_SET);
 			if (pos == (off_t)-1)
-			goto err;
+				goto err;
 
 			if ((stress_time_now() - t_start) > threshold)
 				goto timeout_close;
@@ -846,7 +844,7 @@ static char *stress_random_pid(void)
 
 	mixup = stress_mwc32();
 	n = stress_proc_scandir("/proc", &dlist, NULL, mixup_sort);
-	if (!n) {
+	if (n <= 0) {
 		stress_dirent_list_free(dlist, n);
 		return path;
 	}
