@@ -152,6 +152,8 @@ static int do_quotactl(
 
 	status->tested++;
 	if (ret < 0) {
+		ret = errno;
+
 		/* Quota not enabled for this file system? */
 		switch (errno) {
 		case ENOSYS:
@@ -163,23 +165,24 @@ static int do_quotactl(
 		case EROFS:
 			/* Read-only device?- skip it */
 			status->erofs++;
-			return errno;
+			break;
 		case ENOTBLK:
 			status->enotblk++;
 			/* Not a block device? - skip it */
-			return errno;
+			break;
 		case EPERM:
 			pr_inf("%s: need CAP_SYS_ADMIN capability to "
 				"run quota stressor, aborting stress test\n",
 				args->name);
-			return errno;
+			break;
 		default:
 			status->failed++;
 			pr_fail("%s: quotactl command %s on %s (%s) failed, errno=%d (%s)\n",
 				args->name, cmdname, dev->name, dev->mount, errno, strerror(errno));
 		}
+		return ret;
 	}
-	return errno;
+	return 0;
 }
 
 /*
@@ -273,9 +276,9 @@ static int do_quotas(stress_args_t *args, stress_dev_info_t *const dev)
 		struct dqinfo dqinfo;
 
 		(void)shim_memset(&dqinfo, 0, sizeof(dqinfo));
-		VOID_RET(int, quotactl(QCMD(Q_GETQUOTA, USRQUOTA), "", 0, (caddr_t)&dqinfo));
-		VOID_RET(int, quotactl(QCMD(Q_GETQUOTA, USRQUOTA), dev->name, ~0, (caddr_t)&dqinfo));
-		VOID_RET(int, quotactl(QCMD(Q_GETQUOTA, -1), dev->name, ~0, (caddr_t)&dqinfo));
+		VOID_RET(int, quotactl(QCMD(Q_GETINFO, USRQUOTA), "", 0, (caddr_t)&dqinfo));
+		VOID_RET(int, quotactl(QCMD(Q_GETINFO, USRQUOTA), dev->name, ~0, (caddr_t)&dqinfo));
+		VOID_RET(int, quotactl(QCMD(Q_GETINFO, -1), dev->name, ~0, (caddr_t)&dqinfo));
 	}
 #endif
 #if defined(Q_SYNC)
@@ -368,13 +371,13 @@ static int stress_quota(stress_args_t *args)
 			if (devs[i].valid &&
 			    !devs[i].name &&
 			    (buf.st_rdev == devs[i].st_dev)) {
-				devs[i].name = strdup(path);
+				devs[i].name = shim_strdup(path);
 				devs[i].mount = mnts[i];
 				if (!devs[i].name) {
 					pr_err("%s: out of memory\n",
 						args->name);
 					(void)closedir(dir);
-					n_devs = i;
+					n_devs = n_mounts;
 					goto tidy;
 				}
 			}
@@ -464,8 +467,10 @@ abort:
 tidy:
 	stress_set_proc_state(args->name, STRESS_STATE_DEINIT);
 
-	for (i = 0; i < n_devs; i++)
-		free(devs[i].name);
+	for (i = 0; i < n_devs; i++) {
+		if (devs[i].name)
+			free(devs[i].name);
+	}
 
 	stress_mount_free(mnts, n_mounts);
 	return rc;
@@ -474,14 +479,14 @@ tidy:
 const stressor_info_t stress_quota_info = {
 	.stressor = stress_quota,
 	.supported = stress_quota_supported,
-	.class = CLASS_OS,
+	.classifier = CLASS_OS,
 	.verify = VERIFY_ALWAYS,
 	.help = help
 };
 #else
 const stressor_info_t stress_quota_info = {
 	.stressor = stress_unimplemented,
-	.class = CLASS_OS,
+	.classifier = CLASS_OS,
 	.verify = VERIFY_ALWAYS,
 	.help = help,
 	.unimplemented_reason = "built without sys/quota.h or only supported on Linux"
