@@ -18,6 +18,7 @@
  *
  */
 #include "stress-ng.h"
+#include "core-mmap.h"
 #include "core-vmstat.h"
 
 #if defined(HAVE_SYS_SYSMACROS_H)
@@ -184,7 +185,7 @@ static int stress_rawdev_ends(
 {
 	size_t i;
 
-	for (i = 0; i < 128; i++) {
+	for (i = 0; (i < 128) && stress_continue(args); i++) {
 		ssize_t ret;
 		off_t offset;
 		double t;
@@ -218,7 +219,6 @@ static int stress_rawdev_ends(
 			metrics->count += ret;
 			stress_bogo_inc(args);
 		}
-		stress_bogo_inc(args);
 	}
 	return 0;
 }
@@ -382,8 +382,10 @@ static int stress_rawdev(stress_args_t *args)
 
 	metrics = (stress_metrics_t *)calloc(SIZEOF_ARRAY(rawdev_methods), sizeof(*metrics));
 	if (!metrics) {
-		pr_inf_skip("%s: cannot allocate metrics table, skipping stressor\n",
-			args->name);
+		pr_inf_skip("%s: cannot allocate %zu byte metrics table%s, "
+			"skipping stressor\n",
+			args->name, SIZEOF_ARRAY(rawdev_methods),
+			stress_get_memfree_str());
 		return EXIT_NO_RESOURCE;
 	}
 
@@ -440,8 +442,8 @@ static int stress_rawdev(stress_args_t *args)
 			PROT_READ | PROT_WRITE,
 			MAP_ANONYMOUS | MAP_PRIVATE, -1, 0);
 	if (buffer == MAP_FAILED) {
-		pr_inf("%s: cannot allocate buffer of %zd bytes with %zd allocation\n",
-			args->name, blksz, mmapsz);
+		pr_inf("%s: failed to mmap buffer of %zu bytes\n",
+			args->name, mmapsz);
 		(void)close(fd);
 		free(metrics);
 		return EXIT_NO_RESOURCE;
@@ -458,8 +460,8 @@ static int stress_rawdev(stress_args_t *args)
 		return EXIT_NO_RESOURCE;
 	}
 
-	if (args->instance == 0)
-		pr_dbg("%s: exercising %s (%zd blocks of size %zd bytes)\n",
+	if (stress_instance_zero(args))
+		pr_dbg("%s: exercising %s (%zu blocks of size %zu bytes)\n",
 			args->name, devpath, blks, blksz);
 
 	stress_set_proc_state(args->name, STRESS_STATE_SYNC_WAIT);
@@ -499,7 +501,7 @@ static int stress_rawdev(stress_args_t *args)
 const stressor_info_t stress_rawdev_info = {
 	.stressor = stress_rawdev,
 	.supported = stress_rawdev_supported,
-	.class = CLASS_IO,
+	.classifier = CLASS_IO,
 	.opts = opts,
 	.verify = VERIFY_ALWAYS,
 	.help = help
@@ -507,7 +509,7 @@ const stressor_info_t stress_rawdev_info = {
 #else
 const stressor_info_t stress_rawdev_info = {
 	.stressor = stress_unimplemented,
-	.class = CLASS_IO,
+	.classifier = CLASS_IO,
 	.opts = opts,
 	.verify = VERIFY_ALWAYS,
 	.help = help,
