@@ -1,6 +1,6 @@
 /*
  * Copyright (C) 2013-2021 Canonical, Ltd.
- * Copyright (C)      2022 Colin Ian King.
+ * Copyright (C) 2022-2025 Colin Ian King.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -18,27 +18,29 @@
  *
  */
 #include "stress-ng.h"
+#include "core-affinity.h"
+#include "core-builtin.h"
 #include "core-capabilities.h"
+#include "core-killpid.h"
 #include "core-net.h"
 
-#if defined(HAVE_LINUX_UDP_H)
-#include <linux/udp.h>
-#endif
-
 #if defined(HAVE_NETINET_IP_H)
 #include <netinet/ip.h>
 #endif
 
+#if defined(HAVE_NETINET_UDP_H)
+#include <netinet/udp.h>
+#elif defined(HAVE_LINUX_UDP_H)
+#include <linux/udp.h>
+#endif
+
 #include <arpa/inet.h>
 
-#define MIN_RAWUDP_PORT		(1024)
-#define MAX_RAWUDP_PORT		(65535)
 #define DEFAULT_RAWUDP_PORT	(13000)
 
 #if !defined(SOL_UDP)
 #define SOL_UDP 	(17)
 #endif
-#define PACKET_SIZE	(2048)
 
 static const stress_help_t help[] = {
 	{ NULL, "rawudp N",	"start N workers exercising raw UDP socket I/O" },
@@ -63,39 +65,22 @@ static int stress_rawudp_supported(const char *name)
 	return 0;
 }
 
-/*
- *  stress_set_rawudp_port()
- *	set port to use
- */
-static int stress_set_rawudp_port(const char *opt)
-{
-	int port;
-
-	stress_set_net_port("rawudp-port", opt,
-		MIN_RAWUDP_PORT, MAX_RAWUDP_PORT, &port);
-	return stress_set_setting("rawudp-port", TYPE_ID_INT, &port);
-}
-
-static int stress_set_rawudp_if(const char *name)
-{
-	return stress_set_setting("rawudp-if", TYPE_ID_STR, name);
-}
-
-static const stress_opt_set_func_t opt_set_funcs[] = {
-	{ OPT_rawudp_port,	stress_set_rawudp_port },
-	{ OPT_rawudp_if,	stress_set_rawudp_if },
-	{ 0,			NULL }
+static const stress_opt_t opts[] = {
+	{ OPT_rawudp_port, "rawudp-port", TYPE_ID_INT_PORT, MIN_PORT, MAX_PORT, NULL },
+	{ OPT_rawudp_if,   "rawudp-if",   TYPE_ID_STR, 0, 0, NULL },
+	END_OPT,
 };
 
 #if defined(HAVE_LINUX_UDP_H)
 
+#define PACKET_SIZE	sizeof(struct iphdr) + sizeof(struct udphdr) + sizeof(pid_t)
+
 /*
  *  stress_rawudp_client()
  *	client sender
  */
-static void NORETURN stress_rawudp_client(
-	const stress_args_t *args,
-	const pid_t ppid,
+static void NORETURN OPTIMIZE3 stress_rawudp_client(
+	stress_args_t *args,
 	in_addr_t addr,
 	const int port)
 {
@@ -104,22 +89,23 @@ static void NORETURN stress_rawudp_client(
 	char buf[PACKET_SIZE];
 	struct iphdr *ip = (struct iphdr *)buf;
 	struct udphdr *udp = (struct udphdr *)(buf + sizeof(struct iphdr));
+	uint8_t *data = (uint8_t *)(buf + sizeof(struct iphdr) + sizeof(struct udphdr));
 	struct sockaddr_in s_in;
 	int one = 1;
 
 	stress_parent_died_alarm();
 	(void)sched_settings_apply(true);
 
-	(void)memset(buf, 0, sizeof(buf));
+	(void)shim_memset(buf, 0, sizeof(buf));
 
 	s_in.sin_family = AF_INET;
-	s_in.sin_port = (in_port_t)port;
+	s_in.sin_port = htons(port);
 	s_in.sin_addr.s_addr = addr;
 
 	ip->ihl      = 5;	/* Header length in 32 bit words */
 	ip->version  = 4;	/* IPv4 */
 	ip->tos      = stress_mwc8() & 0x1e;
-	ip->tot_len  = sizeof(struct iphdr) + sizeof(struct udphdr);
+	ip->tot_len  = htons(PACKET_SIZE);
 	ip->ttl      = 16;  	/* Not too many hops! */
 	ip->protocol = SOL_UDP;	/* UDP protocol */
 	ip->saddr = (in_addr_t)addr;
@@ -127,19 +113,19 @@ static void NORETURN stress_rawudp_client(
 
 	udp->source = htons(port);
 	udp->dest = htons(port);
-	udp->len = htons(sizeof(struct udphdr));
+	udp->len = htons(sizeof(struct udphdr) + sizeof(pid_t));
 
 	do {
 		int fd;
 		ssize_t n;
 
-		if ((fd = socket(PF_INET, SOCK_RAW, IPPROTO_UDP)) < 0) {
+		if (UNLIKELY((fd = socket(PF_INET, SOCK_RAW, IPPROTO_UDP)) < 0)) {
 			pr_fail("%s: socket failed, errno=%d (%s)\n",
 				args->name, errno, strerror(errno));
 			goto err;
 		}
 
-		if (setsockopt(fd,  IPPROTO_IP, IP_HDRINCL, &one, sizeof(one)) < 0) {
+		if (UNLIKELY(setsockopt(fd,  IPPROTO_IP, IP_HDRINCL, &one, sizeof(one)) < 0)) {
 			pr_fail("%s: setsocketopt failed, errno=%d (%s)\n",
 				args->name, errno, strerror(errno));
 			(void)close(fd);
@@ -148,21 +134,21 @@ static void NORETURN stress_rawudp_client(
 
 		ip->tos = stress_mwc8() & 0x1e;
 		ip->id = htons(id++);
-		ip->check = stress_ipv4_checksum((uint16_t *)buf, sizeof(struct iphdr) + sizeof(struct udphdr));
+		ip->check = stress_ipv4_checksum((uint16_t *)buf, PACKET_SIZE);
 
-		n = sendto(fd, buf, ip->tot_len, 0, (struct sockaddr *)&s_in, sizeof(s_in));
-		if (n < 0) {
+		*(pid_t *)data = args->pid;
+
+		n = sendto(fd, buf, PACKET_SIZE, 0, (struct sockaddr *)&s_in, sizeof(s_in));
+		if (UNLIKELY(n < 0)) {
 			pr_fail("%s: raw socket sendto failed on port %d, errno=%d (%s)\n",
 				args->name, port, errno, strerror(errno));
 		}
 		(void)close(fd);
-	} while (keep_stressing(args));
+	} while (stress_continue(args));
 
 	rc = EXIT_SUCCESS;
 
 err:
-	/* Inform parent we're all done */
-	(void)kill(ppid, SIGALRM);
 	_exit(rc);
 }
 
@@ -170,8 +156,8 @@ err:
  *  stress_rawudp_server()
  *	server reader
  */
-static int stress_rawudp_server(
-	const stress_args_t *args,
+static int OPTIMIZE3 stress_rawudp_server(
+	stress_args_t *args,
 	in_addr_t addr,
 	const int port)
 {
@@ -182,7 +168,9 @@ static int stress_rawudp_server(
 	char buf[PACKET_SIZE];
 	const struct iphdr *ip = (struct iphdr *)buf;
 	const struct udphdr *udp = (struct udphdr *)(buf + sizeof(struct iphdr));
+	const uint8_t *data = (uint8_t *)(buf + sizeof(struct iphdr) + sizeof(struct udphdr));
 	double t_start, duration = 0.0, bytes = 0.0, rate;
+	char msg[64];
 
 	if (stress_sig_stop_stressing(args->name, SIGALRM) < 0) {
 		rc = EXIT_FAILURE;
@@ -212,19 +200,33 @@ static int stress_rawudp_server(
 		ssize_t n;
 
 		n = recv(fd, buf, sizeof(buf), 0);
-		if (n > 0) {
-			if ((ip->saddr == addr) &&
+		if (LIKELY(n > 0)) {
+			if (((in_addr_t)ip->saddr == addr) &&
 			    (ip->protocol == SOL_UDP) &&
 			    (ntohs(udp->source) == port)) {
+				if (UNLIKELY(*(const pid_t *)data != args->pid)) {
+					pr_fail("%s: data check failure, "
+						"got 0x%" PRIxMAX ", "
+						 "expected 0x%" PRIxMAX "\n",
+						args->name,
+						(intmax_t)*(const pid_t *)data,
+						(intmax_t)args->pid);
+					rc = EXIT_FAILURE;
+				}
 				bytes += (double)n;
-				inc_counter(args);
+				stress_bogo_inc(args);
 			}
 		}
-	} while (keep_stressing(args));
+	} while (stress_continue(args));
 
 	duration = stress_time_now() - t_start;
 	rate = (duration > 0.0) ? bytes / duration : 0.0;
-	stress_misc_stats_set(args->misc_stats, 0, "MB recv'd per sec", rate / (double)MB);
+	stress_metrics_set(args, 0, "MB recv'd per sec",
+		rate / (double)MB, STRESS_METRIC_HARMONIC_MEAN);
+	rate = (duration > 0.0) ? (double)stress_bogo_get(args) / duration : 0.0;
+	(void)snprintf(msg, sizeof(msg), "packets (%zu bytes) received per sec", PACKET_SIZE);
+	stress_metrics_set(args, 1, msg,
+		rate, STRESS_METRIC_HARMONIC_MEAN);
 
 die_close:
 	(void)close(fd);
@@ -236,21 +238,24 @@ static void stress_sock_sigpipe_handler(int signum)
 {
 	(void)signum;
 
-	keep_stressing_set_flag(false);
+	stress_continue_set_flag(false);
 }
 
 /*
  *  stress_rawudp
  *	stress raw socket I/O UDP packet send/receive
  */
-static int stress_rawudp(const stress_args_t *args)
+static int stress_rawudp(stress_args_t *args)
 {
 	pid_t pid;
 	int rawudp_port = DEFAULT_RAWUDP_PORT;
-	int rc = EXIT_FAILURE, reserved_port;
+	int rc = EXIT_FAILURE, reserved_port, parent_cpu;
 	in_addr_t addr = (in_addr_t)inet_addr("127.0.0.1");
 	char *rawudp_if = NULL;
 
+	if (stress_sigchld_set_handler(args) < 0)
+		return EXIT_NO_RESOURCE;
+
 	(void)stress_get_setting("rawudp-if", &rawudp_if);
 	(void)stress_get_setting("rawudp-port", &rawudp_port);
 
@@ -269,6 +274,8 @@ static int stress_rawudp(const stress_args_t *args)
 	}
 
 	rawudp_port += args->instance;
+	if (rawudp_port > MAX_PORT)
+		rawudp_port -= (MAX_PORT - MIN_PORT + 1);
 	reserved_port = stress_net_reserve_ports(rawudp_port, rawudp_port);
 	if (reserved_port < 0) {
 		pr_inf_skip("%s: cannot reserve port %d, skipping stressor\n",
@@ -283,13 +290,16 @@ static int stress_rawudp(const stress_args_t *args)
 	if (stress_sighandler(args->name, SIGPIPE, stress_sock_sigpipe_handler, NULL) < 0)
 		return EXIT_NO_RESOURCE;
 
+	stress_set_proc_state(args->name, STRESS_STATE_SYNC_WAIT);
+	stress_sync_start_wait(args);
 	stress_set_proc_state(args->name, STRESS_STATE_RUN);
 again:
+	parent_cpu = stress_get_cpu();
 	pid = fork();
 	if (pid < 0) {
-		if (stress_redo_fork(errno))
+		if (stress_redo_fork(args, errno))
 			goto again;
-		if (!keep_stressing(args)) {
+		if (UNLIKELY(!stress_continue(args))) {
 			rc = EXIT_SUCCESS;
 			goto finish;
 		}
@@ -297,13 +307,12 @@ again:
 			args->name, errno, strerror(errno));
 		return rc;
 	} else if (pid == 0) {
-		stress_rawudp_client(args, args->pid, addr, rawudp_port);
+		stress_set_proc_state(args->name, STRESS_STATE_RUN);
+		(void)stress_change_cpu(args, parent_cpu);
+		stress_rawudp_client(args, addr, rawudp_port);
 	} else {
-		int status;
-
 		rc = stress_rawudp_server(args, addr, rawudp_port);
-		(void)kill(pid, SIGKILL);
-		(void)shim_waitpid(pid, &status, 0);
+		(void)stress_kill_pid_wait(pid, NULL);
 	}
 finish:
 	stress_set_proc_state(args->name, STRESS_STATE_DEINIT);
@@ -312,19 +321,21 @@ finish:
 	return rc;
 }
 
-stressor_info_t stress_rawudp_info = {
+const stressor_info_t stress_rawudp_info = {
 	.stressor = stress_rawudp,
-	.class = CLASS_NETWORK | CLASS_OS,
-	.opt_set_funcs = opt_set_funcs,
+	.classifier = CLASS_NETWORK | CLASS_OS,
+	.opts = opts,
 	.supported = stress_rawudp_supported,
+	.verify = VERIFY_ALWAYS,
 	.help = help
 };
 #else
-stressor_info_t stress_rawudp_info = {
+const stressor_info_t stress_rawudp_info = {
 	.stressor = stress_unimplemented,
-	.class = CLASS_NETWORK | CLASS_OS,
-	.opt_set_funcs = opt_set_funcs,
+	.classifier = CLASS_NETWORK | CLASS_OS,
+	.opts = opts,
 	.supported = stress_rawudp_supported,
+	.verify = VERIFY_ALWAYS,
 	.help = help,
 	.unimplemented_reason = "built without linux/udp.h or only supported on Linux"
 };
