@@ -19,6 +19,7 @@
  */
 #include "stress-ng.h"
 #include "core-builtin.h"
+#include "core-capabilities.h"
 #include "core-killpid.h"
 #include "core-out-of-memory.h"
 #include "core-resources.h"
@@ -65,17 +66,18 @@ static int stress_resources(stress_args_t *args)
 	UNEXPECTED
 #endif
 
-	s_pids = stress_s_pids_mmap(num_pids);
+	s_pids = stress_sync_s_pids_mmap(num_pids);
 	if (s_pids == MAP_FAILED) {
-		pr_inf_skip("%s: failed to mmap %zu PIDs, skipping stressor\n", args->name, num_pids);
+		pr_inf_skip("%s: failed to mmap %zu PIDs%s, skipping stressor\n",
+			args->name, num_pids, stress_get_memfree_str());
 		return EXIT_NO_RESOURCE;
 	}
 
 	resources = (stress_resources_t *)malloc(num_resources * sizeof(*resources));
 	if (!resources) {
-		pr_inf_skip("%s: cannot allocate %zd resource structures, skipping stressor\n",
-			args->name, num_resources);
-		(void)stress_s_pids_munmap(s_pids, num_pids);
+		pr_inf_skip("%s: cannot allocate %zu resource structures%s, skipping stressor\n",
+			args->name, num_resources, stress_get_memfree_str());
+		(void)stress_sync_s_pids_munmap(s_pids, num_pids);
 		return EXIT_NO_RESOURCE;
 	}
 
@@ -86,7 +88,7 @@ static int stress_resources(stress_args_t *args)
 	do {
 		unsigned int i;
 
-		(void)shim_memset(s_pids, 0, sizeof(*s_pids));
+		(void)shim_memset(s_pids, 0, sizeof(*s_pids) * num_pids);
 		for (i = 0; i < num_pids; i++) {
 			pid_t pid;
 
@@ -97,6 +99,7 @@ static int stress_resources(stress_args_t *args)
 			if (pid == 0) {
 				size_t n;
 
+				stress_set_proc_state(args->name, STRESS_STATE_RUN);
 				stress_set_oom_adjustment(args, true);
 				VOID_RET(int, stress_drop_capabilities(args->name));
 				(void)sched_settings_apply(true);
@@ -118,7 +121,7 @@ static int stress_resources(stress_args_t *args)
 
 	stress_set_proc_state(args->name, STRESS_STATE_DEINIT);
 	free(resources);
-	(void)stress_s_pids_munmap(s_pids, num_pids);
+	(void)stress_sync_s_pids_munmap(s_pids, num_pids);
 
 	return EXIT_SUCCESS;
 }
@@ -131,7 +134,7 @@ static const stress_opt_t opts[] = {
 
 const stressor_info_t stress_resources_info = {
 	.stressor = stress_resources,
-	.class = CLASS_MEMORY | CLASS_OS,
+	.classifier = CLASS_MEMORY | CLASS_OS,
 	.opts = opts,
 	.help = help
 };
