@@ -79,7 +79,7 @@ static void stress_sem_init(const uint32_t instances)
 		return;
 	}
 	stress_set_vma_anon_name(sem_global, sizeof(*sem_global), "shared-semaphore");
-	if (sem_init(sem_global, 0, (unsigned int)instances) < 0) {
+	if (sem_init(sem_global, 1, 1) < 0) {
 		sem_global_errno = errno;
 		(void)munmap((void *)sem_global, sizeof(*sem_global));
 		sem_global = NULL;
@@ -284,6 +284,7 @@ static int stress_sem(stress_args_t *args)
 			continue;
 
 		VOID_RET(int, pthread_cancel(sem_pthreads[i].pthread));
+		VOID_RET(int, pthread_join(sem_pthreads[i].pthread, NULL));
 
 		trywait_count += sem_pthreads[i].trywait_count;
 		timedwait_count += sem_pthreads[i].timedwait_count;
@@ -302,7 +303,7 @@ static int stress_sem(stress_args_t *args)
 	if (!sem_shared)
 		(void)sem_destroy(&sem_local);
 
-	/* ready the bogo flag duo to cancelling a pthread while updating a bogo-op */
+	/* ready the bogo flag due to cancelling a pthread while updating a bogo-op */
 	stress_bogo_ready(args);
 
 	return EXIT_SUCCESS;
