@@ -61,7 +61,7 @@ static unsigned int stress_splice_flag(void)
 
 /*
  *  stress_splice_pipe_size()
- *	attempt to se pipe size of multiples of the splice buffer length
+ *	attempt to set pipe size of multiples of the splice buffer length
  */
 static int stress_splice_pipe_size(const int fd)
 {
@@ -94,7 +94,7 @@ static inline int stress_splice_write(
 		ret = write(fd, buffer, n);
 		if (UNLIKELY(ret < 0))
 			break;
-		size -= n;
+		size -= ret;
 
 	}
 	return (int)ret;
