@@ -20,6 +20,7 @@
 #include "stress-ng.h"
 #include "core-builtin.h"
 #include "core-killpid.h"
+#include "core-mmap.h"
 #include "core-out-of-memory.h"
 #include "core-put.h"
 
@@ -67,7 +68,7 @@ static void stress_stackmmap_push_msync(stress_stack_check_t *prev_check)
 	stress_stack_check_t check;
 	register stress_stack_check_t *ptr;
 	register int i;
-	static char name[] = "stackmmap";
+	static const char name[] = "stackmmap";
 
 	check.prev = prev_check;
 	check.self_addr = &check;
@@ -155,9 +156,10 @@ static int stress_stackmmap(stress_args_t *args)
 	stack_sig = (uint8_t *)stress_mmap_populate(NULL, STRESS_SIGSTKSZ,
 		PROT_READ | PROT_WRITE, MAP_SHARED | MAP_ANONYMOUS, -1, 0);
 	if (stack_sig == MAP_FAILED) {
-		pr_inf_skip("%s: skipping stressor, cannot mmap signal stack, "
-			"errno=%d (%s)\n",
-			args->name, errno, strerror(errno));
+		pr_inf_skip("%s: failed to mmap %zu byte signal stressor%s, "
+			"errno=%d (%s), skipping stressor\n",
+			args->name, (size_t)STRESS_SIGSTKSZ,
+			stress_get_memfree_str(), errno, strerror(errno));
 		rc = EXIT_NO_RESOURCE;
 		(void)close(fd);
 		goto tidy_dir;
@@ -168,19 +170,22 @@ static int stress_stackmmap(stress_args_t *args)
 		PROT_READ | PROT_WRITE, MAP_SHARED, fd, 0);
 	if (stack_mmap == MAP_FAILED) {
 		if (errno == ENXIO) {
-			pr_inf_skip("%s: skipping stressor, mmap not possible on file %s\n",
-				args->name, filename);
+			pr_inf_skip("%s: mmap failed of %zu bytes on file %s%s, errno=%d (%s),"
+				"skipping stressor\n",
+				args->name, (size_t)MMAPSTACK_SIZE, filename,
+				stress_get_memfree_str(), errno, strerror(errno));
 			rc = EXIT_NO_RESOURCE;
 			(void)close(fd);
 			goto tidy_stack_sig;
 		}
-		pr_fail("%s: mmap failed, errno=%d (%s)\n",
-			args->name, errno, strerror(errno));
+		pr_fail("%s: mmap failed of %zu bytes failed%s, errno=%d (%s)\n",
+			args->name, (size_t)MMAPSTACK_SIZE,
+			stress_get_memfree_str(), errno, strerror(errno));
 		(void)close(fd);
 		goto tidy_stack_sig;
 	}
 	(void)close(fd);
-	stress_set_vma_anon_name(stack_mmap, STRESS_SIGSTKSZ, "altstack-file");
+	stress_set_vma_anon_name(stack_mmap, MMAPSTACK_SIZE, "altstack-file");
 
 	if (shim_madvise((void *)stack_mmap, MMAPSTACK_SIZE, MADV_RANDOM) < 0) {
 		pr_dbg("%s: madvise failed, errno=%d (%s)\n",
@@ -254,6 +259,7 @@ again:
 		} else {
 			/* Child */
 
+			stress_set_proc_state(args->name, STRESS_STATE_RUN);
 			stress_parent_died_alarm();
 			(void)sched_settings_apply(true);
 
@@ -305,14 +311,14 @@ tidy_dir:
 
 const stressor_info_t stress_stackmmap_info = {
 	.stressor = stress_stackmmap,
-	.class = CLASS_VM | CLASS_MEMORY,
+	.classifier = CLASS_VM | CLASS_MEMORY,
 	.verify = VERIFY_ALWAYS,
 	.help = help
 };
 #else
 const stressor_info_t stress_stackmmap_info = {
 	.stressor = stress_unimplemented,
-	.class = CLASS_VM | CLASS_MEMORY,
+	.classifier = CLASS_VM | CLASS_MEMORY,
 	.verify = VERIFY_ALWAYS,
 	.help = help,
 	.unimplemented_reason = "built without ucontext.h or swapcontext()"
