@@ -22,6 +22,7 @@
 #include "core-builtin.h"
 #include "core-pragma.h"
 #include "core-target-clones.h"
+#include "math.h"
 
 #if defined(HAVE_SYS_TREE_H)
 #include <sys/tree.h>
@@ -31,8 +32,6 @@
 #include <bsd/sys/tree.h>
 #endif
 
-#if defined(HAVE_LIB_BSD) &&	\
-    !defined(__APPLE__)
 /* BSD red-black tree */
 #if defined(RB_HEAD) &&		\
     defined(RB_PROTOTYPE) &&	\
@@ -61,7 +60,6 @@
     defined(SPLAY_REMOVE)
 #define HAVE_SPLAY_TREE
 #endif
-#endif
 
 #define MIN_TREE_SIZE		(1000)
 #define MAX_TREE_SIZE		(25000000)	/* Must be uint32_t sized or less */
@@ -97,8 +95,10 @@ static const stress_help_t help[] = {
 	{ NULL,	NULL,		 NULL }
 };
 
+#if defined(HAVE_SIGLONGJMP)
 static volatile bool do_jmp = true;
 static sigjmp_buf jmp_env;
+#endif
 
 struct binary_node {
 	struct tree_node *left;
@@ -160,6 +160,7 @@ struct tree_node {
 
 STRESS_PRAGMA_POP
 
+#if defined(HAVE_SIGLONGJMP)
 /*
  *  stress_tree_handler()
  *	SIGALRM generic handler
@@ -171,8 +172,10 @@ static void MLOCKED_TEXT stress_tree_handler(int signum)
 	if (do_jmp) {
 		do_jmp = false;
 		siglongjmp(jmp_env, 1);		/* Ugly, bounce back */
+		stress_no_return();
 	}
 }
+#endif
 
 #if defined(HAVE_RB_TREE) ||	\
     defined(HAVE_SPLAY_TREE)
@@ -388,7 +391,7 @@ static void OPTIMIZE3 stress_tree_binary(
 {
 	size_t i;
 	struct tree_node *node, *head = NULL;
-	struct tree_node *find;
+	const struct tree_node *find;
 	double t;
 
 	t = stress_time_now();
@@ -602,7 +605,7 @@ static void OPTIMIZE3 stress_tree_avl(
 {
 	size_t i;
 	struct tree_node *node, *head = NULL;
-	struct tree_node *find;
+	const struct tree_node *find;
 	double t;
 
 	t = stress_time_now();
@@ -676,15 +679,18 @@ static btree_node_t * OPTIMIZE3 btree_split_node(
 	uint32_t *new_value,
 	const int pos,
 	btree_node_t *node,
-	btree_node_t *child)
+	btree_node_t *child,
+	bool *alloc_fail)
 {
 	btree_node_t *new_node;
 	register int j;
 	const int median = (pos > BTREE_MIN) ? BTREE_MIN + 1 : BTREE_MIN;
 
 	new_node = (btree_node_t *)calloc(1, sizeof(*new_node));
-	if (UNLIKELY(!new_node))
+	if (UNLIKELY(!new_node)) {
+		*alloc_fail = true;
 		return NULL;
+	}
 
 	j = median + 1;
 	while (j <= BTREE_MAX) {
@@ -739,7 +745,7 @@ static btree_node_t * OPTIMIZE3 btree_insert_value(
 		if (node->count < BTREE_MAX) {
 			btree_insert_node(*new_value, pos, node, child);
 		} else {
-			child = btree_split_node(*new_value, new_value, pos, node, child);
+			child = btree_split_node(*new_value, new_value, pos, node, child, alloc_fail);
 			*make_new_node = true;
 			return child;
 		}
@@ -761,7 +767,7 @@ static bool OPTIMIZE3 btree_insert(btree_node_t **root, const uint32_t value)
 
 		node = (btree_node_t *)calloc(1, sizeof(*node));
 		if (UNLIKELY(!node))
-			return false;
+			return true;
 		node->count = 1;
 		node->value[1] = new_value;
 		node->node[0] = *root;
@@ -779,12 +785,8 @@ static void OPTIMIZE3 btree_remove_tree(btree_node_t **node)
 	if (!*node)
 		return;
 
-PRAGMA_UNROLL_N(4)
-	for (i = 0; i <= (*node)->count; i++) {
+	for (i = 0; i <= (*node)->count; i++)
 		btree_remove_tree(&(*node)->node[i]);
-		free((*node)->node[i]);
-		(*node)->node[i] = NULL;
-	}
 	free(*node);
 	*node = NULL;
 }
@@ -835,7 +837,12 @@ static void stress_tree_btree(
 	t = stress_time_now();
 PRAGMA_UNROLL_N(4)
 	for (node = nodes, i = 0; i < n; i++, node++)
-		btree_insert(&root, node->value);
+		if (UNLIKELY(btree_insert(&root, node->value) == true)) {
+			pr_fail("%s: btree node #%zu allocation failure\n",
+				args->name, i);
+			btree_remove_tree(&root);
+			return;
+		}
 	metrics->insert += stress_time_now() - t;
 
 	/* Mandatory forward tree check */
@@ -844,7 +851,7 @@ PRAGMA_UNROLL_N(4)
 	for (node = nodes, i = 0; i < n; i++, node++) {
 		find = btree_find(root, node->value);
 		if (UNLIKELY(!find)) {
-			pr_fail("%s: btree node #%zd not found\n",
+			pr_fail("%s: btree node #%zu not found\n",
 				args->name, i);
 			*rc = EXIT_FAILURE;
 		}
@@ -961,10 +968,15 @@ static int stress_tree(stress_args_t *args)
 	uint64_t tree_size = DEFAULT_TREE_SIZE;
 	struct tree_node *nodes;
 	size_t n, i, j, tree_method = 0;
-	struct sigaction old_action;
-	int ret, rc = EXIT_SUCCESS;
+	int rc = EXIT_SUCCESS;
 	stress_tree_func func;
 	stress_tree_metrics_t *metrics;
+	double mantissa;
+	uint64_t exponent;
+#if defined(HAVE_SIGLONGJMP)
+	struct sigaction old_action;
+	int ret;
+#endif
 
 	stress_catch_sigill();
 
@@ -994,6 +1006,7 @@ static int stress_tree(stress_args_t *args)
 		return EXIT_NO_RESOURCE;
 	}
 
+#if defined(HAVE_SIGLONGJMP)
 	ret = sigsetjmp(jmp_env, 1);
 	if (ret) {
 		/*
@@ -1006,7 +1019,7 @@ static int stress_tree(stress_args_t *args)
 		free(nodes);
 		return EXIT_FAILURE;
 	}
-
+#endif
 
 	for (i = 0; i < n; i++)
 		nodes[i].value = (uint32_t)i;
@@ -1023,10 +1036,15 @@ static int stress_tree(stress_args_t *args)
 		stress_bogo_inc(args);
 	} while ((rc == EXIT_SUCCESS) && stress_continue(args));
 
+#if defined(HAVE_SIGLONGJMP)
 	do_jmp = false;
 	(void)stress_sigrestore(args->name, SIGALRM, &old_action);
 
 tidy:
+#endif
+	mantissa = 1.0;
+	exponent = 0;
+
 	for (i = 0, j = 0; i < SIZEOF_ARRAY(stress_tree_metrics); i++) {
 		double duration = stress_tree_metrics[i].insert +
 				  stress_tree_metrics[i].find +
@@ -1035,13 +1053,28 @@ tidy:
 		if ((duration > 0.0) && (stress_tree_metrics[i].count > 0.0)) {
 			double rate = stress_tree_metrics[i].count / duration;
 			char msg[64];
+			double f;
+			int e;
 
 			(void)snprintf(msg, sizeof(msg), "%s tree operations per sec", stress_tree_methods[i].name);
 			stress_metrics_set(args, j, msg,
 				rate, STRESS_METRIC_HARMONIC_MEAN);
+
+			f = frexp(rate, &e);
+			mantissa *= f;
+			exponent += e;
 			j++;
 		}
 	}
+
+	if (j > 0) {
+		double geomean, inverse_n = 1.0 / j;
+
+		geomean = pow(mantissa, inverse_n) *
+			  pow(2.0, (double)exponent * inverse_n);
+		pr_dbg("%s: %.2f tree ops per second (geometric mean of per stressor tree op rates)\n",
+			args->name, geomean);
+	}
 	stress_set_proc_state(args->name, STRESS_STATE_DEINIT);
 	free(nodes);
 
@@ -1050,7 +1083,7 @@ tidy:
 
 const stressor_info_t stress_tree_info = {
 	.stressor = stress_tree,
-	.class = CLASS_CPU_CACHE | CLASS_CPU | CLASS_MEMORY | CLASS_SEARCH,
+	.classifier = CLASS_CPU_CACHE | CLASS_CPU | CLASS_MEMORY | CLASS_SEARCH,
 	.opts = opts,
 	.verify = VERIFY_OPTIONAL,
 	.help = help
