@@ -272,6 +272,8 @@ static size_t TARGET_CLONES OPTIMIZE3 stress_vm_addr_inc(
 	for (ptr = buf; LIKELY(ptr < ptr_end); ptr++) {
 		*ptr = rnd;
 	}
+	if (g_opt_flags & OPT_FLAGS_AGGRESSIVE)
+		stress_cpu_data_cache_flush((void *)buf, sz);
 	for (ptr = buf; LIKELY(ptr < ptr_end); ptr++) {
 		if (UNLIKELY(*ptr != rnd))
 			errs++;
@@ -280,7 +282,7 @@ static size_t TARGET_CLONES OPTIMIZE3 stress_vm_addr_inc(
 }
 
 /*
- *  stress_vm_addr_inc()
+ *  stress_vm_addr_incinv()
  *	set data on inverted incrementing addresses
  */
 static size_t TARGET_CLONES OPTIMIZE3 stress_vm_addr_incinv(
@@ -320,13 +322,13 @@ static size_t TARGET_CLONES OPTIMIZE3 stress_vm_addr_dec(
 	uint8_t *ptr;
 
 PRAGMA_UNROLL_N(4)
-	for (ptr = (uint8_t *)buf + sz - 1; ptr != buf; ptr--) {
+	for (ptr = (uint8_t *)buf + sz - 1; ptr >= buf; ptr--) {
 		*ptr = rnd;
 	}
 	if (g_opt_flags & OPT_FLAGS_AGGRESSIVE)
 		stress_cpu_data_cache_flush((void *)buf, sz);
 PRAGMA_UNROLL_N(4)
-	for (ptr = (uint8_t *)buf + sz - 1; ptr != buf; ptr--) {
+	for (ptr = (uint8_t *)buf + sz - 1; ptr >= buf; ptr--) {
 		if (UNLIKELY(*ptr != rnd))
 			errs++;
 	}
@@ -335,7 +337,7 @@ PRAGMA_UNROLL_N(4)
 }
 
 /*
- *  stress_vm_addr_dec()
+ *  stress_vm_addr_decinv()
  *	set data on inverted decrementing addresses
  */
 static size_t TARGET_CLONES OPTIMIZE3 stress_vm_addr_decinv(
@@ -351,6 +353,8 @@ PRAGMA_UNROLL_N(4)
 
 		*(buf + i) = rnd;
 	}
+	if (g_opt_flags & OPT_FLAGS_AGGRESSIVE)
+		stress_cpu_data_cache_flush((void *)buf, sz);
 PRAGMA_UNROLL_N(4)
 	for (n = sz; n; n--) {
 		const size_t i = ((n - 1) ^ mask) & mask;
@@ -544,7 +548,6 @@ static int stress_vm_addr(stress_args_t *args)
 	(void)stress_get_setting("vm-addr-numa", &context.vm_addr_numa);
 
 	context.vm_addr_method = &vm_addr_methods[vm_addr_method];
-	context.vm_addr_mlock = false;
 	context.bit_error_count = MAP_FAILED;
 #if defined(HAVE_LINUX_MEMPOLICY_H)
 	context.numa_mask = NULL;
@@ -583,7 +586,7 @@ static int stress_vm_addr(stress_args_t *args)
 						&context.vm_addr_numa);
 #else
 		if (stress_instance_zero(args))
-			pr_inf("%s: --vm-addr-uma selected but not supported by this system, disabling option\n",
+			pr_inf("%s: --vm-addr-numa selected but not supported by this system, disabling option\n",
 				args->name);
 		context.vm_addr_numa = false;
 #endif
