@@ -80,23 +80,25 @@ typedef void (*stress_vnni_func_t)(stress_args_t *args);
 typedef bool (*stress_vnni_capable_func_t)(void);
 
 typedef struct {
-	char 				*name;			/* method name */
+	const char 			*name;			/* method name */
 	const stress_vnni_func_t	 vnni_func;		/* method function */
 	const stress_vnni_capable_func_t vnni_capable_func;	/* capability check */
 	const uint32_t			 vnni_checksum_le;	/* little endian */
 	const uint32_t			 vnni_checksum_be;	/* big endian */
 	const bool			 vnni_intrinsic;	/* uses intrinsics */
-	bool				 vnni_capable;		/* is capable */
-	double				 count;			/* usage count */
-	double				 duration;		/* usage duration */
 } stress_vnni_method_t;
 
+typedef struct {
+	stress_metrics_t	metrics;		/* usage metrics */
+	bool			vnni_capable;		/* cable of VNNI operations */
+} stress_vnni_data_t;
+
 static bool vnni_checksum_okay;
 
 static uint32_t OPTIMIZE3 stress_vnni_checksum(void)
 {
 	uint32_t sum = 0;
-	uint8_t *ptr = result;
+	const uint8_t *ptr = result;
 	const uint8_t *end = result + VEC_SIZE_BYTES;
 
 	while (ptr < end) {
@@ -367,8 +369,8 @@ PRAGMA_UNROLL_N(VEC_VNNI128_LOOPS)
 static void TARGET_CLONES OPTIMIZE3 stress_vnni_vpdpwssd(stress_args_t *args)
 {
 	register int i, j;
-	int16_t *a16 = (int16_t *)a_init;
-	int16_t *b16 = (int16_t *)b_init;
+	const int16_t *a16 = (int16_t *)a_init;
+	const int16_t *b16 = (int16_t *)b_init;
 	int32_t *r32 = (int32_t *)result;
 	register const int32_t *c32 = (int32_t *)c_init;
 
@@ -425,44 +427,47 @@ static bool stress_always_capable(void)
 
 static void stress_vnni_all(stress_args_t *args);
 
-static stress_vnni_method_t stress_vnni_methods[] = {
-	{ "all",	 stress_vnni_all,	  stress_always_capable,      0xffffffff, 0xffffffff, false, false, 0.0, 0.0 },
+static const stress_vnni_method_t stress_vnni_methods[] = {
+	{ "all",	 stress_vnni_all,	  stress_always_capable,      0xffffffff, 0xffffffff, false },
 #if defined(HAVE_STRESS_VNNI_VPADDB512)
-	{ "vpaddb512",	 stress_vnni_vpaddb512,   stress_avx512_bw_capable,   0xd93496ff, 0xd93496ff, true,  false, 0.0, 0.0 },
+	{ "vpaddb512",	 stress_vnni_vpaddb512,   stress_avx512_bw_capable,   0xd93496ff, 0xd93496ff, true  },
 #endif
 #if defined(HAVE_STRESS_VNNI_VPADDB256)
-	{ "vpaddb256",	 stress_vnni_vpaddb256,   stress_avx_vnni_capable,    0xd93496ff, 0xd93496ff, true,  false, 0.0, 0.0 },
+	{ "vpaddb256",	 stress_vnni_vpaddb256,   stress_avx_vnni_capable,    0xd93496ff, 0xd93496ff, true  },
 #endif
 #if defined(HAVE_STRESS_VNNI_VPADDB128)
-	{ "vpaddb128",	 stress_vnni_vpaddb128,   stress_avx_vnni_capable,    0xd93496ff, 0xd93496ff, true,  false, 0.0, 0.0 },
+	{ "vpaddb128",	 stress_vnni_vpaddb128,   stress_avx_vnni_capable,    0xd93496ff, 0xd93496ff, true  },
 #endif
-	{ "vpaddb",	 stress_vnni_vpaddb,      stress_always_capable,      0xd93496ff, 0xd93496ff, false, false, 0.0, 0.0 },
+	{ "vpaddb",	 stress_vnni_vpaddb,      stress_always_capable,      0xd93496ff, 0xd93496ff, false },
 #if defined(HAVE_STRESS_VNNI_VPDPBUSD512)
-	{ "vpdpbusd512", stress_vnni_vpdpbusd512, stress_avx512_vnni_capable, 0xc10ef48a, 0x1b509895, true,  false, 0.0, 0.0 },
+	{ "vpdpbusd512", stress_vnni_vpdpbusd512, stress_avx512_vnni_capable, 0xc10ef48a, 0x1b509895, true  },
 #endif
 #if defined(HAVE_STRESS_VNNI_VPDPBUSD256)
-	{ "vpdpbusd256", stress_vnni_vpdpbusd256, stress_avx_vnni_capable,    0xc10ef48a, 0x1b509895, true,  false, 0.0, 0.0 },
+	{ "vpdpbusd256", stress_vnni_vpdpbusd256, stress_avx_vnni_capable,    0xc10ef48a, 0x1b509895, true  },
 #endif
 #if defined(HAVE_STRESS_VNNI_VPDPBUSD128)
-	{ "vpdpbusd128", stress_vnni_vpdpbusd128, stress_avx_vnni_capable,    0xc10ef48a, 0x1b509895, true,  false, 0.0, 0.0 },
+	{ "vpdpbusd128", stress_vnni_vpdpbusd128, stress_avx_vnni_capable,    0xc10ef48a, 0x1b509895, true  },
 #endif
-	{ "vpdpbusd",	 stress_vnni_vpdpbusd,    stress_always_capable,      0xc10ef48a, 0x1b509895, false, false, 0.0, 0.0 },
+	{ "vpdpbusd",	 stress_vnni_vpdpbusd,    stress_always_capable,      0xc10ef48a, 0x1b509895, false },
 #if defined(HAVE_STRESS_VNNI_VPDPWSSD512)
-	{ "vpdpwssd512", stress_vnni_vpdpwssd512, stress_avx512_vnni_capable, 0x8e323fb8, 0xeef5d2a3, true,  false, 0.0, 0.0 },
+	{ "vpdpwssd512", stress_vnni_vpdpwssd512, stress_avx512_vnni_capable, 0x8e323fb8, 0xeef5d2a3, true  },
 #endif
 #if defined(HAVE_STRESS_VNNI_VPDPWSSD256)
-	{ "vpdpwssd256", stress_vnni_vpdpwssd256, stress_avx_vnni_capable,    0x8e323fb8, 0xeef5d2a3, true,  false, 0.0, 0.0 },
+	{ "vpdpwssd256", stress_vnni_vpdpwssd256, stress_avx_vnni_capable,    0x8e323fb8, 0xeef5d2a3, true  },
 #endif
 #if defined(HAVE_STRESS_VNNI_VPDPWSSD128)
-	{ "vpdpwssd128", stress_vnni_vpdpwssd128, stress_avx_vnni_capable,    0x8e323fb8, 0xeef5d2a3, true,  false, 0.0, 0.0 },
+	{ "vpdpwssd128", stress_vnni_vpdpwssd128, stress_avx_vnni_capable,    0x8e323fb8, 0xeef5d2a3, true  },
 #endif
-	{ "vpdpwssd",	 stress_vnni_vpdpwssd,    stress_always_capable,      0x8e323fb8, 0xeef5d2a3, false, false, 0.0, 0.0 },
+	{ "vpdpwssd",	 stress_vnni_vpdpwssd,    stress_always_capable,      0x8e323fb8, 0xeef5d2a3, false },
 };
 
+static stress_vnni_data_t stress_vnni_data[SIZEOF_ARRAY(stress_vnni_methods)];
+
 static void OPTIMIZE3 stress_vnni_exercise(stress_args_t *args, const size_t n)
 {
 	uint32_t checksum, expected_checksum;
-	stress_vnni_method_t *method = &stress_vnni_methods[n];
+	const stress_vnni_method_t * const method = &stress_vnni_methods[n];
+	stress_vnni_data_t *data = &stress_vnni_data[n];
 	register int j;
 	register const stress_vnni_func_t func = method->vnni_func;
 	double t;
@@ -474,8 +479,8 @@ static void OPTIMIZE3 stress_vnni_exercise(stress_args_t *args, const size_t n)
 	for (j = 0; j < 1024; j++) {
 		func(args);
 	}
-	method->duration += stress_time_now() - t;
-	method->count += (double)j;
+	data->metrics.duration += stress_time_now() - t;
+	data->metrics.count += (double)j;
 	/* and checksum the last computation */
 	checksum = stress_vnni_checksum();
 	expected_checksum = little_endian ? method->vnni_checksum_le : method->vnni_checksum_be;
@@ -491,8 +496,8 @@ static void stress_vnni_all(stress_args_t *args)
 {
 	size_t i;
 
-	for (i = 1; stress_continue(args) && (i < SIZEOF_ARRAY(stress_vnni_methods)); i++) {
-		if (LIKELY(stress_vnni_methods[i].vnni_capable)) {
+	for (i = 1; LIKELY(stress_continue(args) && (i < SIZEOF_ARRAY(stress_vnni_methods))); i++) {
+		if (LIKELY(stress_vnni_data[i].vnni_capable)) {
 			stress_vnni_exercise(args, i);
 		}
 	}
@@ -522,15 +527,15 @@ static int stress_vnni(stress_args_t *args)
 
 	avx_capable = false;
 	for (i = 0; i < SIZEOF_ARRAY(stress_vnni_methods); i++) {
-		stress_vnni_methods[i].vnni_capable = stress_vnni_methods[i].vnni_capable_func();
+		stress_vnni_data[i].vnni_capable = stress_vnni_methods[i].vnni_capable_func();
 
 		/* Keep count of capable intrinsic functions */
-		if ((stress_vnni_methods[i].vnni_capable) && (stress_vnni_methods[i].vnni_intrinsic))
+		if ((stress_vnni_data[i].vnni_capable) && (stress_vnni_methods[i].vnni_intrinsic))
 			intrinsic_count++;
 	}
 
-	if (!stress_vnni_methods[vnni_method].vnni_capable) {
-		if (args->instance == 0) {
+	if (!stress_vnni_data[vnni_method].vnni_capable) {
+		if (stress_instance_zero(args)) {
 			pr_inf_skip("%s: vnni method '%s' not available for this processor model, "
 				"skipping stressor\n",
 				args->name, stress_vnni_methods[vnni_method].name);
@@ -542,12 +547,12 @@ static int stress_vnni(stress_args_t *args)
 	    ((intrinsic_count == 0) ||
 	    ((vnni_method != 0) /* all */ && !stress_vnni_methods[vnni_method].vnni_intrinsic))) {
 		pr_inf_skip("%s: no vector neural network instructions available "
-			"and --vmmi-intrinsic selected, skipping stressor\n",
+			"and --vnni-intrinsic selected, skipping stressor\n",
 			args->name);
 		return EXIT_NO_RESOURCE;
 	}
 
-	if ((!avx_capable) && (args->instance == 0)) {
+	if ((!avx_capable) && (stress_instance_zero(args))) {
 		pr_inf("%s: no vector neural network instructions available, using generic optimized versions\n",
 			args->name);
 	}
@@ -567,11 +572,11 @@ static int stress_vnni(stress_args_t *args)
 	stress_set_proc_state(args->name, STRESS_STATE_DEINIT);
 
 	for (i = 1, j = 0; i < SIZEOF_ARRAY(stress_vnni_methods); i++) {
-		if (stress_vnni_methods[i].vnni_capable &&
-		    (stress_vnni_methods[i].count > 0.0)) {
+		if (stress_vnni_data[i].vnni_capable &&
+		    (stress_vnni_data[i].metrics.count > 0.0)) {
 			char buf[64];
-			double rate = stress_vnni_methods[i].duration > 0.0 ?
-				stress_vnni_methods[i].count / stress_vnni_methods[i].duration : 0.0;
+			double rate = stress_vnni_data[i].metrics.duration > 0.0 ?
+				stress_vnni_data[i].metrics.count / stress_vnni_data[i].metrics.duration : 0.0;
 
 			(void)snprintf(buf, sizeof(buf), "%s ops per sec", stress_vnni_methods[i].name);
 			stress_metrics_set(args, j, buf,
@@ -596,7 +601,7 @@ static const stress_opt_t opts[] = {
 
 const stressor_info_t stress_vnni_info = {
 	.stressor = stress_vnni,
-	.class = CLASS_CPU | CLASS_INTEGER | CLASS_COMPUTE | CLASS_VECTOR,
+	.classifier = CLASS_CPU | CLASS_INTEGER | CLASS_COMPUTE | CLASS_VECTOR,
 	.opts = opts,
 	.verify = VERIFY_ALWAYS,
 	.help = help
