@@ -24,6 +24,7 @@
 #include "core-builtin.h"
 #include "core-cpu.h"
 #include "core-killpid.h"
+#include "core-mmap.h"
 #include "core-target-clones.h"
 
 #include <ctype.h>
@@ -41,7 +42,8 @@ static const stress_help_t help[] = {
 	{ NULL,	NULL,			NULL }
 };
 
-#if defined(HAVE_LIB_Z)
+#if defined(HAVE_LIB_Z) &&	\
+    defined(HAVE_SIGLONGJMP)
 
 #include "zlib.h"
 
@@ -91,7 +93,7 @@ typedef struct morse {
 
 static const morse_t ALIGN64 morse[] = {
 	{ 'a', ".-" },
-	{ 'b', "-.." },
+	{ 'b', "-..." },
 	{ 'c', "-.-." },
 	{ 'd', "-.." },
 	{ 'e', "." },
@@ -192,6 +194,7 @@ static void NORETURN MLOCKED_TEXT stress_bad_read_handler(int signum)
 	(void)signum;
 
 	siglongjmp(jmpbuf, 1);
+	stress_no_return();
 }
 
 /*
@@ -1362,7 +1365,7 @@ static const stress_opt_t opts[] = {
 	{ OPT_zlib_method,       "zlib-method",       TYPE_ID_SIZE_T_METHOD, 0, 0, stress_zlib_method },
 	{ OPT_zlib_window_bits,  "zlib-window-bits",  TYPE_ID_CALLBACK, 0, 0, stress_zlib_window_bits },
 	{ OPT_zlib_stream_bytes, "zlib-stream-bytes", TYPE_ID_UINT64_BYTES_VM, 0, MAX_MEM_LIMIT, NULL },
-	{ OPT_zlib_strategy,     "zlib-stategy",      TYPE_ID_UINT32, Z_DEFAULT_STRATEGY, Z_FIXED, NULL },
+	{ OPT_zlib_strategy,     "zlib-strategy",      TYPE_ID_UINT32, Z_DEFAULT_STRATEGY, Z_FIXED, NULL },
 	END_OPT,
 };
 
@@ -1415,9 +1418,9 @@ static void stress_zlib_get_args(stress_zlib_args_t *params) {
 	}
 	if (!stress_get_setting("zlib-mem-level", &params->mem_level)) {
 		if (g_opt_flags & OPT_FLAGS_MAXIMIZE)
-			params->level = ZLIB_MAX_MEM_LEVEL;
+			params->mem_level = ZLIB_MAX_MEM_LEVEL;
 		if (g_opt_flags & OPT_FLAGS_MINIMIZE)
-			params->level = ZLIB_MIN_MEM_LEVEL;
+			params->mem_level = ZLIB_MIN_MEM_LEVEL;
 	}
 	(void)stress_get_setting("zlib-method", &params->method);
 	(void)stress_get_setting("zlib-window-bits", &params->window_bits);
@@ -1802,6 +1805,7 @@ again:
 
 		return EXIT_FAILURE;
 	} else if (pid == 0) {
+		stress_set_proc_state(args->name, STRESS_STATE_RUN);
 		(void)stress_change_cpu(args, parent_cpu);
 		stress_parent_died_alarm();
 		(void)sched_settings_apply(true);
@@ -1882,6 +1886,6 @@ const stressor_info_t stress_zlib_info = {
 	.classifier = CLASS_CPU | CLASS_CPU_CACHE | CLASS_MEMORY | CLASS_COMPUTE,
 	.verify = VERIFY_OPTIONAL,
 	.help = help,
-	.unimplemented_reason = "built without zlib library support"
+	.unimplemented_reason = "built without zlib library support or siglongjmp support"
 };
 #endif
